Package: cardargus
Title: Generate SVG Information Cards with Embedded Fonts and Badges
Version: 0.2.5
Date: 2026-07-27
Authors@R: c(
    person("Andre", "Leite", email = "leite@castlab.org", role = c("aut", "cre")),
    person("Marcos", "Wasiliew", email = "marcos.wasilew@gmail.com", role = "aut"),
    person("Hugo", "Vasconcelos", email = "hugo.vasconcelos@ufpe.br", role = "aut"),
    person("Carlos", "Amorim", email = "carlos.agaf@ufpe.br", role = "aut"),
    person("Diogo", "Bezerra", email = "diogo.bezerra@ufpe.br", role = "aut"))
Description: Create self-contained SVG information cards with embedded 'Google 
    Fonts', shields-style badges, and custom logos. Cards are fully 
    portable SVG files ideal for dashboards, reports, and web applications.
    Includes functions to export cards to PNG and PDF formats and display them in 
    'R Markdown' and 'Quarto' documents.
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-US
URL: https://strategicprojects.github.io/cardargus/,
        https://github.com/StrategicProjects/cardargus
BugReports: https://github.com/StrategicProjects/cardargus/issues
Imports: glue, cli, digest, gdtools, later
Suggests: base64enc, chromote, curl, gfonts, htmltools, knitr, magick,
        rmarkdown, rsvg, showtext, sysfonts, systemfonts, testthat (>=
        3.0.0), xml2
Config/testthat/edition: 3
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-27 12:03:13 UTC; leite
Author: Andre Leite [aut, cre],
  Marcos Wasiliew [aut],
  Hugo Vasconcelos [aut],
  Carlos Amorim [aut],
  Diogo Bezerra [aut]
Maintainer: Andre Leite <leite@castlab.org>
Repository: CRAN
Date/Publication: 2026-07-27 12:20:02 UTC
Built: R 4.5.2; ; 2026-07-27 13:29:17 UTC; unix

# Generated by roxygen2: do not edit by hand

export(batch_svg_to_base64_chrome)
export(batch_svg_to_png)
export(batch_svg_to_png_chrome)
export(card_to_grob)
export(chrome_available)
export(compress_number)
export(create_badge)
export(create_badge_row)
export(create_bottom_logo_row)
export(create_logo_row)
export(ensure_chrome)
export(find_chrome_path)
export(font_available)
export(font_cache_dir)
export(get_font_css)
export(get_svg_path)
export(icon_building)
export(icon_construction)
export(icon_house)
export(icon_map_pin)
export(icon_money)
export(include_card)
export(include_card_png)
export(install_fonts)
export(is_light_color)
export(list_bundled_svgs)
export(list_fonts)
export(load_svg_for_embed)
export(register_cardargus_knitr)
export(register_font)
export(register_google_font)
export(save_card_for_knitr)
export(save_svg)
export(setup_fonts)
export(svg_card)
export(svg_to_formats)
export(svg_to_pdf)
export(svg_to_pdf_chrome)
export(svg_to_png)
export(svg_to_png_chrome)
export(svg_to_png_with_margin)
export(svgs_dir)
importFrom(cli,cli_abort)
importFrom(cli,cli_alert_danger)
importFrom(cli,cli_alert_info)
importFrom(cli,cli_alert_success)
importFrom(cli,cli_alert_warning)
importFrom(cli,cli_bullets)
importFrom(cli,cli_h1)
importFrom(cli,cli_h2)
importFrom(cli,cli_h3)
importFrom(cli,cli_progress_step)
importFrom(cli,cli_warn)
importFrom(grDevices,as.raster)
importFrom(grDevices,col2rgb)
importFrom(stats,setNames)
importFrom(utils,download.file)
importFrom(utils,unzip)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# cardargus <img src="man/figures/logo.svg" align="right" height="139" alt="" />

<!-- badges: start -->

![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/cardargus)  
![CRAN
Downloads](https://cranlogs.r-pkg.org/badges/grand-total/cardargus)  
![License](https://img.shields.io/badge/license-MIT-darkviolet.svg) 
![](https://img.shields.io/badge/devel%20version-0.1.2-blue.svg)
[![Lifecycle:
experimental](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)
<!-- badges: end -->

**cardargus** is an R package for creating informative SVG cards with
embedded styles, Google Fonts, badges, and logos. Cards are
self-contained SVG files, ideal for dashboards, reports, and
visualizations.

## Installation

Install from CRAN:

``` r
install.packages("cardargus")
```

Or install the development version from GitHub:

``` r
# install.packages("devtools")
devtools::install_github("StrategicProjects/cardargus")
```

## Basic Example

``` r
library(cardargus)

# Create an informative card
card <- svg_card(
  font = "Jost",
  title = "FAR",
  
  badges_data = list(
    list(label = "Units",         value = "500",     color = "white"),
    list(label = "Federal Grant", value = "$100M",   color = "white"),
    list(label = "State Match",   value = "$80M",    color = "white")
  ),
  
  fields = list(
    list(
      list(label = "Project Name", value = "Boa Vista Residential",
           with_icon = icon_house())
    ),
    list(
      list(label = "Address", value = "123 Flower Street - Downtown")
    ),
    list(
      list(label = "City",   value = "Olinda"),
      list(label = "Region", value = "Pernambuco")
    ),
    list(
      list(label = "Developer",   value = "State Government"),
      list(label = "Contractor",  value = "ABC Construction"), 
      list(label = "Type", value = "PS")  # optional
    )
  ),
  
  bg_color    = "#FF9900",
  title_color = "#fff",
  label_color = "#fff",
  width = 450,
  
  # You can use bundled SVGs or any local file path
  logos = c(
    get_svg_path("seduh.svg"), 
    get_svg_path("morar_bem.svg")
  ),
  logos_height = 40,
  
  bottom_logos = c(
    get_svg_path("tree.svg"), 
    get_svg_path("gov_pe.svg")
  ),
  bottom_logos_height = 40,
  
  footer = paste0(
    "Source: SEDUH/PE on ", 
    format(Sys.time(), "%Y/%m/%d at %H:%M"))
)
```

``` r
include_card_png(card, dpi = 300, width = '50%')
```

<div class="cardargus-card" style="width:50%; max-width:100%;">

<img src="data:image/png;base64,[encoded data, 165320 characters]" alt="Card generated by cardargus" style="width:100%; height:auto;" />

</div>

``` r
# Save as SVG
save_svg(card, "my_card.svg")

# Convert to high-quality PNG
svg_to_png(card, "my_card.png", dpi = 300)
```

## Displaying Cards in R Markdown / Quarto

cardargus provides functions to display cards directly in your
documents:

``` r
# Display card as inline SVG (best quality)
include_card(card)

# Display card as PNG (better compatibility)
include_card_png(card, dpi = 150)

# Save and get path for knitr::include_graphics()
path <- save_card_for_knitr(card, "my_card", format = "png")
knitr::include_graphics(path)
```

For chunk-based workflows, register the cardargus knitr engine:

``` r
# In your setup chunk
register_cardargus_knitr()
```

Then use `cardargus` as a chunk engine:

```` markdown
```{cardargus}`
svg_card(title = "My Card", ...)
```
````

## Features

- 📦 **Self-contained SVG**: All styles and fonts embedded
- 🎨 **Customizable**: Colors, fonts, icons, and layouts
- 🌈 **Gradient Backgrounds**: CSS-style linear gradients support
- 🏷️ **Badges**: Shields.io-style with dynamic colors and uniform height
- 🖼️ **Icons**: Built-in SVG icon library
- 📄 **Export**: High-quality PNG with transparent background
- 🔤 **Google Fonts**: Native support via showtext/sysfonts
- 📊 **R Markdown/Quarto**: Direct display functions

## Gradient Backgrounds

Use CSS-style linear gradients for dynamic card backgrounds:

``` r
# Horizontal gradient
card <- svg_card(
  title = "HOUSING",
  bg_color = "linear-gradient(to right, #1a5a3a, #2e7d32)",
  ...
)

# Diagonal gradient  
card <- svg_card(
  title = "PROGRAM",
  bg_color = "linear-gradient(135deg, #667eea, #764ba2)",
  ...
)

# Vertical gradient
card <- svg_card(
  title = "PROJECT",
  bg_color = "linear-gradient(to bottom, #00c6ff, #0072ff)",
  ...
)
```

Supported directions: `to right`, `to left`, `to top`, `to bottom`, or
angles like `45deg`, `135deg`, `180deg`.

## Custom Cards

``` r
# Define badges
badges <- list(
  list(label = "Units", value = "500", color = "white"),
  list(label = "Status", value = "Active", color = "#4CAF50")
)

# Define fields with custom icon
fields <- list(
  list(
    list(label = "Project", value = "Housing Development")
  ),
  list(
    list(label = "City", value = "Pesqueira"),
    list(label = "State", value = "Pernambuco")
  )
)

# Create card with logos
card <- svg_card(
  title = "HOUSING",
  badges_data = badges,
  fields = fields,
  bg_color = "#2c3e50",
  title_color = "#ecf0f1",
  width = 200,
  logos = c(get_svg_path("morar_bem.svg")),
  logos_height = 40,
)
```

<div class="cardargus-card" style="width:25%; max-width:100%;">

<img src="data:image/png;base64,[encoded data, 58564 characters]" alt="Card generated by cardargus" style="width:100%; height:auto;" />

</div>

## Bundled SVGs

``` r
# List available SVGs
list_bundled_svgs()

# Get full path
get_svg_path("morar_bem.svg")
```

## Available Icons

``` r
# Built-in icons
icon_house()        # House (default)
icon_building()     # Building
icon_construction() # Construction
icon_map_pin()      # Location
icon_money()        # Money

# Or use your own SVG file
# with_icon = "/path/to/custom_icon.svg"
```

## Font Setup

For best font rendering:

``` r
# Setup Google Fonts (recommended)
setup_fonts()

# Or download fonts for offline use
install_fonts()

# Check font availability
font_available("Jost")
```

## Chrome Rendering (Recommended)

For **perfect font rendering** with Google Fonts, use headless Chrome:

``` r
# Check if Chrome is available
chrome_available()

# If Chrome is not installed, download it automatically (~150MB)
ensure_chrome(download = TRUE)

# Convert to PNG with Chrome (best quality)
svg_to_png_chrome(card, "my_card.png", dpi = 300)

# Convert to PDF (vector output)
svg_to_pdf_chrome(card, "my_card.pdf")

# In R Markdown / Quarto - force Chrome engine
include_card_png(card, dpi = 300, engine = "chrome")
```

Install `chromote` for Chrome support:

``` r
install.packages("chromote")
```

## Authors

- **Andre Leite** - *Maintainer* - <leite@castlab.org>
- **Hugo Vasconcelos** - <hugo.vasconcelos@ufpe.br>
- **Diogo Bezerra** - <diogo.bezerra@ufpe.br>

## License

MIT License - see [LICENSE](LICENSE) for details.

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----icons, eval=FALSE--------------------------------------------------------
# library(cardargus)
# 
# # House icon (default for fields with with_icon = TRUE)
# icon_house(width = 50, height = 56)
# 
# # Building icon
# icon_building(width = 50, height = 56)
# 
# # Construction icon
# icon_construction(width = 50, height = 56)
# 
# # Map pin icon
# icon_map_pin(width = 50, height = 56)
# 
# # Money icon
# icon_money(width = 50, height = 56)

## ----field-icons, eval=FALSE--------------------------------------------------
# fields <- list(
#   # Default house icon
#   list(
#     list(label = "Project", value = "Housing Complex", with_icon = TRUE)
#   ),
#   # Built-in building icon
#   list(
#     list(label = "Building", value = "Tower A",
#          with_icon = icon_building())
#   ),
#   # Construction icon
#   list(
#     list(label = "Status", value = "Under Construction",
#          with_icon = icon_construction())
#   ),
#   # Custom SVG file path
#   list(
#     list(label = "Custom", value = "My Item",
#          with_icon = "/path/to/my_icon.svg")
#   )
# )
# 
# card <- svg_card(
#   title = "PROJECT",
#   badges_data = list(),
#   fields = fields,
#   bg_color = "#3498db"
# )

## ----custom-icons, eval=FALSE-------------------------------------------------
# # Custom colors
# icon <- icon_house(
#   width = 60,
#   height = 70,
#   stroke_color = "#e74c3c",  # Red outline
#   stroke_width = 3,
#   fill = "#ffebee"           # Light red fill
# )
# 
# # Building with custom size
# building <- icon_building(
#   width = 40,
#   height = 48,
#   stroke_color = "#2c3e50",
#   fill = "#ecf0f1"
# )

## ----bundled, eval=FALSE------------------------------------------------------
# # List all bundled SVGs
# list_bundled_svgs()
# #> [1] "gov_pe3.svg" "morar_bem.svg" "seduh.svg" "tree.svg" "example_card.svg"
# 
# # Get full path to a logo
# path <- get_svg_path("morar_bem.svg")
# 
# # Get the SVGs directory
# svgs_dir()

## ----logos-cards, eval=FALSE--------------------------------------------------
# card <- svg_card(
#   title = "FAR",
#   badges_data = list(
#     list(label = "Units", value = "192", color = "white")
#   ),
#   fields = list(...),
#   bg_color = "#fab255",
# 
#   # Top-right logos (bundled or custom paths)
#   logos = c(
#     get_svg_path("morar_bem.svg"),
#     get_svg_path("seduh.svg"),
#     "/path/to/my_custom_logo.svg"  # Any local file
#   ),
#   logos_height = 40,
# 
#   # Bottom-left logos
#   bottom_logos = c(
#     get_svg_path("gov_pe3.svg")
#   ),
#   bottom_logos_height = 35
# )

## ----external, eval=FALSE-----------------------------------------------------
# # Load and resize an external SVG
# logo_data <- load_svg_for_embed(
#   svg_path = "path/to/my_logo.svg",
#   target_height = 50
# )
# 
# # The result contains:
# # - svg_content: The processed SVG string
# # - width: Calculated width (maintains aspect ratio)
# # - height: The target height

## ----logo-rows, eval=FALSE----------------------------------------------------
# # Top-right logo row
# logo_row <- create_logo_row(
#   logos = c("logo1.svg", "logo2.svg"),
#   target_height = 40,
#   spacing = 15,
#   card_width = 500,
#   x_offset = 20,
#   y_offset = 20
# )
# 
# # Bottom-left logo row
# bottom_row <- create_bottom_logo_row(
#   logos = c("logo3.svg"),
#   target_height = 30,
#   spacing = 10,
#   x_offset = 20,
#   card_height = 400,
#   y_offset = 20
# )

## ----custom-svg, eval=FALSE---------------------------------------------------
# # Create a simple custom logo
# my_logo <- '<svg width="50" height="50" viewBox="0 0 50 50">
#   <circle cx="25" cy="25" r="20" fill="#3498db"/>
#   <text x="25" y="30" text-anchor="middle" fill="white" font-size="16">AB</text>
# </svg>'
# 
# # Use in a card
# card <- svg_card(
#   title = "PROJECT",
#   logos = list(my_logo),
#   logos_height = 40,
#   ...
# )

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----svg, eval=FALSE----------------------------------------------------------
# library(cardargus)
# 
# card <- svg_card(
#   title = "PROJECT",
#   badges_data = list(list(label = "Status", value = "Active")),
#   fields = list(list(list(label = "Name", value = "Test"))),
#   bg_color = "#3498db"
# )
# 
# # Save to file (with embedded fonts)
# save_svg(card, "my_card.svg")

## ----png-basic, eval=FALSE----------------------------------------------------
# # Convert SVG string to PNG
# svg_to_png(card, "my_card.png")
# 
# # With custom DPI (default is 300)
# svg_to_png(card, "my_card.png", dpi = 300)

## ----png-chrome, eval=FALSE---------------------------------------------------
# # Check if Chrome is available
# if (chrome_available()) {
#   # High-quality PNG with Chrome
#   svg_to_png_chrome(card, "my_card.png", dpi = 300)
# }
# 
# # Convert to PDF (vector output)
# svg_to_pdf_chrome(card, "my_card.pdf")

## ----chrome-install, eval=FALSE-----------------------------------------------
# # Install chromote package
# install.packages("chromote")
# 
# # Check if Chrome is available
# chrome_available()
# 
# # If Chrome is not installed, download it automatically
# ensure_chrome(download = TRUE)
# 
# # This downloads "Chrome for Testing" (~150MB) to your user cache
# # No admin/root permissions required!

## ----png-bg, eval=FALSE-------------------------------------------------------
# # Transparent background (preserves rounded corners)
# svg_to_png(card, "card_transparent.png", dpi = 300)
# 
# # White background
# svg_to_png(card, "card_white.png", dpi = 300, background = "white")
# 
# # Chrome also supports backgrounds
# svg_to_png_chrome(card, "card_white.png", dpi = 300, background = "white")

## ----png-margin, eval=FALSE---------------------------------------------------
# # Add 20px margin with white background
# svg_to_png_with_margin(
#   card,
#   output_path = "card_with_margin.png",
#   margin = 20,
#   background = "white",
#   dpi = 300
# )

## ----pdf-basic, eval=FALSE----------------------------------------------------
# # Vector PDF (via rsvg)
# svg_to_pdf(card, "my_card.pdf")

## ----pdf-chrome, eval=FALSE---------------------------------------------------
# if (chrome_available()) {
#   svg_to_pdf_chrome(card, "my_card.pdf")
# }

## ----pdf-knitr, eval=FALSE----------------------------------------------------
# # Saves card.pdf using Chrome when available, else rsvg
# save_card_for_knitr(card, format = "pdf")

## ----formats, eval=FALSE------------------------------------------------------
# # Export to SVG, PNG and (vector) PDF
# svg_to_formats(
#   card,
#   output_base = "exports/my_card",  # Without extension
#   formats = c("svg", "png", "pdf"),
#   dpi = 300
# )
# # Creates: exports/my_card.svg, exports/my_card.png, exports/my_card.pdf

## ----batch, eval=FALSE--------------------------------------------------------
# # Create multiple cards
# cards <- list(
#   svg_card(title = "Card 1", badges_data = list(), fields = list()),
#   svg_card(title = "Card 2", badges_data = list(), fields = list()),
#   svg_card(title = "Card 3", badges_data = list(), fields = list())
# )
# 
# # Batch convert to PNG
# batch_svg_to_png(
#   svg_list = cards,
#   output_dir = "output/cards",
#   prefix = "card",
#   dpi = 300,
#   background = "white"
# )
# # Creates: output/cards/card_001.png, card_002.png, card_003.png

## ----inline, eval=FALSE-------------------------------------------------------
# # Display card directly in document (HTML only)
# include_card(card)
# 
# # With custom width
# include_card(card, width = "80%")

## ----png-display, eval=FALSE--------------------------------------------------
# # Display as PNG with auto engine selection
# include_card_png(card, dpi = 150)
# 
# # Force Chrome for best font rendering
# include_card_png(card, dpi = 300, engine = "chrome")
# 
# # Force rsvg (faster, no Chrome dependency)
# include_card_png(card, dpi = 300, engine = "rsvg")

## ----knitr-save, eval=FALSE---------------------------------------------------
# # Save and get path for knitr::include_graphics
# path <- save_card_for_knitr(
#   card,
#   filename = "my_card",
#   format = "png",
#   dpi = 300,
#   engine = "chrome"  # or "auto", "rsvg"
# )
# knitr::include_graphics(path)

## ----engine, eval=FALSE-------------------------------------------------------
# # Register the cardargus engine
# register_cardargus_knitr()
# 
# # Then in a chunk with engine='cardargus':
# # ```{cardargus}
# # svg_card(title = "Dynamic Card", ...)
# # ```

## ----grob, eval=FALSE---------------------------------------------------------
# library(grid)
# 
# # Convert card to grob (auto-selects best engine)
# grob <- card_to_grob(card, dpi = 150)
# 
# # Force Chrome rendering
# grob <- card_to_grob(card, dpi = 150, engine = "chrome")
# 
# # Draw in a grid viewport
# grid.newpage()
# grid.draw(grob)

## ----size, eval=FALSE---------------------------------------------------------
# # Smaller file (lower quality)
# svg_to_png_chrome(card, "card_web.png", dpi = 96)
# 
# # Medium file (balanced)
# svg_to_png_chrome(card, "card_standard.png", dpi = 150)
# 
# # Larger file (high quality)
# svg_to_png_chrome(card, "card_print.png", dpi = 300)

## ----compare, eval=FALSE------------------------------------------------------
# # Same card, different engines
# svg_to_png(card, "card_rsvg.png", dpi = 300)
# svg_to_png_chrome(card, "card_chrome.png", dpi = 300)
# 
# # Chrome generally produces better results for:
# # - Google Fonts and custom web fonts
# # - Complex CSS styling
# # - Font weights and variants

## ----errors, eval=FALSE-------------------------------------------------------
# # Safe conversion with fallback
# convert_card <- function(card, output_path, dpi = 300) {
#   if (chrome_available()) {
#     svg_to_png_chrome(card, output_path, dpi = dpi)
#   } else {
#     message("Chrome not available, using rsvg")
#     svg_to_png(card, output_path, dpi = dpi)
#   }
# }
# 
# # With explicit error handling
# tryCatch({
#   svg_to_png_chrome(card, "output.png", dpi = 300)
#   message("Conversion successful!")
# }, error = function(e) {
#   message("Chrome conversion failed: ", e$message)
#   message("Falling back to rsvg...")
#   svg_to_png(card, "output.png", dpi = 300)
# })

## ----chrome-not-found, eval=FALSE---------------------------------------------
# # Check Chrome availability
# chrome_available()
# 
# # Option 1: Download Chrome for Testing automatically
# ensure_chrome(download = TRUE)
# 
# # Option 2: Set path to existing Chrome manually
# Sys.setenv(CHROMOTE_CHROME = "/path/to/chrome")
# 
# # Option 3: Find where Chrome is installed
# find_chrome_path()

## ----batch-tip, eval=FALSE----------------------------------------------------
# # For many cards, rsvg may be faster if fonts are embedded
# svg_to_png(card, "output.png", dpi = 300)
# 
# # Or accept the Chrome overhead for perfect rendering
# svg_to_png_chrome(card, "output.png", dpi = 300)

## ----timeout, eval=FALSE------------------------------------------------------
# # Increase timeout for complex SVGs
# svg_to_png_chrome(card, "output.png", timeout = 60)  # 60 seconds

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----single-badge, eval=FALSE-------------------------------------------------
# library(cardargus)
# 
# # Basic badge
# badge <- create_badge(
#   label = "Status",
#   value = "Active",
#   color = "#4CAF50"
# )
# 
# # Save to file
# writeLines(badge, "status_badge.svg")

## ----colors, eval=FALSE-------------------------------------------------------
# # Green for success
# badge_success <- create_badge("Build", "Passing", "#4CAF50")
# 
# # Red for failure
# badge_error <- create_badge("Build", "Failing", "#e74c3c")
# 
# # Blue for information
# badge_info <- create_badge("Version", "1.0.0", "#3498db")
# 
# # Orange for warning
# badge_warning <- create_badge("Coverage", "45%", "#fab255")
# 
# # White (default)
# badge_default <- create_badge("License", "MIT", "white")

## ----auto-color, eval=FALSE---------------------------------------------------
# # Dark background -> white text
# create_badge("Test", "Dark", "#2c3e50")
# 
# # Light background -> dark text
# create_badge("Test", "Light", "#f1c40f")

## ----badge-row, eval=FALSE----------------------------------------------------
# badges <- list(
#   list(label = "Units", value = "192"),
#   list(label = "Budget", value = "$36.4M"),
#   list(label = "Status", value = "Active", color = "#4CAF50")
# )
# 
# # All badges will have uniform height
# row <- create_badge_row(badges, default_color = "white", uniform_height = TRUE)

## ----in-cards, eval=FALSE-----------------------------------------------------
# card <- svg_card(
#   title = "PROJECT",
#   badges_data = list(
#     list(label = "Units", value = "500", color = "white"),
#     list(label = "Budget", value = "$50M", color = "#4CAF50"),
#     list(label = "Phase", value = "2/3", color = "#3498db")
#   ),
#   fields = list(...),
#   bg_color = "#fab255"
# )

## ----uniform, eval=FALSE------------------------------------------------------
# # These badges will all have the same height
# badges <- list(
#   list(label = "A", value = "1"),              # Short
#   list(label = "Long Label", value = "Value"), # Long
#   list(label = "X", value = "Y")               # Short
# )
# 
# row <- create_badge_row(badges, uniform_height = TRUE)

## ----customize, eval=FALSE----------------------------------------------------
# badge <- create_badge(
#   label = "Custom",
#   value = "Badge",
#   color = "#9b59b6",
#   font = "Roboto",          # Custom font
#   fontsize = 12,            # Font size
#   style = "margin: 5px;"    # Additional CSS
# )

## ----save, eval=FALSE---------------------------------------------------------
# # Single badge
# badge <- create_badge("Version", "1.0.0", "#3498db")
# writeLines(badge, "version.svg")
# 
# # Badge row
# row <- create_badge_row(badges)
# writeLines(row, "badges.svg")

## ----rmarkdown, eval=FALSE----------------------------------------------------
# # Display inline
# badge <- create_badge("Status", "Active", "#4CAF50")
# knitr::asis_output(badge)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----bg-color, eval=FALSE-----------------------------------------------------
# library(cardargus)
# 
# # Orange card
# card_orange <- svg_card(title = "ALERT", bg_color = "#fab255", ...)
# 
# # Blue card
# card_blue <- svg_card(title = "INFO", bg_color = "#3498db", ...)
# 
# # Green card
# card_green <- svg_card(title = "SUCCESS", bg_color = "#2ecc71", ...)
# 
# # Red card
# card_red <- svg_card(title = "URGENT", bg_color = "#e74c3c", ...)
# 
# # Dark card
# card_dark <- svg_card(title = "PREMIUM", bg_color = "#2c3e50", ...)

## ----gradient, eval=FALSE-----------------------------------------------------
# # Horizontal gradient (left to right)
# card <- svg_card(
#   title = "HOUSING",
#   bg_color = "linear-gradient(to right, #1a5a3a, #2e7d32)",
#   ...
# )
# 
# # Diagonal gradient (135 degrees)
# card <- svg_card(
#   title = "PROGRAM",
#   bg_color = "linear-gradient(135deg, #667eea, #764ba2)",
#   ...
# )
# 
# # Vertical gradient (top to bottom)
# card <- svg_card(
#   title = "PROJECT",
#   bg_color = "linear-gradient(to bottom, #00c6ff, #0072ff)",
#   ...
# )
# 
# # Multi-color gradient
# card <- svg_card(
#   title = "RAINBOW",
#   bg_color = "linear-gradient(to right, #ff6b6b, #feca57, #48dbfb)",
#   ...
# )

## ----text-colors, eval=FALSE--------------------------------------------------
# card <- svg_card(
#   title = "MY CARD",
#   title_color = "#FFD700",      # Gold title
#   label_color = "#E0E0E0",      # Light gray labels
#   footer_color = "white",       # White footer
#   value_text_color = "#212529", # Dark gray values
#   value_bg_color = "#f8f8ff",   # Field background
#   ...
# )

## ----fonts, eval=FALSE--------------------------------------------------------
# # Using Roboto
# card <- svg_card(
#   title = "CARD",
#   font = "Roboto",
#   ...
# )
# 
# # Using Open Sans
# card <- svg_card(
#   title = "CARD",
#   font = "Open Sans",
#   ...
# )
# 
# # Using Montserrat
# card <- svg_card(
#   title = "CARD",
#   font = "Montserrat",
#   ...
# )

## ----font-sizes, eval=FALSE---------------------------------------------------
# card <- svg_card(
#   title = "CARD",
#   title_fontsize = 32,    # Larger title
#   label_fontsize = 12,    # Labels
#   value_fontsize = 11,    # Values
#   footer_fontsize = 8,    # Footer
#   ...
# )

## ----width, eval=FALSE--------------------------------------------------------
# # Narrow card
# card_narrow <- svg_card(width = 400, ...)
# 
# # Default card
# card_default <- svg_card(width = 500, ...)
# 
# # Wide card
# card_wide <- svg_card(width = 600, ...)

## ----padding, eval=FALSE------------------------------------------------------
# card <- svg_card(
#   padding = 25,         # Internal padding
#   corner_radius = 12,   # More rounded corners
#   ...
# )

## ----footer, eval=FALSE-------------------------------------------------------
# # With automatic timestamp
# card <- svg_card(
#   footer = NULL,  # Uses automatic timestamp
#   ...
# )
# 
# # With custom text
# card <- svg_card(
#   footer = "Source: Housing Dept. - Updated January 2026",
#   ...
# )
# 
# # Without footer
# card <- svg_card(
#   footer = "",
#   ...
# )

## ----logos, eval=FALSE--------------------------------------------------------
# # Without logos
# card <- svg_card(
#   logos = list(),
#   ...
# )
# 
# # With file paths
# card <- svg_card(
#   logos = c("path/to/logo1.svg", "path/to/logo2.svg"),
#   logos_height = 40,
#   ...
# )
# 
# # With bundled logos
# card <- svg_card(
#   logos = c(get_svg_path("morar_bem.svg")),
#   logos_height = 40,
#   bottom_logos = c(get_svg_path("gov_pe3.svg")),
#   bottom_logos_height = 35,
#   ...
# )

## ----complete, eval=FALSE-----------------------------------------------------
# card <- svg_card(
#   title = "HOUSING PROGRAM",
#   badges_data = list(
#     list(label = "Units", value = "1,000", color = "white"),
#     list(label = "Investment", value = "$100M", color = "#4CAF50")
#   ),
#   fields = list(
#     list(list(label = "Project", value = "Downtown Development", with_icon = TRUE)),
#     list(
#       list(label = "City", value = "Boston"),
#       list(label = "State", value = "MA")
#     )
#   ),
#   bg_color = "#2c3e50",
#   width = 550,
#   padding = 25,
#   corner_radius = 12,
#   font = "Roboto",
#   title_fontsize = 24,
#   title_color = "#ecf0f1",
#   label_color = "#bdc3c7",
#   footer = "Real-time updated data",
#   footer_color = "#95a5a6"
# )

## ----display, eval=FALSE------------------------------------------------------
# # In R Markdown or Quarto
# include_card(card)
# 
# # Or as PNG for better compatibility
# include_card_png(card, dpi = 150)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup, eval=FALSE--------------------------------------------------------
# library(cardargus)
# 
# # Automatically setup fonts (recommended)
# setup_fonts()

## ----check, eval=FALSE--------------------------------------------------------
# # Check if a font is available
# font_available("Jost")
# font_available("Arial")
# 
# # List cached fonts
# list_fonts()

## ----google, eval=FALSE-------------------------------------------------------
# # Register a specific Google Font
# register_google_font("Roboto")
# register_google_font("Open Sans")
# register_google_font("Lato")

## ----install, eval=FALSE------------------------------------------------------
# # Download and cache fonts
# install_fonts()
# 
# # Install specific fonts
# install_fonts(fonts = c("Jost", "Montserrat", "Roboto"))
# 
# # Check cache location
# font_cache_dir()

## ----custom, eval=FALSE-------------------------------------------------------
# # Use a different font in cards
# card <- svg_card(
#   title = "MY CARD",
#   font = "Montserrat",  # Use Montserrat instead of Jost
#   ...
# )

## ----css, eval=FALSE----------------------------------------------------------
# # Generate CSS for a font
# css <- get_font_css("Jost")
# cat(css)
# # @import url("https://fonts.googleapis.com/css2?family=Jost:wght@400;500;600;700&display=swap");
# 
# # Multiple weights
# css <- get_font_css("Montserrat", weights = c(400, 700))

## ----cache, eval=FALSE--------------------------------------------------------
# # Get cache directory
# font_cache_dir()
# #> "/home/user/.cache/cardargus"
# 
# # List cached fonts
# list_fonts()

## ----resources, eval=FALSE----------------------------------------------------
# # SVGs directory (bundled logos)
# svgs_dir()
# 
# # Path to a specific SVG
# get_svg_path("morar_bem.svg")
# 
# # List available SVGs
# list_bundled_svgs()

## ----packages, eval=FALSE-----------------------------------------------------
# install.packages(c("showtext", "sysfonts", "systemfonts"))

## ----png, eval=FALSE----------------------------------------------------------
# # Setup fonts first
# setup_fonts()
# 
# # Download fonts for embedding
# install_fonts()
# 
# # Create and export
# card <- svg_card(...)
# svg_to_png(card, "card.png", dpi = 300)

## ----chrome, eval=FALSE-------------------------------------------------------
# # Ensure Chrome is available
# ensure_chrome(download = TRUE)
# 
# # Create and export
# card <- svg_card(...)
# svg_to_png_chrome(card, "card.png", dpi = 300)

## ----server, eval=FALSE-------------------------------------------------------
# # Option 1: Download fonts locally
# install_fonts()
# 
# # Update font cache (Linux)
# system("fc-cache -fv")
# 
# # Option 2: Use Chrome rendering (recommended)
# ensure_chrome(download = TRUE)
# svg_to_png_chrome(card, "card.png")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup, eval=FALSE--------------------------------------------------------
# # Install from GitHub
# devtools::install_github("castlab/cardargus")

## ----load, eval=FALSE---------------------------------------------------------
# library(cardargus)

## ----first-card, eval=FALSE---------------------------------------------------
# library(cardargus)
# 
# card <- svg_card(
#   title = "PROJECT",
#   badges_data = list(
#     list(label = "Units", value = "192", color = "white"),
#     list(label = "Budget", value = "$5.2M", color = "white"),
#     list(label = "Status", value = "Active", color = "#4CAF50")
#   ),
#   fields = list(
#     list(
#       list(label = "Project Name", value = "Downtown Housing", with_icon = TRUE)
#     ),
#     list(
#       list(label = "Address", value = "123 Main Street, City Center")
#     ),
#     list(
#       list(label = "City", value = "New York"),
#       list(label = "State", value = "NY")
#     ),
#     list(
#       list(label = "Developer", value = "ABC Construction"),
#       list(label = "Owner", value = "City Housing Authority")
#     )
#   ),
#   bg_color = "#3498db",
#   width = 500,
#   footer = "Source: Housing Department - January 2026"
# )

## ----display, eval=FALSE------------------------------------------------------
# # Display card as SVG (best quality)
# include_card(card)
# 
# # Display card as PNG (better compatibility)
# include_card_png(card, dpi = 150)

## ----engine, eval=FALSE-------------------------------------------------------
# # In setup chunk
# register_cardargus_knitr()
# 
# # Then use cardargus chunks:
# # ```{cardargus}
# # svg_card(title = "My Card", ...)
# # ```

## ----save-svg, eval=FALSE-----------------------------------------------------
# save_svg(card, "my_card.svg")

## ----save-png, eval=FALSE-----------------------------------------------------
# # Standard conversion (uses rsvg)
# svg_to_png(card, "my_card.png", dpi = 300)
# 
# # Chrome-based conversion (best font rendering)
# svg_to_png_chrome(card, "my_card.png", dpi = 300)
# 
# # PNG with white background
# svg_to_png(card, "my_card.png", dpi = 300, background = "white")

## ----chrome, eval=FALSE-------------------------------------------------------
# # Check if Chrome is available
# chrome_available()
# 
# # PNG with Chrome (best quality)
# svg_to_png_chrome(card, "my_card.png", dpi = 300)
# 
# # Vector PDF with Chrome
# svg_to_pdf_chrome(card, "my_card.pdf")
# 
# # In R Markdown / Quarto
# include_card_png(card, dpi = 300, engine = "chrome")

## ----logos, eval=FALSE--------------------------------------------------------
# # List available logos
# list_bundled_svgs()
# 
# # Use in a card
# card <- svg_card(
#   title = "FAR",
#   badges_data = list(...),
#   fields = list(...),
#   bg_color = "#fab255",
#   logos = c(get_svg_path("morar_bem.svg"), get_svg_path("seduh.svg")),
#   logos_height = 40,
#   bottom_logos = c(get_svg_path("gov_pe3.svg")),
#   bottom_logos_height = 35
# )

## ----custom-card, eval=FALSE--------------------------------------------------
# # Define badges
# badges <- list(
#   list(label = "Units", value = "500", color = "white"),
#   list(label = "Investment", value = "$50M", color = "#4CAF50")
# )
# 
# # Define fields with custom icons
# fields <- list(
#   list(
#     list(label = "Project", value = "Housing Development",
#          with_icon = icon_building())  # Or use a file path: "/path/to/icon.svg"
#   ),
#   list(
#     list(label = "City", value = "Boston"),
#     list(label = "State", value = "MA")
#   ),
#   list(
#     list(label = "Start", value = "Jan/2024"),
#     list(label = "End", value = "Dec/2025"),
#     list(label = "Progress", value = "45%")
#   )
# )
# 
# # Create card
# card <- svg_card(
#   title = "HOUSING",
#   badges_data = badges,
#   fields = fields,
#   bg_color = "#2c3e50",
#   title_color = "#ecf0f1",
#   width = 500
# )

# zzz.R - Package load and attach hooks

#' @importFrom cli cli_alert_success cli_alert_info cli_alert_warning
#' @importFrom cli cli_alert_danger cli_progress_step
#' @importFrom cli cli_h1 cli_h2 cli_h3 cli_bullets cli_warn cli_abort
#' @importFrom grDevices col2rgb as.raster
#' @importFrom stats setNames
#' @importFrom utils download.file unzip
#' @keywords internal
"_PACKAGE"

.onLoad <- function(libname, pkgname) {
  # 1) Register knitr helpers only if knitr is installed
  if (requireNamespace("knitr", quietly = TRUE)) {
    tryCatch(
      register_cardargus_knitr(),
      error = function(e) {
        # Silently ignore to avoid breaking package load
      }
    )
  }
  
  # 2) Check for required system fonts
  if (requireNamespace("systemfonts", quietly = TRUE)) {
    tryCatch({
      fonts <- systemfonts::system_fonts()
      jost_available <- "Jost" %in% fonts$family
      
      if (!jost_available) {
        # Try to register Jost from Google Fonts if sysfonts is available
        if (requireNamespace("sysfonts", quietly = TRUE)) {
          tryCatch({
            sysfonts::font_add_google("Jost", "Jost")
          }, error = function(e) {
            # Silently ignore - user will be notified at attach
          })
        }
      }
    }, error = function(e) {
      # Silently ignore font check errors
    })
  }
}

# xml_utils.R - Robust SVG parsing helpers.
#
# Reading geometry and fonts out of arbitrary SVG strings with regular
# expressions is fragile: attribute order, single vs double quotes, units
# (`px`, `%`), whitespace and multiline tags all break naive patterns. These
# helpers parse with `xml2` when it is available and the document is
# well-formed, and fall back to the original regex behaviour otherwise (e.g.
# `xml2` not installed, or editor SVGs with undeclared namespace prefixes).

#' Extract the leading number from a dimension string
#'
#' @param x A character such as `"500"`, `"40px"`, `"100%"`, or `NA`.
#' @return Numeric value, or `NA_real_` if none is found.
#' @keywords internal
num_lead <- function(x) {
  if (length(x) != 1 || is.na(x) || !nzchar(x)) return(NA_real_)
  suppressWarnings(as.numeric(sub(".*?([0-9.]+).*", "\\1", x)))
}

#' Parse a viewBox string into width/height
#'
#' @param vb A viewBox string `"minx miny width height"` or `NA`.
#' @return Numeric vector `c(width, height)`, possibly `c(NA, NA)`.
#' @keywords internal
parse_viewbox <- function(vb) {
  if (length(vb) != 1 || is.na(vb) || !nzchar(vb)) return(c(NA_real_, NA_real_))
  parts <- suppressWarnings(as.numeric(strsplit(trimws(vb), "[ ,]+")[[1]]))
  if (length(parts) == 4 && all(!is.na(parts))) return(parts[3:4])
  c(NA_real_, NA_real_)
}

#' Parse an SVG string into an xml2 document
#'
#' @param svg_content SVG string (or object coercible to character).
#' @return An `xml_document`, or `NULL` if `xml2` is unavailable or parsing
#'   fails (including editor SVGs with undeclared namespace prefixes).
#' @keywords internal
svg_xml_parse <- function(svg_content) {
  if (!requireNamespace("xml2", quietly = TRUE)) return(NULL)
  svg <- as.character(svg_content)
  if (length(svg) != 1 || is.na(svg) || !nzchar(svg)) return(NULL)
  # Editor SVGs (Inkscape) may use undeclared namespace prefixes
  # (sodipodi:, inkscape:), which strict parsing rejects. Retry in libxml2
  # RECOVER mode and silence its warnings; on any failure, return NULL so the
  # caller falls back to regex parsing.
  suppressWarnings(tryCatch(
    xml2::read_xml(svg),
    error = function(e) tryCatch(
      xml2::read_xml(svg, options = c("RECOVER", "NOERROR", "NOWARNING", "NOBLANKS")),
      error = function(e2) NULL
    )
  ))
}

#' Read SVG root geometry (attribute and viewBox dimensions)
#'
#' @param svg_content SVG string.
#' @return A list with numeric `attr_w`, `attr_h`, `vb_w`, `vb_h` (any may be NA).
#' @keywords internal
svg_geometry <- function(svg_content) {
  doc <- svg_xml_parse(svg_content)
  if (!is.null(doc)) {
    root <- xml2::xml_root(doc)
    vbwh <- parse_viewbox(xml2::xml_attr(root, "viewBox"))
    return(list(
      attr_w = num_lead(xml2::xml_attr(root, "width")),
      attr_h = num_lead(xml2::xml_attr(root, "height")),
      vb_w = vbwh[1],
      vb_h = vbwh[2]
    ))
  }
  svg_geometry_regex(svg_content)
}

#' Regex fallback for [svg_geometry()]
#' @param svg_content SVG string.
#' @keywords internal
svg_geometry_regex <- function(svg_content) {
  svg <- as.character(svg_content)

  attr_num <- function(attr) {
    m <- regexpr(paste0(attr, '\\s*=\\s*"[^"]*"'), svg, perl = TRUE)
    if (m[1] == -1) return(NA_real_)
    num_lead(regmatches(svg, m))
  }

  vb <- regmatches(svg, regexec('viewBox\\s*=\\s*"([^"]+)"', svg))[[1]]
  vbwh <- if (length(vb) > 1) parse_viewbox(vb[2]) else c(NA_real_, NA_real_)

  list(attr_w = attr_num("width"), attr_h = attr_num("height"),
       vb_w = vbwh[1], vb_h = vbwh[2])
}

#' Read a single root SVG dimension robustly
#'
#' @param svg_content SVG string.
#' @param attr `"width"` or `"height"`.
#' @param prefer Which source to try first: `"attr"` (width/height attribute,
#'   the default) or `"viewBox"`.
#' @return Numeric value in CSS px, or `NA_real_` if not found.
#' @keywords internal
svg_dimension <- function(svg_content, attr = c("width", "height"),
                          prefer = c("attr", "viewBox")) {
  attr <- match.arg(attr)
  prefer <- match.arg(prefer)
  g <- svg_geometry(svg_content)

  from_attr <- if (attr == "width") g$attr_w else g$attr_h
  from_vb   <- if (attr == "width") g$vb_w else g$vb_h

  order <- if (prefer == "attr") list(from_attr, from_vb) else list(from_vb, from_attr)
  for (val in order) {
    if (length(val) == 1 && !is.na(val)) return(val)
  }
  NA_real_
}

#' Read SVG width and height robustly
#'
#' @param svg_content SVG string.
#' @param prefer Which source to try first per dimension (see [svg_dimension()]).
#' @return A list with numeric `width` and `height` (either may be NA).
#' @keywords internal
svg_dims <- function(svg_content, prefer = c("attr", "viewBox")) {
  prefer <- match.arg(prefer)
  list(
    width  = svg_dimension(svg_content, "width", prefer),
    height = svg_dimension(svg_content, "height", prefer)
  )
}

#' Detect font families referenced by an SVG
#'
#' @param svg_content SVG string.
#' @return Unique character vector of font family names (generics removed).
#' @keywords internal
svg_font_families <- function(svg_content) {
  doc <- svg_xml_parse(svg_content)
  if (is.null(doc)) return(svg_font_families_regex(svg_content))

  families <- character()

  # Presentation attribute: font-family="Jost" (any element, any namespace)
  attr_nodes <- xml2::xml_find_all(doc, "//*[@font-family]")
  if (length(attr_nodes)) {
    families <- c(families, xml2::xml_attr(attr_nodes, "font-family"))
  }

  # CSS inside <style> elements: font-family: "Jost", sans-serif;
  style_nodes <- xml2::xml_find_all(doc, "//*[local-name()='style']")
  if (length(style_nodes)) {
    css <- paste(xml2::xml_text(style_nodes), collapse = "\n")
    css_matches <- regmatches(
      css, gregexpr("font-family\\s*:\\s*([^;}{]+)", css, perl = TRUE)
    )[[1]]
    for (m in css_matches) {
      families <- c(families, sub("^font-family\\s*:\\s*", "", m))
    }
  }

  normalize_font_families(families)
}

#' Regex fallback for [svg_font_families()]
#' @param svg_content SVG string.
#' @keywords internal
svg_font_families_regex <- function(svg_content) {
  svg <- as.character(svg_content)
  families <- character()

  css_matches <- regmatches(
    svg, gregexpr("font-family\\s*:\\s*([^;}{]+)", svg, perl = TRUE)
  )[[1]]
  for (m in css_matches) {
    families <- c(families, sub("^font-family\\s*:\\s*", "", m))
  }

  attr_matches <- regmatches(
    svg, gregexpr('font-family\\s*=\\s*"([^"]+)"', svg, perl = TRUE)
  )[[1]]
  for (m in attr_matches) {
    families <- c(families, sub('"$', "", sub('^font-family\\s*=\\s*"', "", m)))
  }

  normalize_font_families(families)
}

#' Normalize a vector of raw font-family values
#'
#' Takes the first family of each declaration, strips quotes/whitespace, dedups,
#' and drops CSS generic family names.
#'
#' @param families Character vector of raw font-family values.
#' @keywords internal
normalize_font_families <- function(families) {
  if (!length(families)) return(character())
  families <- vapply(families, function(v) {
    first <- strsplit(v, ",", fixed = TRUE)[[1]][1]
    gsub("[\"']", "", trimws(first))
  }, character(1), USE.NAMES = FALSE)

  families <- unique(families[nzchar(families)])
  generics <- c("sans-serif", "serif", "monospace", "cursive", "fantasy", "system-ui")
  families[!(tolower(families) %in% generics)]
}

# Robustness tests for the xml2-backed SVG parsing helpers (issue #13).

test_that("svg_dimension reads width/height in CSS px", {
  svg <- '<svg xmlns="http://www.w3.org/2000/svg" width="500" height="320"></svg>'
  expect_equal(cardargus:::svg_dimension(svg, "width"), 500)
  expect_equal(cardargus:::svg_dimension(svg, "height"), 320)
})

test_that("svg_dimension handles units and single quotes", {
  svg <- "<svg xmlns='http://www.w3.org/2000/svg' width='40px' height='100%'></svg>"
  expect_equal(cardargus:::svg_dimension(svg, "width"), 40)
  expect_equal(cardargus:::svg_dimension(svg, "height"), 100)
})

test_that("svg_dimension is robust to attribute order and multiline tags", {
  svg <- '<svg\n  xmlns="http://www.w3.org/2000/svg"\n  viewBox="0 0 640 480"\n  height="240"\n  width="320">\n</svg>'
  expect_equal(cardargus:::svg_dimension(svg, "width", prefer = "attr"), 320)
  expect_equal(cardargus:::svg_dimension(svg, "width", prefer = "viewBox"), 640)
  expect_equal(cardargus:::svg_dimension(svg, "height", prefer = "viewBox"), 480)
})

test_that("svg_dimension falls back to viewBox (incl. comma separators)", {
  svg <- '<svg xmlns="http://www.w3.org/2000/svg" viewBox="0,0,200,100"></svg>'
  expect_equal(cardargus:::svg_dimension(svg, "width"), 200)
  expect_equal(cardargus:::svg_dimension(svg, "height"), 100)
})

test_that("svg_dims returns both dimensions", {
  svg <- '<svg xmlns="http://www.w3.org/2000/svg" width="80" height="40"></svg>'
  d <- cardargus:::svg_dims(svg)
  expect_equal(d$width, 80)
  expect_equal(d$height, 40)
})

test_that("regex fallback matches xml2 path for dimensions", {
  svg <- '<svg width="123" height="45" viewBox="0 0 246 90"></svg>'
  expect_equal(cardargus:::svg_geometry_regex(svg)$attr_w, 123)
  expect_equal(cardargus:::svg_geometry_regex(svg)$attr_h, 45)
  expect_equal(cardargus:::svg_geometry_regex(svg)$vb_w, 246)
  expect_equal(cardargus:::svg_geometry_regex(svg)$vb_h, 90)
})

test_that("svg_font_families finds attribute and CSS families, drops generics", {
  svg <- paste0(
    '<svg xmlns="http://www.w3.org/2000/svg">',
    '<style>.t{font-family:"Jost", sans-serif;}</style>',
    '<text font-family="Montserrat">hi</text>',
    '<text font-family=\'Open Sans, serif\'>yo</text>',
    '</svg>'
  )
  fams <- cardargus:::svg_font_families(svg)
  expect_true(all(c("Jost", "Montserrat", "Open Sans") %in% fams))
  expect_false(any(c("sans-serif", "serif") %in% fams))
})

test_that("svg_font_families_regex agrees with the xml2 path", {
  svg <- '<svg><text font-family="Jost">x</text></svg>'
  expect_equal(cardargus:::svg_font_families_regex(svg), "Jost")
})

test_that("helpers tolerate non-parseable / empty input via fallback", {
  expect_equal(cardargus:::svg_dimension("not xml at all", "width"), NA_real_)
  expect_equal(cardargus:::svg_font_families(""), character())
})

# utils.R - Utility functions for SVG cards

# Session-local mutable state (e.g. deterministic id counter for generate_id()).
.cardargus_state <- new.env(parent = emptyenv())
.cardargus_state$id_counter <- 0L

#' Check if a color is light
#'
#' @param color A hex color string or color name
#' @return Logical indicating if the color is light
#' @export
#' @examples
#' is_light_color("#FFFFFF")
#' is_light_color("#000000")
is_light_color <- function(color) {
  if (is.null(color) || length(color) != 1 || is.na(color) || !nzchar(color)) {
    cli::cli_abort("{.arg color} must be a single non-empty color string.")
  }
  # Convert color to RGB
  if (grepl("^#", color)) {
    hex <- gsub("^#", "", color)
    if (nchar(hex) == 3) {
      hex <- paste0(substr(hex, 1, 1), substr(hex, 1, 1),
                    substr(hex, 2, 2), substr(hex, 2, 2),
                    substr(hex, 3, 3), substr(hex, 3, 3))
    }
    r <- strtoi(substr(hex, 1, 2), 16)
    g <- strtoi(substr(hex, 3, 4), 16)
    b <- strtoi(substr(hex, 5, 6), 16)
    if (anyNA(c(r, g, b))) {
      cli::cli_abort("{.arg color} is not a valid hex color: {.val {color}}.")
    }
  } else {
    # Try to get RGB from color name
    rgb_vals <- col2rgb(color)
    r <- rgb_vals[1]
    g <- rgb_vals[2]
    b <- rgb_vals[3]
  }
  
  # Calculate luminance
  luminance <- (0.299 * r + 0.587 * g + 0.114 * b) / 255
  return(luminance > 0.5)
}

#' Compress number to abbreviated format
#'
#' @param x Numeric value to compress
#' @param digits Number of decimal places
#' @return Character string with abbreviated number
#' @export
#' @examples
#' compress_number(1234567)
#' compress_number(36400000)
compress_number <- function(x, digits = 1) {
  if (is.null(x) || length(x) != 1) return("S/I")
  if (is.na(x)) return("S/I")
  if (x == 0) return("0,0")
  
  abs_x <- abs(x)
  
  if (abs_x >= 1e9) {
    value <- round(x / 1e9, digits)
    suffix <- " bilh\u00F5es"
  } else if (abs_x >= 1e6) {
    value <- round(x / 1e6, digits)
    suffix <- " milh\u00F5es"
  } else if (abs_x >= 1e3) {
    value <- round(x / 1e3, digits)
    suffix <- " mil"
  } else {
    value <- round(x, digits)
    suffix <- ""
  }
  
  # Format with Brazilian locale (comma as decimal separator)
  formatted <- format(value, nsmall = digits, decimal.mark = ",", big.mark = ".")
  paste0(trimws(formatted), suffix)
}

#' Get Google Font CSS for embedding in SVG
#'
#' @param font_name Name of the Google Font
#' @param weights Vector of font weights to include
#' @return Character string with CSS @font-face rules
#' @export
#' @examples
#' get_font_css("Jost")
get_font_css <- function(font_name = "Jost", weights = c(400, 500, 600, 700)) {
  # Generate @font-face rules for Google Fonts
  css <- sprintf(
    '@import url("https://fonts.googleapis.com/css2?family=%s:wght@%s&display=swap");',
    gsub(" ", "+", font_name),
    paste(weights, collapse = ";")
  )
  return(css)
}

#' Generate unique ID suffix
#'
#' @description
#' Produces a short, collision-resistant suffix derived from the supplied
#' content plus a session-local counter. Using a counter (instead of
#' [stats::runif()]) keeps output deterministic and reproducible within a
#' session while remaining unique across calls.
#'
#' @param ... Values to hash
#' @return Character string with unique ID
#' @keywords internal
generate_id <- function(...) {
  .cardargus_state$id_counter <- .cardargus_state$id_counter + 1L
  digest::digest(paste(..., .cardargus_state$id_counter, sep = "_"), algo = "crc32")
}

#' Escape XML special characters
#'
#' @param text Text to escape
#' @return Escaped text
#' @keywords internal
escape_xml <- function(text) {
  if (is.null(text) || length(text) == 0) return("")
  text <- as.character(text)
  text[is.na(text)] <- ""
  text <- gsub("&", "&amp;", text)
  text <- gsub("<", "&lt;", text)
  text <- gsub(">", "&gt;", text)
  text <- gsub("'", "&apos;", text)
  text <- gsub('"', "&quot;", text)
  return(text)
}

#' Calculate text width using gdtools
#'
#' @param text Text to measure
#' @param font Font name
#' @param fontsize Font size in pixels
#' @return Numeric width in pixels
#' @keywords internal
text_width <- function(text, font = "Jost", fontsize = 12) {
  if (is.null(text) || length(text) != 1 || is.na(text) || text == "") return(0)
  metrics <- gdtools::str_metrics(text, fontname = font, fontsize = fontsize)
  return(as.numeric(metrics["width"]))
}

#' Calculate text height using gdtools
#'
#' @param text Text to measure
#' @param font Font name
#' @param fontsize Font size in pixels
#' @return Numeric height in pixels
#' @keywords internal
text_height <- function(text, font = "Jost", fontsize = 12) {
  if (is.null(text) || length(text) != 1 || is.na(text) || text == "") return(fontsize)
  metrics <- gdtools::str_metrics(text, fontname = font, fontsize = fontsize)
  return(as.numeric(metrics["ascent"]) + as.numeric(metrics["descent"]))
}

#' Wrap text to fit within a given width
#'
#' @param text Text to wrap
#' @param max_width Maximum width in pixels
#' @param font Font name
#' @param fontsize Font size
#' @return List with wrapped lines and total height
#' @keywords internal
wrap_text <- function(text, max_width, font = "Jost", fontsize = 12) {
  if (is.null(text) || length(text) != 1 || is.na(text) || text == "") {
    return(list(lines = "", height = fontsize * 1.2, line_height = fontsize * 1.2))
  }
  
  words <- strsplit(text, " ")[[1]]
  lines <- character()
  current_line <- ""
  
  for (word in words) {
    test_line <- if (current_line == "") word else paste(current_line, word)
    if (text_width(test_line, font, fontsize) <= max_width) {
      current_line <- test_line
    } else {
      if (current_line != "") {
        lines <- c(lines, current_line)
      }
      current_line <- word
    }
  }
  
  if (current_line != "") {
    lines <- c(lines, current_line)
  }
  
  line_height <- fontsize * 1.3
  total_height <- length(lines) * line_height
  
  list(lines = lines, height = total_height, line_height = line_height)
}

test_that("compress_number formats with Brazilian locale and suffixes", {
  expect_equal(compress_number(0), "0,0")
  expect_equal(compress_number(999), "999,0")
  expect_match(compress_number(1234567), "milh")
  expect_match(compress_number(36400000), "milh")
  expect_match(compress_number(2.5e9), "bilh")
  expect_match(compress_number(1500), "mil")
})

test_that("compress_number handles NULL / NA / non-scalar safely", {
  expect_equal(compress_number(NA), "S/I")
  expect_equal(compress_number(NULL), "S/I")
  # Non-scalar input must not error (regression: issue #2)
  expect_equal(compress_number(c(1500, 2e6)), "S/I")
})

test_that("is_light_color classifies hex and named colors", {
  expect_true(is_light_color("#FFFFFF"))
  expect_false(is_light_color("#000000"))
  expect_true(is_light_color("#FFF"))   # 3-digit shorthand
  expect_true(is_light_color("white"))
  expect_false(is_light_color("black"))
})

test_that("is_light_color rejects invalid input (issue #5)", {
  expect_error(is_light_color(NA))
  expect_error(is_light_color(NULL))
  expect_error(is_light_color("#GGGGGG"))
  expect_error(is_light_color(c("#fff", "#000")))
})

test_that("escape_xml escapes special characters and tolerates NA/NULL (issue #2)", {
  expect_equal(escape_xml("a & b < c > d"), "a &amp; b &lt; c &gt; d")
  expect_equal(escape_xml("\"x\" 'y'"), "&quot;x&quot; &apos;y&apos;")
  expect_equal(escape_xml(NULL), "")
  expect_equal(escape_xml(NA), "")
  # Vectorized: must not error on length > 1
  expect_equal(escape_xml(c("a&b", NA)), c("a&amp;b", ""))
})

test_that("generate_id is deterministic within a session and unique per call (issue #12)", {
  st <- cardargus:::.cardargus_state
  st$id_counter <- 0L
  a1 <- cardargus:::generate_id("x", "y")
  a2 <- cardargus:::generate_id("x", "y")
  expect_false(identical(a1, a2))  # unique across calls

  st$id_counter <- 0L
  b1 <- cardargus:::generate_id("x", "y")
  expect_identical(a1, b1)         # reproducible from same counter state
})

#' Get the path to package SVGs directory
#'
#' Returns the path to the inst/svgs directory where SVG files are stored.
#'
#' @return Character string with the path to SVGs directory
#' @export
svgs_dir <- function() {
  system.file("svgs", package = "cardargus")
}

#' Get path to a bundled SVG file
#'
#' Returns the full path to a SVG file bundled with the package.
#'
#' @param filename Name of the SVG file (e.g., "morar_bem.svg")
#' @param height Optional target height (px). If provided, returns the SVG content resized
#'   for embedding instead of the file path.
#' @param width Optional target width (px). Only used when returning resized SVG content.
#'
#' @return If `height` and `width` are both NULL, returns the full file path.
#'   Otherwise returns the resized SVG content (character string).
#' @export
get_svg_path <- function(filename, height = NULL, width = NULL) {
  path <- system.file("svgs", filename, package = "cardargus")
  if (path == "") {
    available <- list.files(svgs_dir(), pattern = "\\.svg$", full.names = FALSE)
    cli::cli_abort(c(
      "x" =  glue::glue("SVG file {.val {filename}} not found.", filename = filename),
      "i" = glue::glue("Available files: {.val {available}}", available = available)
    ))
  }
  
  if (is.null(height) && is.null(width)) {
    return(path)
  }
  
  if (!exists("load_svg_for_embed", mode = "function")) {
    cli::cli_abort("Internal helper {.fn load_svg_for_embed} was not found.")
  }
  
  if (is.null(height)) height <- 40
  load_svg_for_embed(svg_path = path, target_height = height, target_width = width)$svg_content
}

#' List available bundled SVG files
#'
#' @return Character vector of available SVG filenames
#' @export
list_bundled_svgs <- function() {
  dir <- svgs_dir()
  if (dir == "") return(character(0))
  list.files(dir, pattern = "\\.svg$", full.names = FALSE)
}

# ------------------------------------------------------------------------------
# Optional: register fonts for interactive devices (showtext/sysfonts)
# ------------------------------------------------------------------------------

#' Register Google Font (sysfonts)
#'
#' Registers a Google Font using \pkg{sysfonts}.
#'
#' @param font_family Font family name (e.g., "Jost")
#' @return Invisible NULL
#' @export
register_google_font <- function(font_family) {
  if (!requireNamespace("sysfonts", quietly = TRUE)) {
    cli::cli_abort(c(
      "x" = "Package {.pkg sysfonts} is required.",
      "i" = "Install with: {.code install.packages('sysfonts')}"
    ))
  }
  tryCatch(
    sysfonts::font_add_google(name = font_family, family = font_family),
    error = function(e) cli::cli_warn("Could not register font {.val {font_family}}: {e$message}")
  )
  invisible(NULL)
}

#' Setup showtext for cardargus
#'
#' Registers fonts (via sysfonts) and optionally enables showtext auto mode.
#'
#' @param fonts Character vector of Google Font names to register.
#' @param auto Enable showtext auto mode.
#' @return Invisible NULL
#' @export
setup_fonts <- function(fonts = c("Jost", "Montserrat"), auto = TRUE) {
  if (!requireNamespace("showtext", quietly = TRUE)) {
    cli::cli_abort(c(
      "x" = "Package {.pkg showtext} is required.",
      "i" = "Install with: {.code install.packages('showtext')}"
    ))
  }
  if (!requireNamespace("sysfonts", quietly = TRUE)) {
    cli::cli_abort(c(
      "x" = "Package {.pkg sysfonts} is required.",
      "i" = "Install with: {.code install.packages('sysfonts')}"
    ))
  }
  
  for (fam in fonts) {
    tryCatch(
      sysfonts::font_add_google(name = fam, family = fam),
      error = function(e) cli::cli_warn("Could not register font {.val {fam}}: {e$message}")
    )
  }
  
  if (isTRUE(auto)) showtext::showtext_auto()
  invisible(NULL)
}

# ------------------------------------------------------------------------------
# Internal: cache + download WOFF2 from Google Fonts
# ------------------------------------------------------------------------------

#' Get font cache directory
#'
#' Returns the directory where cardargus caches downloaded font files.
#' Fonts in this directory are automatically embedded in SVG/PNG exports.
#'
#' @param persistent Logical. If TRUE (default), uses persistent cache via
#'   \code{tools::R_user_dir()}. If FALSE or if persistent cache is unavailable,
#'   uses a session-specific temporary directory.
#' @return A character path to the cache directory.
#' @export
font_cache_dir <- function(persistent = TRUE) {
  if (persistent) {
    dir <- tryCatch(
      tools::R_user_dir("cardargus", which = "cache"),
      error = function(e) NULL
    )
  } else {
    dir <- NULL
  }
  
  if (is.null(dir)) {
    dir <- file.path(tempdir(), "cardargus-fonts")
  }
  
  if (!dir.exists(dir)) dir.create(dir, recursive = TRUE, showWarnings = FALSE)
  dir
}

#' Build a cache filename for a font family
#' @param family Font family name.
#' @return Path to the cached font file.
#' @keywords internal
font_cache_path <- function(family) {
  key <- gsub("[^A-Za-z0-9._-]+", "_", family)
  file.path(font_cache_dir(), paste0(key, ".woff2"))
}

#' Download a Google Font as WOFF2 to the cache
#'
#' @param family Font family name.
#' @param weight Numeric/character weight (default "400").
#' @return Path to cached WOFF2 file or NA_character_.
#' @keywords internal
download_google_font_woff2 <- function(family, weight = "400") {
  out <- font_cache_path(family)
  if (file.exists(out)) return(out)

  key <- gsub("[^A-Za-z0-9._-]+", "_", family)

  # Try gfonts package if available
  if (requireNamespace("gfonts", quietly = TRUE)) {
    res <- tryCatch({
      tmp <- tempfile("cardargus_font_")
      dir.create(tmp)

      gfonts::download_font(
        family = family,
        output_dir = tmp,
        variants = "regular"
      )

      candidates <- list.files(tmp, pattern = "\\.(woff2|ttf|woff|otf)$",
                               full.names = TRUE, recursive = TRUE)

      if (!length(candidates)) stop("No font files downloaded.")

      # Prefer a real woff2; otherwise keep the actual extension so that
      # embed_svg_fonts() declares the correct @font-face format().
      woff2 <- candidates[grepl("\\.woff2$", candidates, ignore.case = TRUE)]
      src <- if (length(woff2)) woff2[1] else candidates[1]
      ext <- tolower(tools::file_ext(src))
      dest <- file.path(font_cache_dir(), paste0(key, ".", ext))

      file.copy(src, dest, overwrite = TRUE)
      dest
    }, error = function(e) NULL)

    if (!is.null(res) && file.exists(res)) return(res)
  }
  
  # Fallback: fetch CSS from Google Fonts API
  family_encoded <- gsub(" ", "+", family)
  
  # URLs to try (different API formats)
  urls_to_try <- c(
    sprintf("https://fonts.googleapis.com/css2?family=%s:wght@400&display=swap", family_encoded),
    sprintf("https://fonts.googleapis.com/css2?family=%s&display=swap", family_encoded),
    sprintf("https://fonts.googleapis.com/css?family=%s:400&display=swap", family_encoded),
    sprintf("https://fonts.googleapis.com/css?family=%s&display=swap", family_encoded)
  )
  
  # User-Agent that triggers woff2 response
  ua <- "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/120.0.0.0 Safari/537.36"
  
  for (css_url in urls_to_try) {
    css <- NULL
    
    # Try with curl package first (more reliable for headers)
    if (requireNamespace("curl", quietly = TRUE)) {
      css <- tryCatch({
        h <- curl::new_handle()
        curl::handle_setheaders(h, "User-Agent" = ua)
        curl::handle_setopt(h, connecttimeout = 15L, timeout = 60L)
        req <- curl::curl_fetch_memory(css_url, handle = h)
        if (req$status_code == 200) rawToChar(req$content) else NULL
      }, error = function(e) NULL)
    }
    
    # Fallback to base R (may not get woff2 on all systems)
    if (is.null(css) || !nzchar(css)) {
      css <- tryCatch({
        paste(readLines(css_url, warn = FALSE), collapse = "\n")
      }, error = function(e) NULL)
    }
    
    if (is.null(css) || !nzchar(css)) next
    
    # Extract woff2 URL from CSS
    # Pattern matches: url(https://...woff2) or url("https://...woff2")
    m <- regexec('url\\(["\']?([^)"\']+\\.woff2)["\']?\\)', css)
    hit <- regmatches(css, m)[[1]]
    
    if (length(hit) >= 2) {
      font_url <- hit[2]
      
      ok <- tryCatch({
        utils::download.file(font_url, out, mode = "wb", quiet = TRUE)
        TRUE
      }, error = function(e) FALSE)
      
      if (isTRUE(ok) && file.exists(out) && file.info(out)$size > 1000) {
        return(out)
      }
    }
  }
  
  # If woff2 not found, try ttf as last resort
  for (css_url in urls_to_try) {
    css <- tryCatch({
      paste(readLines(css_url, warn = FALSE), collapse = "\n")
    }, error = function(e) NULL)
    
    if (is.null(css) || !nzchar(css)) next
    
    # Try ttf
    m <- regexec('url\\(["\']?([^)"\']+\\.ttf)["\']?\\)', css)
    hit <- regmatches(css, m)[[1]]
    
    if (length(hit) >= 2) {
      font_url <- hit[2]
      ttf_out <- sub("\\.woff2$", ".ttf", out)
      
      ok <- tryCatch({
        utils::download.file(font_url, ttf_out, mode = "wb", quiet = TRUE)
        TRUE
      }, error = function(e) FALSE)
      
      if (isTRUE(ok) && file.exists(ttf_out) && file.info(ttf_out)$size > 1000) {
        return(ttf_out)
      }
    }
  }
  
  NA_character_
}

# ------------------------------------------------------------------------------
# Internal: detect fonts used by SVG
# ------------------------------------------------------------------------------

#' Detect font families used in an SVG
#'
#' @param svg_content Character SVG.
#' @return Character vector of font family names (unique).
#' @keywords internal
detect_svg_fonts <- function(svg_content) {
  # Parsing is delegated to xml2 (with a regex fallback); see R/xml_utils.R.
  svg_font_families(svg_content)
}

# ------------------------------------------------------------------------------
# Internal: ensure fonts exist (family -> woff2 path)
# ------------------------------------------------------------------------------

#' Ensure cardargus fonts are available (WOFF2 cache)
#'
#' @param families Character vector of font families.
#' @param weight Weight to request when downloading (default "400").
#' @return Named character vector family -> font file path (NA if unavailable).
#' @keywords internal
ensure_cardargus_fonts <- function(families, weight = "400") {
  families <- unique(as.character(families))
  out <- setNames(rep(NA_character_, length(families)), families)
  
  cache_dir <- font_cache_dir()
  
  for (fam in families) {
    key <- gsub("[^A-Za-z0-9._-]+", "_", fam)
    
    for (ext in c("woff2", "ttf", "woff", "otf")) {
      fp <- file.path(cache_dir, paste0(key, ".", ext))
      if (file.exists(fp)) {
        out[[fam]] <- fp
        break
      }
    }
    
    if (is.na(out[[fam]])) {
      fp <- download_google_font_woff2(fam, weight = weight)
      out[[fam]] <- if (!is.na(fp) && file.exists(fp)) fp else NA_character_
    }
  }
  
  out
}

#' Check if a font is available for embedding
#'
#' @param family Font family name
#' @return TRUE if font is cached (any supported format), FALSE otherwise
#' @export
#' @examples
#' font_available("Jost")
#' font_available("Montserrat")
font_available <- function(family) {
  cache_dir <- font_cache_dir()
  key <- gsub("[^A-Za-z0-9._-]+", "_", family)
  
  for (ext in c("woff2", "ttf", "woff", "otf")) {
    if (file.exists(file.path(cache_dir, paste0(key, ".", ext)))) {
      return(TRUE)
    }
  }
  FALSE
}

#' List registered/cached fonts
#'
#' @return Character vector of font family names that are cached
#' @export
#' @examples
#' list_fonts()
list_fonts <- function() {
  cache_dir <- font_cache_dir()
  files <- list.files(cache_dir, pattern = "\\.(woff2|ttf|woff|otf)$", full.names = FALSE)
  unique(gsub("\\.(woff2|ttf|woff|otf)$", "", files))
}

#' Pre-download fonts for offline use
#'
#' Downloads and caches the specified fonts (or default fonts)
#' so they are available for PNG conversion without internet access.
#'
#' @param fonts Character vector of Google Font names to download.
#'   Default is c("Jost", "Montserrat", "Roboto", "Open Sans").
#' @param verbose Print status messages
#' @return Named logical vector indicating success for each font
#' @export
#' @examples
#' \donttest{
#' install_fonts()
#' install_fonts(c("Jost", "Roboto"))
#' }
install_fonts <- function(fonts = c("Jost", "Montserrat", "Roboto", "Open Sans"), 
                          verbose = TRUE) {
  results <- setNames(logical(length(fonts)), fonts)
  
  if (verbose) {
    cli::cli_h2("Downloading fonts from Google Fonts")
    cli::cli_alert_info("Requires internet connection")
    
    if (requireNamespace("curl", quietly = TRUE)) {
      cli::cli_alert_info("Using {.pkg curl} package for downloads")
    } else if (requireNamespace("gfonts", quietly = TRUE)) {
      cli::cli_alert_info("Using {.pkg gfonts} package for downloads")
    } else {
      cli::cli_alert_warning("Tip: Install {.pkg curl} or {.pkg gfonts} for more reliable downloads")
    }
  }
  
  for (fam in fonts) {
    if (verbose) cli::cli_progress_step("Downloading {.val {fam}}...")
    
    fp <- download_google_font_woff2(fam)
    success <- !is.na(fp) && file.exists(fp)
    results[[fam]] <- success
    
    if (verbose) {
      if (success) {
        format_type <- if (grepl("\\.ttf$", fp)) "TTF" else "WOFF2"
        cli::cli_alert_success(glue::glue("{fam} ({format_type})", format_type = format_type))
      } else {
        cli::cli_alert_danger("{fam} - check internet connection")
      }
    }
  }
  
  if (verbose) {
    n_ok <- sum(results)
    n_total <- length(results)
    
    cli::cli_h3("Summary")
    cli::cli_alert_info("Downloaded {n_ok}/{n_total} fonts")
    cli::cli_alert_info("Cache: {.path {font_cache_dir()}}")
    
    if (n_ok < n_total) {
      cli::cli_bullets(c(
        "i" = "Tip: Install {.pkg gfonts} for more reliable downloads:",
        " " = "{.code install.packages('gfonts')}",
        "i" = "Or use {.fn register_font} to register local font files."
      ))
    }
  }
  
  invisible(results)
}

#' Register a local font file for embedding
#'
#' Copies a local font file (TTF or WOFF2) to the cardargus cache directory
#' so it can be embedded in SVG exports.
#'
#' @param font_path Path to a local .ttf or .woff2 font file
#' @param family Font family name to register (e.g., "Jost"). If NULL,
#'   the filename without extension is used.
#' @return Path to the cached font file (invisible)
#' @export
register_font <- function(font_path, family = NULL) {
  if (!file.exists(font_path)) {
    cli::cli_abort("Font file not found: {.path {font_path}}")
  }
  
  ext <- tolower(tools::file_ext(font_path))
  if (!ext %in% c("ttf", "woff2", "woff", "otf")) {
    cli::cli_abort("Unsupported font format. Use TTF, WOFF2, WOFF, or OTF files.")
  }
  
  if (is.null(family)) {
    family <- tools::file_path_sans_ext(basename(font_path))
    family <- gsub("-?(Regular|Bold|Italic|Light|Medium|SemiBold|ExtraBold|Black|Thin).*$", "", family, ignore.case = TRUE)
  }
  
  cache_dir <- font_cache_dir()
  key <- gsub("[^A-Za-z0-9._-]+", "_", family)
  out_path <- file.path(cache_dir, paste0(key, ".", ext))
  
  file.copy(font_path, out_path, overwrite = TRUE)
  
  if (file.exists(out_path)) {
    cli::cli_alert_success("Font {.val {family}} registered successfully.")
    cli::cli_alert_info("Cached at: {.path {out_path}}")
  } else {
    cli::cli_abort("Failed to copy font file to cache.")
  }
  
  invisible(out_path)
}

# ------------------------------------------------------------------------------
# Internal: embed fonts into SVG
# ------------------------------------------------------------------------------

#' Embed a font file into an SVG via @font-face
#'
#' @param svg_content Character SVG.
#' @param font_family Font family name to embed.
#' @param woff2_path Path to a font file (.woff2, .ttf, .woff, or .otf).
#' @return SVG string with embedded @font-face rule.
#' @keywords internal
embed_svg_fonts <- function(svg_content, font_family, woff2_path) {
  if (!requireNamespace("base64enc", quietly = TRUE)) {
    cli::cli_abort(c(
      "x" = "Package {.pkg base64enc} is required to embed fonts.",
      "i" = "Install with: {.code install.packages('base64enc')}"
    ))
  }
  if (is.na(woff2_path) || !file.exists(woff2_path)) {
    return(as.character(svg_content))
  }
  
  svg <- as.character(svg_content)
  
  escaped_family <- gsub("([\\[\\](){}|.+*?^$\\\\])", "\\\\\\1", font_family, perl = TRUE)
  
  pattern_dq <- sprintf('@font-face\\s*\\{[^}]*font-family\\s*:\\s*"%s"[^}]*\\}', escaped_family)
  svg <- gsub(pattern_dq, "", svg, perl = TRUE)
  
  pattern_sq <- sprintf("@font-face\\s*\\{[^}]*font-family\\s*:\\s*'%s'[^}]*\\}", escaped_family)
  svg <- gsub(pattern_sq, "", svg, perl = TRUE)
  
  svg <- gsub("\n{3,}", "\n\n", svg, perl = TRUE)
  
  raw <- readBin(woff2_path, "raw", n = file.info(woff2_path)$size)
  b64 <- base64enc::base64encode(raw)
  
  ext <- tolower(tools::file_ext(woff2_path))
  
  mime_and_format <- switch(ext,
                            "woff2" = c("font/woff2", "woff2"),
                            "ttf"   = c("font/ttf", "truetype"),
                            "woff"  = c("font/woff", "woff"),
                            "otf"   = c("font/otf", "opentype"),
                            c("font/woff2", "woff2")
  )
  mime_type <- mime_and_format[1]
  format_str <- mime_and_format[2]
  
  css <- sprintf(
    '@font-face{font-family:"%s";src:url("data:%s;base64,%s") format("%s");font-weight:100 900;font-style:normal;}',
    font_family, mime_type, b64, format_str
  )
  
  if (grepl("<style[^>]*>", svg, perl = TRUE)) {
    svg <- sub("(<style[^>]*>)", paste0("\\1\n", css, "\n"), svg, perl = TRUE)
  } else if (grepl("<defs>", svg, fixed = TRUE)) {
    svg <- sub("<defs>", paste0("<defs>\n<style>\n", css, "\n</style>\n"), svg, fixed = TRUE)
  } else if (grepl("<defs[^>]*>", svg, perl = TRUE)) {
    svg <- sub("(<defs[^>]*>)", paste0("\\1\n<style>\n", css, "\n</style>\n"), svg, perl = TRUE)
  } else {
    svg <- sub("(<svg[^>]*>)", paste0("\\1\n<defs><style>\n", css, "\n</style></defs>\n"), svg, perl = TRUE)
  }
  
  svg
}

#' Prepare SVG for rasterization (sanitize + embed fonts)
#'
#' @description
#' Removes Google Fonts @import rules (not supported by librsvg), sanitizes
#' editor metadata (handled by sanitize_svg_for_raster()), then embeds fonts
#' referenced by the SVG into @font-face blocks.
#'
#' @param svg_content Character SVG.
#' @return A sanitized SVG with embedded WOFF2 fonts.
#' @keywords internal
prepare_svg_for_raster <- function(svg_content) {
  svg <- as.character(svg_content)
  
  if (!exists("sanitize_svg_for_raster", mode = "function")) {
    cli::cli_abort("Internal function {.fn sanitize_svg_for_raster} was not found.")
  }
  svg <- sanitize_svg_for_raster(svg)
  
  families <- detect_svg_fonts(svg)
  if (!length(families)) return(svg)
  
  paths <- ensure_cardargus_fonts(families)
  for (fam in names(paths)) {
    fp <- paths[[fam]]
    if (!is.na(fp) && file.exists(fp)) {
      svg <- embed_svg_fonts(svg, font_family = fam, woff2_path = fp)
    } else {
      cli::cli_warn("Font {.val {fam}} could not be embedded. PNG may use fallback font.")
    }
  }
  
  svg
}

# conversion.R - SVG conversion utilities
#
# Save and convert SVG cards to PNG/PDF using high-quality renderers.

#' Sanitize SVG for rasterization engines (librsvg / ImageMagick)
#'
#' @description
#' SVGs exported by editors such as Inkscape may contain `sodipodi:*` and/or
#' `inkscape:*` nodes/attributes. If these namespace prefixes appear without the
#' corresponding `xmlns:*` declarations, strict parsers (notably `librsvg`) can
#' fail with errors like:
#' \preformatted{Namespace prefix sodipodi on namedview is not defined}
#'
#' This helper removes non-rendering metadata blocks and attributes commonly
#' responsible for parse failures, producing a more interoperable SVG string.
#' It also removes Google Fonts @import rules which are not supported by
#' rasterization engines.
#'
#' @param svg_input An SVG string or an object coercible to character
#'   (e.g. `htmltools::HTML`).
#'
#' @return A character string containing a sanitized SVG.
#'
#' @keywords internal
sanitize_svg_for_raster <- function(svg_input) {
  svg <- as.character(svg_input)
  
  # Remove problematic metadata blocks (safe for rendering)
  svg <- gsub("<sodipodi:namedview[^>]*?/\\s*>", "", svg, perl = TRUE)
  svg <- gsub("<sodipodi:namedview[\\s\\S]*?</sodipodi:namedview\\s*>", "", svg, perl = TRUE)
  
  svg <- gsub("<inkscape:page[^>]*?/\\s*>", "", svg, perl = TRUE)
  svg <- gsub("<inkscape:page[\\s\\S]*?</inkscape:page\\s*>", "", svg, perl = TRUE)
  
  # Metadata can also trigger strict parsers; safe to remove for raster output
  svg <- gsub("<metadata[\\s\\S]*?</metadata\\s*>", "", svg, perl = TRUE)
  
  # Remove attributes with undefined prefixes
  svg <- gsub("\\s+sodipodi:[a-zA-Z0-9_.-]+\\s*=\\s*\"[^\"]*\"", "", svg, perl = TRUE)
  svg <- gsub("\\s+inkscape:[a-zA-Z0-9_.-]+\\s*=\\s*\"[^\"]*\"", "", svg, perl = TRUE)
  
  # Remove Google Fonts @import rules (not supported by librsvg/magick)
  # These patterns handle various URL formats and encoding (&amp; vs &)
  # Pattern 1: @import url("...googleapis...")
  svg <- gsub('@import\\s+url\\(["\'][^"\']*googleapis[^"\']*["\']\\)\\s*;?', "", svg, perl = TRUE)
  # Pattern 2: @import url(...googleapis...) without quotes
  svg <- gsub('@import\\s+url\\([^)]*googleapis[^)]*\\)\\s*;?', "", svg, perl = TRUE)
  # Pattern 3: @import url("...fonts.google...")  
  svg <- gsub('@import\\s+url\\(["\'][^"\']*fonts\\.google[^"\']*["\']\\)\\s*;?', "", svg, perl = TRUE)
  # Pattern 4: Any remaining @import with google in URL
  svg <- gsub('@import[^;]*google[^;]*;', "", svg, perl = TRUE)
  
  svg
}

#' Ensure output directory exists
#' @param path Output file path.
#' @keywords internal
ensure_output_dir <- function(path) {
  output_dir <- dirname(path)
  if (!dir.exists(output_dir) && output_dir != ".") {
    dir.create(output_dir, recursive = TRUE, showWarnings = FALSE)
  }
}

#' Write SVG content to a temporary .svg file
#' @param svg_content Character SVG content.
#' @return Path to a temporary SVG file.
#' @keywords internal
write_svg_tempfile <- function(svg_content) {
  tf <- tempfile(fileext = ".svg")
  writeLines(as.character(svg_content), tf, useBytes = TRUE)
  tf
}

#' Parse a numeric width/height from the root <svg ...> tag
#'
#' @description
#' Tries:
#' 1) width="..." / height="..." (supports px, numbers)
#' 2) viewBox="minx miny width height" as fallback
#'
#' @param svg_content Character SVG.
#' @param attr "width" or "height".
#' @return Numeric value (in CSS px) or NA_real_ if not found.
#' @keywords internal
parse_svg_root_dim <- function(svg_content, attr = c("width", "height")) {
  attr <- match.arg(attr)
  # Prefer the width/height attribute, then fall back to viewBox.
  # Parsing is delegated to xml2 (with a regex fallback); see R/xml_utils.R.
  svg_dimension(svg_content, attr, prefer = "attr")
}

# ------------------------------------------------------------------------------
# Public API
# ------------------------------------------------------------------------------

#' Save SVG string to file (sanitized + embedded fonts)
#'
#' @description
#' Saves an SVG string to disk. Before saving, the function:
#' 1) sanitizes the SVG to remove problematic Inkscape/Sodipodi metadata that can
#'    break strict XML parsers, and
#' 2) detects and embeds fonts (WOFF2 via @font-face) for deterministic rendering.
#'
#' This function expects the font helpers to be available in the package:
#' `detect_svg_fonts()`, `ensure_cardargus_fonts()`, and `embed_svg_fonts()`.
#'
#' @param svg_content SVG string (or object coercible to character).
#' @param output_path Output file path.
#'
#' @return Path to the saved SVG file.
#' @export
#'
#' @examples
#' svg <- svg_card("FAR", list(), list())
#' save_svg(svg, tempfile(fileext = ".svg"))
#' 
save_svg <- function(svg_content, output_path) {
  ensure_output_dir(output_path)
  
  svg <- as.character(svg_content)
  
  # 1) Remove problematic metadata/namespaces first (prevents XML parsing errors)
  svg <- sanitize_svg_for_raster(svg)
  
  # 2) Detect fonts and embed them (WOFF2 via @font-face)
  #    (Assumes these helpers exist in the package.)
  fonts_used <- detect_svg_fonts(svg)
  if (length(fonts_used)) {
    font_paths <- ensure_cardargus_fonts(fonts_used)
    for (family in names(font_paths)) {
      woff2_path <- font_paths[[family]]
      if (!is.na(woff2_path) && file.exists(woff2_path)) {
        svg <- embed_svg_fonts(svg, font_family = family, woff2_path = woff2_path)
      }
    }
  }
  
  writeLines(svg, output_path, useBytes = TRUE)
  output_path
}

#' Convert SVG to PNG
#'
#' @description
#' Convert an SVG string or SVG file path to a high-quality PNG image.
#' The function sanitizes the SVG and embeds required WOFF2 fonts (downloaded
#' on demand into a user cache) to ensure consistent font rendering.
#'
#' **Important note about DPI**: `rsvg` rasterizes primarily based on pixel
#' dimensions. To make DPI matter, this function scales output pixel size by
#' `(dpi / 96)` when `width` / `height` are not explicitly provided.
#'
#' @param svg_input SVG string or path to an SVG file.
#' @param output_path Output path for the PNG file (optional; a temp file is used if NULL).
#' @param width Output width in pixels (NULL to infer from SVG and scale by DPI).
#' @param height Output height in pixels (NULL to infer from SVG and scale by DPI).
#' @param dpi Resolution in dots per inch (default 300 for high quality).
#' @param background Background color. Use "transparent" or "none" for transparency
#'   (default), or specify a color like "white", "#FFFFFF", etc.
#'
#' @return Path to the generated PNG file.
#' @export
#'
#' @examples
#' svg <- svg_card("FAR", list(), list())
#' file_name <- tempfile(fileext = ".png")
#' png_path <- svg_to_png(svg, file_name, dpi = 300)
#' png_path <- svg_to_png(svg, file_name, dpi = 300, background = "white")
svg_to_png <- function(svg_input,
                       output_path = NULL,
                       width = NULL,
                       height = NULL,
                       dpi = 300,
                       background = "transparent") {
  
  if (!requireNamespace("rsvg", quietly = TRUE) &&
      !requireNamespace("magick", quietly = TRUE)) {
    cli::cli_abort(c(
      "x" = "No SVG renderer available.",
      "i" = "Install {.pkg rsvg} or {.pkg magick}:",
      " " = "{.code install.packages('rsvg')}"
    ))
  }
  
  # Read SVG (file path vs string)
  if (is.character(svg_input) && length(svg_input) == 1 && file.exists(svg_input)) {
    svg_content <- paste(readLines(svg_input, warn = FALSE), collapse = "\n")
  } else {
    svg_content <- as.character(svg_input)
  }
  
  # Sanitize + embed fonts (download/cache if needed)
  # (prepare_svg_for_raster() should be defined in fonts.R)
  svg_content <- prepare_svg_for_raster(svg_content)
  
  # Output path
  if (is.null(output_path)) output_path <- tempfile(fileext = ".png")
  ensure_output_dir(output_path)
  
  # Normalize background parameter
  bg_transparent <- tolower(background) %in% c("transparent", "none", "")
  
  # DPI scaling (SVG/CSS ref 96 DPI)
  scale <- dpi / 96
  if (is.null(width)) {
    w0 <- parse_svg_root_dim(svg_content, "width")
    if (!is.na(w0)) width <- max(1L, round(w0 * scale))
  }
  if (is.null(height)) {
    h0 <- parse_svg_root_dim(svg_content, "height")
    if (!is.na(h0)) height <- max(1L, round(h0 * scale))
  }
  
  svg_file <- write_svg_tempfile(svg_content)
  
  # Prefer rsvg first
  if (requireNamespace("rsvg", quietly = TRUE)) {
    ok <- tryCatch({
      rsvg::rsvg_png(svg_file, file = output_path, width = width, height = height)
      
      # Apply solid background if requested (flatten)
      if (!bg_transparent && requireNamespace("magick", quietly = TRUE)) {
        img <- magick::image_read(output_path)
        img <- magick::image_background(img, background, flatten = TRUE)
        magick::image_write(img, output_path, format = "png")
      }
      TRUE
    }, error = function(e) FALSE)
    
    if (isTRUE(ok)) return(output_path)
  }
  
  # Fallback: magick (IMPORTANT: set size at rasterization time via image_read_svg)
  if (requireNamespace("magick", quietly = TRUE)) {
    ok <- tryCatch({
      if (is.null(width) || is.null(height)) {
        img <- magick::image_read_svg(svg_file)
      } else {
        img <- magick::image_read_svg(svg_file, width = width, height = height)
      }
      
      if (!bg_transparent) {
        img <- magick::image_background(img, background, flatten = TRUE)
      }
      
      magick::image_write(img, output_path, format = "png")
      TRUE
    }, error = function(e) {
      cli::cli_abort("Failed to convert SVG to PNG: {e$message}")
    })
    
    if (isTRUE(ok)) return(output_path)
  }
  
  cli::cli_abort("Failed to convert SVG to PNG (no working renderer found).")
}

#' Convert SVG to multiple formats
#'
#' @description
#' Convert an SVG string or file to multiple formats. Supported formats are:
#' \itemize{
#'   \item \code{"svg"} - saves the SVG
#'   \item \code{"png"} - rasterizes to PNG via \code{\link{svg_to_png}}
#'   \item \code{"pdf"} - converts to PDF (prefers \pkg{rsvg})
#' }
#'
#' @param svg_input SVG string or file path.
#' @param output_base Base name for output files (without extension).
#' @param formats Vector of formats to generate ("png", "svg", "pdf").
#' @param dpi Resolution for raster formats.
#' @param background Background color for PNG output.
#'
#' @return Named list with paths to generated files.
#' @export
svg_to_formats <- function(svg_input,
                           output_base,
                           formats = c("svg", "png"),
                           dpi = 300,
                           background = "transparent") {
  
  results <- list()
  
  # Get SVG content
  if (is.character(svg_input) && length(svg_input) == 1 && file.exists(svg_input)) {
    svg_content <- paste(readLines(svg_input, warn = FALSE), collapse = "\n")
  } else {
    svg_content <- as.character(svg_input)
  }
  
  # Sanitize + embed fonts ONCE (prepare_svg_for_raster handles this)
  svg_prepared <- prepare_svg_for_raster(svg_content)
  
  for (fmt in formats) {
    output_path <- paste0(output_base, ".", fmt)
    
    if (identical(fmt, "svg")) {
      # Write directly - fonts are already embedded by prepare_svg_for_raster
      ensure_output_dir(output_path)
      writeLines(svg_prepared, output_path, useBytes = TRUE)
      results$svg <- output_path
      
    } else if (identical(fmt, "png")) {
      # svg_to_png will use the already-prepared content
      results$png <- svg_to_png(svg_prepared, output_path, dpi = dpi, background = background)
      
    } else if (identical(fmt, "pdf")) {
      ensure_output_dir(output_path)
      
      if (requireNamespace("rsvg", quietly = TRUE)) {
        svg_file <- write_svg_tempfile(svg_prepared)
        tryCatch({
          rsvg::rsvg_pdf(svg_file, file = output_path)
          results$pdf <- output_path
        }, error = function(e) {
          cli::cli_warn("Failed to create PDF via rsvg: {e$message}")
        })
      } else if (requireNamespace("magick", quietly = TRUE)) {
        svg_file <- write_svg_tempfile(svg_prepared)
        tryCatch({
          img <- magick::image_read_svg(svg_file)
          magick::image_write(img, path = output_path, format = "pdf")
          results$pdf <- output_path
        }, error = function(e) {
          cli::cli_warn("Failed to create PDF via magick: {e$message}")
        })
      } else {
        cli::cli_warn("PDF conversion requires {.pkg rsvg} or {.pkg magick}.")
      }
    }
  }
  
  results
}

#' Convert SVG to PNG with optional margin and background
#'
#' @description
#' Creates a PNG with extra margin around the card.
#' Fonts are embedded before rasterization for consistent appearance.
#'
#' @param svg_input SVG string or path to SVG file.
#' @param output_path Output path for PNG file (optional; a temp file is used if NULL).
#' @param margin Margin in pixels to add around the card.
#' @param margin_color Color of the margin area (default transparent).
#' @param dpi Resolution in dots per inch.
#' @param background Background color for the card rasterization (default transparent).
#'
#' @return Path to the generated PNG file.
#' @export
svg_to_png_with_margin <- function(svg_input,
                                   output_path = NULL,
                                   margin = 20,
                                   margin_color = "transparent",
                                   dpi = 300,
                                   background = "transparent") {
  
  if (!requireNamespace("magick", quietly = TRUE)) {
    cli::cli_abort(c(
      "x" = "{.fn svg_to_png_with_margin} requires the {.pkg magick} package.",
      "i" = "Install with: {.code install.packages('magick')}"
    ))
  }
  
  # First convert to PNG (rasterize card) using embedded fonts
  temp_png <- tempfile(fileext = ".png")
  svg_to_png(svg_input, temp_png, dpi = dpi, background = background)
  
  if (is.null(output_path)) output_path <- tempfile(fileext = ".png")
  ensure_output_dir(output_path)
  
  img <- magick::image_read(temp_png)
  info <- magick::image_info(img)
  
  new_width  <- info$width + 2 * margin
  new_height <- info$height + 2 * margin
  
  is_transparent_margin <- tolower(margin_color) %in% c("transparent", "none", "")
  
  # Create the background canvas
  bg <- if (is_transparent_margin) {
    magick::image_blank(new_width, new_height, color = "none")
  } else {
    magick::image_blank(new_width, new_height, color = margin_color)
  }
  
  # operator="over" is important to avoid blank composites with alpha
  result <- magick::image_composite(
    bg, img,
    operator = "over",
    offset = sprintf("+%d+%d", margin, margin)
  )
  
  magick::image_write(result, output_path, format = "png")
  output_path
}

#' Batch convert multiple SVG cards to PNG
#'
#' @description
#' Convert a list of SVG strings to PNG files in a given directory.
#' Fonts are embedded automatically for consistent rendering.
#'
#' @param svg_list List of SVG strings.
#' @param output_dir Output directory.
#' @param prefix File name prefix.
#' @param dpi Resolution.
#' @param background Background color for PNG output.
#'
#' @return Character vector of output paths.
#' @export
batch_svg_to_png <- function(svg_list,
                             output_dir = ".",
                             prefix = "card",
                             dpi = 300,
                             background = "transparent") {
  
  if (!dir.exists(output_dir)) {
    dir.create(output_dir, recursive = TRUE, showWarnings = FALSE)
  }
  
  paths <- character(length(svg_list))
  
  for (i in seq_along(svg_list)) {
    output_path <- file.path(output_dir, sprintf("%s_%03d.png", prefix, i))
    paths[i] <- svg_to_png(svg_list[[i]], output_path, dpi = dpi, background = background)
  }
  
  paths
}

test_that("parse_svg_root_dim reads width/height attributes", {
  svg <- '<svg xmlns="http://www.w3.org/2000/svg" width="500" height="320"></svg>'
  expect_equal(cardargus:::parse_svg_root_dim(svg, "width"), 500)
  expect_equal(cardargus:::parse_svg_root_dim(svg, "height"), 320)
})

test_that("parse_svg_root_dim falls back to viewBox", {
  svg <- '<svg viewBox="0 0 640 480"></svg>'
  expect_equal(cardargus:::parse_svg_root_dim(svg, "width"), 640)
  expect_equal(cardargus:::parse_svg_root_dim(svg, "height"), 480)
})

test_that("sanitize_svg_for_raster strips Inkscape metadata and @import", {
  svg <- paste0(
    '<svg xmlns="http://www.w3.org/2000/svg">',
    '<sodipodi:namedview id="nv"/>',
    '<style>@import url("https://fonts.googleapis.com/css2?family=Jost");</style>',
    '<rect sodipodi:role="line" width="10" height="10"/>',
    '</svg>'
  )
  out <- cardargus:::sanitize_svg_for_raster(svg)
  expect_false(grepl("sodipodi", out))
  expect_false(grepl("@import", out))
  expect_match(out, "<rect")
})

test_that("detect_svg_fonts finds families and drops generics", {
  svg <- paste0(
    '<svg><style>.t{font-family:"Jost", sans-serif;}</style>',
    '<text font-family="Montserrat">hi</text></svg>'
  )
  fams <- cardargus:::detect_svg_fonts(svg)
  expect_true("Jost" %in% fams)
  expect_true("Montserrat" %in% fams)
  expect_false("sans-serif" %in% fams)
})

# logos.R - SVG logo icons and logo management

#' Load and process external SVG file for embedding
#'
#' Reads an SVG file and processes it to be embedded inside another SVG.
#' Removes XML declarations, adjusts dimensions, and prepares for embedding.
#'
#' @details
#' This function is useful when you want to embed custom logos or icons in cards.
#' You can pass any SVG file path to the `logos`, `bottom_logos`, or `with_icon`
#' parameters of `svg_card()`.
#'
#' @param svg_path Path to the SVG file
#' @param target_height Desired height in pixels
#' @param target_width Optional desired width (calculated from aspect ratio if NULL)
#' @return A list with svg_content, width, and height
#' @export
#' @examples
#' \dontrun{ # Need a external svg file
#' # Load a custom logo
#' logo <- load_svg_for_embed("/path/to/logo.svg", target_height = 40)
#' 
#' # Or just pass the path directly to svg_card():
#' svg_card(
#'   title = "My Card",
#'   logos = c("/path/to/logo1.svg", "/path/to/logo2.svg"),
#'   ...
#' )
#' }
load_svg_for_embed <- function(svg_path, target_height = 40, target_width = NULL) {
  if (!file.exists(svg_path)) {
    stop("SVG file not found: ", svg_path)
  }
  
  # Read SVG content
  svg_content <- paste(readLines(svg_path, warn = FALSE), collapse = "\n")
  
  # Remove XML declaration and comments
  svg_content <- gsub('<\\?xml[^>]*\\?>', '', svg_content)
  svg_content <- gsub('<!--[^>]*-->', '', svg_content)
  
  # Extract original dimensions from viewBox (preferred for aspect ratio) or
  # width/height. Parsing delegated to xml2 with regex fallback (R/xml_utils.R).
  geom <- svg_geometry(svg_content)
  has_viewbox <- !is.na(geom$vb_w) && !is.na(geom$vb_h)

  if (has_viewbox) {
    orig_width <- geom$vb_w
    orig_height <- geom$vb_h
  } else if (!is.na(geom$attr_w) && !is.na(geom$attr_h)) {
    orig_width <- geom$attr_w
    orig_height <- geom$attr_h
  } else {
    # Default to square if can't determine
    orig_width <- 100
    orig_height <- 100
  }
  
  # Calculate new dimensions maintaining aspect ratio
  aspect_ratio <- orig_width / orig_height
  new_height <- target_height
  new_width <- if (!is.null(target_width)) target_width else round(target_height * aspect_ratio)
  
  # Remove existing width/height attributes from opening svg tag
  svg_content <- gsub('(<svg[^>]*?)\\s+width="[^"]*"', '\\1', svg_content)
  svg_content <- gsub('(<svg[^>]*?)\\s+height="[^"]*"', '\\1', svg_content)
  
  # Ensure viewBox exists
  if (!has_viewbox) {
    svg_content <- gsub('<svg', sprintf('<svg viewBox="0 0 %s %s"', orig_width, orig_height), svg_content)
  }
  
  # Add new width and height
  svg_content <- gsub('<svg', sprintf('<svg width="%.0f" height="%.0f"', new_width, new_height), svg_content)
  
  # Remove problematic namespace declarations that can cause issues when embedded
  svg_content <- gsub('xmlns:inkscape="[^"]*"', '', svg_content)
  svg_content <- gsub('xmlns:sodipodi="[^"]*"', '', svg_content)
  svg_content <- gsub('sodipodi:[a-z]+="[^"]*"', '', svg_content)
  svg_content <- gsub('inkscape:[a-z]+="[^"]*"', '', svg_content)
  
  # Clean up extra spaces
  svg_content <- gsub('\\s+>', '>', svg_content)
  svg_content <- gsub('<svg\\s+', '<svg ', svg_content)
  
  list(
    svg_content = svg_content,
    width = new_width,
    height = new_height
  )
}

#' Create logo row for top-right corner of card
#'
#' Takes a list of SVG logos and arranges them horizontally with proper spacing.
#' Returns the SVG elements positioned for the top-right corner.
#'
#' @param logos List of SVG strings or file paths
#' @param target_height Height for all logos (default 40)
#' @param spacing Horizontal spacing between logos (default 10)
#' @param card_width Total card width to calculate positioning
#' @param x_offset Right margin from card edge
#' @param y_offset Top margin
#' @return A list with svg_content and total_width
#' @export
create_logo_row <- function(logos, target_height = 40, spacing = 10, 
                            card_width = 500, x_offset = 20, y_offset = 20) {
  if (length(logos) == 0) {
    return(list(svg_content = "", total_width = 0))
  }
  
  processed_logos <- list()
  total_width <- 0
  
  for (i in seq_along(logos)) {
    logo <- logos[[i]]
    
    if (is.character(logo) && file.exists(logo)) {
      # It's a file path - load and process
      processed <- load_svg_for_embed(logo, target_height = target_height)
      processed_logos[[i]] <- processed
      total_width <- total_width + processed$width + (if (i > 1) spacing else 0)
    } else if (is.character(logo)) {
      # It's already SVG content - extract dimensions and resize
      logo_dims <- svg_dims(logo, prefer = "attr")

      if (!is.na(logo_dims$width) && !is.na(logo_dims$height)) {
        orig_width <- logo_dims$width
        orig_height <- logo_dims$height
        aspect_ratio <- orig_width / orig_height
        new_width <- round(target_height * aspect_ratio)
        
        # Update dimensions in SVG
        logo <- gsub('width="[0-9.]+"', sprintf('width="%.0f"', new_width), logo)
        logo <- gsub('height="[0-9.]+"', sprintf('height="%.0f"', target_height), logo)
        
        processed_logos[[i]] <- list(svg_content = logo, width = new_width, height = target_height)
        total_width <- total_width + new_width + (if (i > 1) spacing else 0)
      } else {
        # Can't determine dimensions, use target_height as square
        processed_logos[[i]] <- list(svg_content = logo, width = target_height, height = target_height)
        total_width <- total_width + target_height + (if (i > 1) spacing else 0)
      }
    }
  }
  
  # Build positioned logo SVG content
  start_x <- card_width - x_offset - total_width
  current_x <- start_x
  svg_parts <- c()
  
  for (i in seq_along(processed_logos)) {
    logo_data <- processed_logos[[i]]
    
    # Wrap each logo in a group with translation
    svg_part <- sprintf(
      '<g transform="translate(%.0f, %.0f)">%s</g>',
      current_x, y_offset, logo_data$svg_content
    )
    svg_parts <- c(svg_parts, svg_part)
    
    current_x <- current_x + logo_data$width + spacing
  }
  
  list(
    svg_content = paste(svg_parts, collapse = "\n"),
    total_width = total_width,
    start_x = start_x
  )
}

#' Create logo row for bottom-left corner of card
#'
#' Takes a list of SVG logos and arranges them horizontally for the bottom-left.
#'
#' @param logos List of SVG strings or file paths
#' @param target_height Height for all logos (default 30)
#' @param spacing Horizontal spacing between logos (default 10)
#' @param x_offset Left margin from card edge
#' @param card_height Total card height for y positioning
#' @param y_offset Bottom margin
#' @return A list with svg_content and total_width
#' @export
create_bottom_logo_row <- function(logos, target_height = 30, spacing = 10,
                                   x_offset = 20, card_height = 400, y_offset = 20) {
  if (length(logos) == 0) {
    return(list(svg_content = "", total_width = 0))
  }
  
  processed_logos <- list()
  total_width <- 0
  
  for (i in seq_along(logos)) {
    logo <- logos[[i]]
    
    if (is.character(logo) && file.exists(logo)) {
      processed <- load_svg_for_embed(logo, target_height = target_height)
      processed_logos[[i]] <- processed
      total_width <- total_width + processed$width + (if (i > 1) spacing else 0)
    } else if (is.character(logo)) {
      logo_dims <- svg_dims(logo, prefer = "attr")

      if (!is.na(logo_dims$width) && !is.na(logo_dims$height)) {
        orig_width <- logo_dims$width
        orig_height <- logo_dims$height
        aspect_ratio <- orig_width / orig_height
        new_width <- round(target_height * aspect_ratio)
        
        logo <- gsub('width="[0-9.]+"', sprintf('width="%.0f"', new_width), logo)
        logo <- gsub('height="[0-9.]+"', sprintf('height="%.0f"', target_height), logo)
        
        processed_logos[[i]] <- list(svg_content = logo, width = new_width, height = target_height)
        total_width <- total_width + new_width + (if (i > 1) spacing else 0)
      } else {
        processed_logos[[i]] <- list(svg_content = logo, width = target_height, height = target_height)
        total_width <- total_width + target_height + (if (i > 1) spacing else 0)
      }
    }
  }
  
  # Position at bottom-left
  y_position <- card_height - y_offset - target_height
  current_x <- x_offset
  svg_parts <- c()
  
  for (i in seq_along(processed_logos)) {
    logo_data <- processed_logos[[i]]
    
    svg_part <- sprintf(
      '<g transform="translate(%.0f, %.0f)">%s</g>',
      current_x, y_position, logo_data$svg_content
    )
    svg_parts <- c(svg_parts, svg_part)
    
    current_x <- current_x + logo_data$width + spacing
  }
  
  list(
    svg_content = paste(svg_parts, collapse = "\n"),
    total_width = total_width,
    y_position = y_position
  )
}

#' House Icon SVG
#'
#' Generate a house/home icon SVG. You can also use any SVG file path
#' instead of built-in icons.
#'
#' @param width Width of the icon
#' @param height Height of the icon
#' @param stroke_color Stroke color
#' @param stroke_width Stroke width
#' @param fill Fill color (default none)
#' @return SVG string
#' @export
#' @examples
#' icon_house(50, 56)
#' 
#' # You can also use a custom SVG file:
#' # svg_card(..., with_icon = "/path/to/my_icon.svg")
icon_house <- function(width = 50, height = 56, 
                       stroke_color = "white", 
                       stroke_width = 35,
                       fill = "none") {
  sprintf(
    '<svg width="%d" height="%d" viewBox="0 0 572 564" fill="%s" xmlns="http://www.w3.org/2000/svg">
    <path d="M286.666 20.5544H286.667C288.134 21.715 288.135 21.7152 289.632 22.8992L289.749 22.991L296.055 27.8855C297.612 29.0963 297.62 29.102 299.224 30.3494L299.286 30.3982C303.942 33.9851 308.638 37.4306 313.189 40.7546V40.7556C321.461 46.8148 329.655 53.0311 337.908 59.3513L337.915 59.3552L337.921 59.3601C344.353 64.2774 350.827 69.114 357.341 73.8875C365.614 79.9503 373.812 86.1666 382.062 92.4851L382.068 92.49L382.074 92.4939C388.507 97.4112 394.981 102.248 401.494 107.021C409.767 113.084 417.965 119.3 426.216 125.619L426.223 125.624L426.229 125.629C432.661 130.546 439.135 135.381 445.647 140.154V140.155C455.07 147.062 464.394 154.154 473.814 161.36V161.361C477.79 164.402 481.786 167.427 485.808 170.431C490.661 174.058 495.497 177.743 500.38 181.478L500.382 181.479C507.012 186.55 513.763 191.711 520.628 196.791L523.576 198.963C525.209 200.168 526.818 201.371 528.404 202.582L528.463 202.627C533.293 206.281 538.402 209.786 544.27 212.95C547.924 215.038 550.174 216.439 551.839 217.864C552.719 218.617 553.229 219.211 553.545 219.699C554.612 226.321 554.459 233.408 554.206 242.494L554.204 242.543L554.203 242.593C554.126 246.06 554.128 249.524 554.14 252.824L554.153 256.064C554.164 263.428 554.097 270.804 553.974 278.204C553.791 289.17 553.734 300.108 553.706 310.974C553.658 328.506 553.512 346.045 553.304 363.599C553.1 380.713 552.944 397.833 552.85 414.958C552.844 415.915 552.838 417.197 552.832 418.186V418.187C552.786 426.524 552.743 434.86 552.7 443.194C552.522 478.111 552.171 511.279 551.815 545.753H21.2832C20.7333 493.984 20.5786 479.892 20.4639 462.669L20.3359 440.362L20.1943 415.014C20.1911 414.454 20.1792 412.399 20.1758 411.807V411.805L20.085 398.968C20.0155 390.411 19.9296 381.858 19.832 373.31L19.6777 360.494C19.4613 343.026 19.3339 325.556 19.2871 308.082V308.077L19.25 299.915C19.2077 293.111 19.1365 286.304 19.0312 279.492L18.9639 275.404C18.8433 268.123 18.8074 260.831 18.8369 253.517V253.508C18.8501 249.686 18.8353 245.777 18.7422 241.813L18.6973 240.111C18.5223 233.825 18.4012 229.443 18.3936 226.112C18.3865 223.051 18.4852 221.56 18.5771 220.816C18.6272 220.74 18.6858 220.648 18.7588 220.542C18.9536 220.258 19.175 219.946 19.457 219.556C22.9656 216.262 27.149 213.504 32.6768 210.43L32.8105 210.356L32.9414 210.28C38.8184 206.877 44.038 202.807 48.1201 199.693C49.7923 198.462 51.4636 197.236 53.1348 196.014L53.165 195.993L53.1943 195.97C60.023 190.936 66.7359 185.779 73.2754 180.769C79.5598 175.964 85.9122 171.229 92.3311 166.531L92.334 166.529C99.3183 161.415 106.203 156.196 113.013 150.852C120.322 145.117 127.778 139.528 135.377 133.967L135.391 133.957C142.144 129.004 148.787 123.974 155.348 118.948L155.349 118.949C163.393 112.788 171.512 106.72 179.708 100.714L179.716 100.708L179.725 100.702C186.417 95.7851 193.002 90.793 199.502 85.8152C207.547 79.6542 215.667 73.586 223.862 67.5798L223.878 67.5681C231.583 61.9076 239.15 56.1311 246.587 50.4353C253.736 44.9702 260.959 39.5928 268.253 34.2781L268.265 34.2693L268.275 34.2615C272.777 30.9703 277.275 27.5745 281.734 24.0115C283.011 22.9996 283.012 22.9988 284.314 21.9666L284.346 21.9412C285.02 21.4043 285.692 20.8622 286.364 20.3191C286.465 20.3971 286.565 20.476 286.666 20.5544Z" stroke="%s" stroke-width="%d"/>
    </svg>',
    width, height, fill, stroke_color, stroke_width
  )
}

#' Building Icon SVG
#'
#' Generate a building/apartment icon SVG.
#'
#' @param width Width of the icon
#' @param height Height of the icon
#' @param stroke_color Stroke color
#' @param stroke_width Stroke width
#' @param fill Fill color
#' @return SVG string
#' @export
#' @examples
#' icon_building(50, 56)
icon_building <- function(width = 50, height = 56, 
                          stroke_color = "white", 
                          stroke_width = 2,
                          fill = "none") {
  sprintf(
    '<svg width="%d" height="%d" viewBox="0 0 50 56" fill="%s" xmlns="http://www.w3.org/2000/svg">
      <rect x="5" y="10" width="40" height="44" stroke="%s" stroke-width="%d" fill="%s"/>
      <rect x="10" y="16" width="8" height="8" stroke="%s" stroke-width="1" fill="%s"/>
      <rect x="21" y="16" width="8" height="8" stroke="%s" stroke-width="1" fill="%s"/>
      <rect x="32" y="16" width="8" height="8" stroke="%s" stroke-width="1" fill="%s"/>
      <rect x="10" y="28" width="8" height="8" stroke="%s" stroke-width="1" fill="%s"/>
      <rect x="21" y="28" width="8" height="8" stroke="%s" stroke-width="1" fill="%s"/>
      <rect x="32" y="28" width="8" height="8" stroke="%s" stroke-width="1" fill="%s"/>
      <rect x="18" y="42" width="14" height="12" stroke="%s" stroke-width="1" fill="%s"/>
    </svg>',
    width, height, fill,
    stroke_color, stroke_width, fill,
    stroke_color, fill,
    stroke_color, fill,
    stroke_color, fill,
    stroke_color, fill,
    stroke_color, fill,
    stroke_color, fill,
    stroke_color, fill
  )
}

#' Construction Icon SVG
#'
#' Generate a construction/crane icon SVG.
#'
#' @param width Width of the icon
#' @param height Height of the icon
#' @param stroke_color Stroke color
#' @param stroke_width Stroke width
#' @param fill Fill color
#' @return SVG string
#' @export
#' @examples
#' icon_construction(50, 56)
icon_construction <- function(width = 50, height = 56, 
                              stroke_color = "white", 
                              stroke_width = 2,
                              fill = "none") {
  sprintf(
    '<svg width="%d" height="%d" viewBox="0 0 50 56" fill="%s" xmlns="http://www.w3.org/2000/svg">
      <!-- Base -->
      <rect x="20" y="48" width="10" height="6" stroke="%s" stroke-width="%d" fill="%s"/>
      <!-- Tower -->
      <line x1="25" y1="48" x2="25" y2="8" stroke="%s" stroke-width="%d"/>
      <!-- Arm -->
      <line x1="25" y1="8" x2="45" y2="8" stroke="%s" stroke-width="%d"/>
      <!-- Cable -->
      <line x1="40" y1="8" x2="40" y2="25" stroke="%s" stroke-width="1"/>
      <!-- Hook -->
      <path d="M38 25 L42 25 L42 30 Q40 33 38 30 Z" stroke="%s" stroke-width="1" fill="%s"/>
      <!-- Counter weight -->
      <line x1="25" y1="8" x2="10" y2="8" stroke="%s" stroke-width="%d"/>
      <rect x="5" y="8" width="8" height="6" stroke="%s" stroke-width="1" fill="%s"/>
      <!-- Support cables -->
      <line x1="25" y1="15" x2="10" y2="8" stroke="%s" stroke-width="1"/>
      <line x1="25" y1="15" x2="40" y2="8" stroke="%s" stroke-width="1"/>
    </svg>',
    width, height, fill,
    stroke_color, stroke_width, fill,
    stroke_color, stroke_width,
    stroke_color, stroke_width,
    stroke_color,
    stroke_color, fill,
    stroke_color, stroke_width,
    stroke_color, fill,
    stroke_color,
    stroke_color
  )
}

#' Map Pin Icon SVG
#'
#' Generate a map pin/location icon SVG.
#'
#' @param width Width of the icon
#' @param height Height of the icon
#' @param stroke_color Stroke color
#' @param stroke_width Stroke width
#' @param fill Fill color
#' @return SVG string
#' @export
#' @examples
#' icon_map_pin(50, 56)
icon_map_pin <- function(width = 50, height = 56, 
                         stroke_color = "white", 
                         stroke_width = 2,
                         fill = "none") {
  sprintf(
    '<svg width="%d" height="%d" viewBox="0 0 50 56" fill="%s" xmlns="http://www.w3.org/2000/svg">
      <path d="M25 4 C14 4 6 12 6 22 C6 35 25 52 25 52 C25 52 44 35 44 22 C44 12 36 4 25 4 Z" 
            stroke="%s" stroke-width="%d" fill="%s"/>
      <circle cx="25" cy="22" r="8" stroke="%s" stroke-width="%d" fill="%s"/>
    </svg>',
    width, height, fill,
    stroke_color, stroke_width, fill,
    stroke_color, stroke_width, fill
  )
}

#' Dollar/Money Icon SVG
#'
#' Generate a dollar/money icon SVG.
#'
#' @param width Width of the icon
#' @param height Height of the icon
#' @param stroke_color Stroke color
#' @param stroke_width Stroke width
#' @param fill Fill color
#' @return SVG string
#' @export
#' @examples
#' icon_money(50, 56)
icon_money <- function(width = 50, height = 56, 
                       stroke_color = "white", 
                       stroke_width = 2,
                       fill = "none") {
  sprintf(
    '<svg width="%d" height="%d" viewBox="0 0 50 56" fill="%s" xmlns="http://www.w3.org/2000/svg">
      <circle cx="25" cy="28" r="20" stroke="%s" stroke-width="%d" fill="%s"/>
      <text x="25" y="35" font-family="Arial, sans-serif" font-size="24" font-weight="bold" 
            fill="%s" text-anchor="middle">$</text>
    </svg>',
    width, height, fill,
    stroke_color, stroke_width, fill,
    stroke_color
  )
}

#' Generic logo placeholder SVG
#'
#' @description
#' **Deprecated.** Use [load_svg_for_embed()] instead,
#' or simply pass the SVG file path directly to `svg_card()`.
#'
#' @param svg_content Raw SVG content or path to SVG file
#' @param width Desired width
#' @param height Desired height
#' @return SVG string
#' @keywords internal
custom_logo_svg <- function(svg_content, width = 40, height = 40) {
  cli::cli_warn(c(
    "!" = "{.fn custom_logo_svg} is deprecated.",
    "i" = "Use {.fn load_svg_for_embed} or pass file paths directly to {.fn svg_card}."
  ))
  
  if (file.exists(svg_content)) {
    result <- load_svg_for_embed(svg_content, target_height = height, target_width = width)
    return(result$svg_content)
  }
  
  # It's raw SVG content
  svg_content <- gsub('width="[0-9.]+"', sprintf('width="%d"', width), svg_content)
  svg_content <- gsub('height="[0-9.]+"', sprintf('height="%d"', height), svg_content)
  
  return(svg_content)
}

# chrome.R - Chrome-based SVG conversion
#
# Convert SVG to PNG/PDF using headless Chrome via chromote package.
# Provides superior font rendering for Google Fonts.

#' Check if Chrome/Chromium is available for rendering
#'
#' @description
#' Checks whether the chromote package can find and use a Chrome or Chromium
#' installation for headless rendering.
#'
#' @param verbose Print status messages (default FALSE).
#'
#' @return TRUE if Chrome is available, FALSE otherwise.
#' @export
#'
#' @examples
#' if (chrome_available()) {
#'    cat("Using Chrome")
#' } else {
#'    cat("Using Magick")
#' }
#' 
chrome_available <- function(verbose = FALSE) {
  if (!requireNamespace("chromote", quietly = TRUE)) {
    if (verbose) cli::cli_alert_warning("Package {.pkg chromote} is not installed.")
    return(FALSE)
  }
  
  tryCatch({
    if (verbose) cli::cli_alert_success("Chrome found: {.path {find_chrome_path()}}")
    TRUE
  }, error = function(e) {
    if (verbose) cli::cli_alert_warning("Chrome not found: {e$message}")
    FALSE
  })
}

#' Find Chrome executable path
#'
#' @description
#' Attempts to find a Chrome or Chromium executable on the system.
#' Checks common installation paths and environment variables.
#'
#' @return Path to Chrome executable, or NULL if not found.
#' @export
#'
#' @examples
#' path <- find_chrome_path()
#' if (!is.null(path)) {
#'   message("Chrome found at: ", path)
#' }
find_chrome_path <- function() {
  # Check environment variable first
  env_path <- Sys.getenv("CHROMOTE_CHROME", "")
  if (nzchar(env_path) && file.exists(env_path)) {
    return(env_path)
  }
  
  # Try chromote's finder
  if (requireNamespace("chromote", quietly = TRUE)) {
    tryCatch({
      return(chromote::find_chrome())
    }, error = function(e) NULL)
  }
  
  # Manual search for common paths
  candidates <- c(
    # Linux
    "/usr/bin/google-chrome",
    "/usr/bin/google-chrome-stable",
    "/usr/bin/chromium",
    "/usr/bin/chromium-browser",
    "/snap/bin/chromium",
    # macOS
    "/Applications/Google Chrome.app/Contents/MacOS/Google Chrome",
    "/Applications/Chromium.app/Contents/MacOS/Chromium",
    # Windows
    file.path(Sys.getenv("PROGRAMFILES"), "Google/Chrome/Application/chrome.exe"),
    file.path(Sys.getenv("PROGRAMFILES(X86)"), "Google/Chrome/Application/chrome.exe"),
    file.path(Sys.getenv("LOCALAPPDATA"), "Google/Chrome/Application/chrome.exe")
  )
  
  for (path in candidates) {
    if (file.exists(path)) {
      return(path)
    }
  }
  
  # Try 'which' on Unix-like systems
  if (.Platform$OS.type == "unix") {
    for (cmd in c("google-chrome", "google-chrome-stable", "chromium", "chromium-browser")) {
      result <- tryCatch({
        path <- system2("which", cmd, stdout = TRUE, stderr = FALSE)
        if (length(path) > 0 && file.exists(path[1])) path[1] else NULL
      }, error = function(e) NULL, warning = function(w) NULL)
      if (!is.null(result)) return(result)
    }
  }
  
  NULL
}

#' Ensure Chrome is available, downloading if necessary
#'
#' @description
#' Checks if Chrome is available and optionally downloads a standalone
#' Chrome for Testing if not found. This ensures Chrome-based rendering
#' works without requiring a system-wide Chrome installation.
#'
#' @param download If TRUE and Chrome is not found, attempt to download
#'   Chrome for Testing (default FALSE).
#' @param verbose Print status messages (default TRUE).
#'
#' @return TRUE if Chrome is available (or was successfully downloaded),
#'   FALSE otherwise.
#' @export
#'
#' @details
#' When `download = TRUE`, this function will download "Chrome for Testing",
#' a standalone Chrome distribution designed for automation. The download
#' is approximately 150MB and is cached in the user's data directory.
#'
#' Alternatively, you can:
#' - Install Chrome/Chromium system-wide
#' - Set the `CHROMOTE_CHROME` environment variable to point to an existing installation
#'
#' @examples
#' # Check and report status
#' ensure_chrome()
#'
#' # Download Chrome if not available
#' \dontrun{
#' ensure_chrome(download = TRUE)
#' }
ensure_chrome <- function(download = FALSE, verbose = TRUE) {
  # Check if already available
  if (chrome_available(verbose = FALSE)) {
    if (verbose) {
      cli::cli_alert_success("Chrome is available: {.path {find_chrome_path()}}")
    }
    return(TRUE)
  }
  
  if (!download) {
    if (verbose) {
      cli::cli_alert_warning("Chrome/Chromium not found on this system.")
      cli::cli_h3("Options to enable Chrome rendering")
      cli::cli_bullets(c(
        "1" = "Install Chrome or Chromium:",
        " " = "Linux: {.code sudo apt install chromium-browser}",
        " " = "macOS: {.code brew install --cask google-chrome}",
        " " = "Windows: Download from {.url https://www.google.com/chrome/}",
        "2" = "Or set path to existing Chrome:",
        " " = '{.code Sys.setenv(CHROMOTE_CHROME = "/path/to/chrome")}',
        "3" = "Or download Chrome for Testing:",
        " " = "{.code ensure_chrome(download = TRUE)}"
      ))
    }
    return(FALSE)
  }
  
  # Attempt to download Chrome for Testing
  if (verbose) cli::cli_alert_info("Downloading Chrome for Testing...")
  
  if (!requireNamespace("chromote", quietly = TRUE)) {
    if (verbose) {
      cli::cli_alert_danger("Package {.pkg chromote} is required.")
      cli::cli_alert_info("Install with: {.code install.packages('chromote')}")
    }
    return(FALSE)
  }
  
  # Get platform-specific download URL
  download_info <- get_chrome_download_info()
  if (is.null(download_info)) {
    if (verbose) cli::cli_alert_danger("Could not determine download URL for this platform.")
    return(FALSE)
  }
  
  # Create cache directory
  cache_dir <- chrome_cache_dir()
  if (!dir.exists(cache_dir)) {
    dir.create(cache_dir, recursive = TRUE, showWarnings = FALSE)
  }
  
  zip_path <- file.path(cache_dir, download_info$filename)
  chrome_dir <- file.path(cache_dir, "chrome")
  
  # Download if not already cached
  if (!dir.exists(chrome_dir)) {
    if (verbose) cli::cli_progress_step("Downloading Chrome for Testing (~150MB)...")
    
    tryCatch({
      download.file(download_info$url, zip_path, mode = "wb", quiet = !verbose)
      
      if (verbose) cli::cli_progress_step("Extracting...")
      unzip(zip_path, exdir = cache_dir)
      
      # Rename extracted folder
      extracted <- list.dirs(cache_dir, recursive = FALSE, full.names = TRUE)
      extracted <- extracted[grepl("chrome", basename(extracted), ignore.case = TRUE)]
      if (length(extracted) > 0 && extracted[1] != chrome_dir) {
        file.rename(extracted[1], chrome_dir)
      }
      
      # Clean up zip
      unlink(zip_path)
      
    }, error = function(e) {
      if (verbose) cli::cli_alert_danger("Download failed: {e$message}")
      return(FALSE)
    })
  }
  
  # Find the executable
  chrome_exe <- find_chrome_in_dir(chrome_dir)
  
  if (!is.null(chrome_exe) && file.exists(chrome_exe)) {
    # Make executable on Unix
    if (.Platform$OS.type == "unix") {
      Sys.chmod(chrome_exe, mode = "0755")
    }
    
    # Set environment variable
    Sys.setenv(CHROMOTE_CHROME = chrome_exe)
    
    if (verbose) {
      cli::cli_alert_success("Chrome for Testing installed successfully!")
      cli::cli_alert_info("Location: {.path {chrome_exe}}")
    }
    return(TRUE)
  }
  
  if (verbose) cli::cli_alert_danger("Could not find Chrome executable after extraction.")
  FALSE
}

#' Get Chrome cache directory
#' @keywords internal
chrome_cache_dir <- function() {
  tryCatch(
    tools::R_user_dir("cardargus", which = "cache"),
    error = function(e) file.path(tempdir(), "cardargus-chrome")
  )
}

#' Get Chrome for Testing download info
#' @keywords internal
get_chrome_download_info <- function() {
  # Chrome for Testing URLs (stable channel)
  base_url <- "https://storage.googleapis.com/chrome-for-testing-public"
  version <- "131.0.6778.87"
  
  platform <- get_chrome_platform()
  if (is.null(platform)) return(NULL)
  
  list(
    url = sprintf("%s/%s/%s/chrome-%s.zip", base_url, version, platform, platform),
    filename = sprintf("chrome-%s.zip", platform),
    platform = platform
  )
}

#' Get platform identifier for Chrome downloads
#' @keywords internal
get_chrome_platform <- function() {
  os <- Sys.info()["sysname"]
  arch <- Sys.info()["machine"]
  
  if (os == "Linux") {
    return("linux64")
  } else if (os == "Darwin") {
    if (arch == "arm64") {
      return("mac-arm64")
    } else {
      return("mac-x64")
    }
  } else if (os == "Windows") {
    if (grepl("64", arch)) {
      return("win64")
    } else {
      return("win32")
    }
  }
  
  NULL
}

#' Find Chrome executable in a directory
#' @keywords internal
find_chrome_in_dir <- function(dir) {
  if (!dir.exists(dir)) return(NULL)
  
  os <- Sys.info()["sysname"]
  
  if (os == "Windows") {
    candidates <- list.files(dir, pattern = "chrome\\.exe$", 
                             recursive = TRUE, full.names = TRUE)
  } else if (os == "Darwin") {
    candidates <- list.files(dir, pattern = "^Google Chrome for Testing$|^Chromium$|^chrome$", 
                             recursive = TRUE, full.names = TRUE)
    app_dirs <- list.dirs(dir, recursive = TRUE)
    for (app in app_dirs) {
      if (grepl("\\.app/Contents/MacOS$", app)) {
        bins <- list.files(app, full.names = TRUE)
        candidates <- c(candidates, bins)
      }
    }
  } else {
    candidates <- list.files(dir, pattern = "^chrome$", 
                             recursive = TRUE, full.names = TRUE)
  }
  
  for (cand in candidates) {
    if (file.exists(cand)) {
      info <- file.info(cand)
      if (!info$isdir) return(cand)
    }
  }
  
  NULL
}

# ------------------------------------------------------------------------------
# Internal helper: wrap SVG in minimal HTML
# ------------------------------------------------------------------------------

#' Create a temporary HTML file wrapping an SVG
#'
#' @param svg_string SVG content as character string.
#' @param width_px Width in pixels. If NULL, extracted from SVG.
#' @param height_px Height in pixels. If NULL, extracted from SVG.
#' @param background Background color (default "transparent").
#'
#' @return A list with path, width, and height.
#' @keywords internal
write_svg_html_temp <- function(svg_string,
                                width_px = NULL,
                                height_px = NULL,
                                background = "transparent") {
  svg_string <- as.character(svg_string)
  
  if (is.null(width_px)) {
    width_px <- parse_svg_root_dim(svg_string, "width")
  }
  if (is.null(height_px)) {
    height_px <- parse_svg_root_dim(svg_string, "height")
  }
  
  if (is.na(width_px) || is.null(width_px)) width_px <- 800
  if (is.na(height_px) || is.null(height_px)) height_px <- 500
  
  width_px <- as.integer(ceiling(width_px))
  height_px <- as.integer(ceiling(height_px))
  
  html <- sprintf(
    '<!doctype html>
<html>
<head>
<meta charset="utf-8"/>
<style>
  html, body {
    margin: 0; padding: 0;
    width: %dpx; height: %dpx;
    background: %s;
    overflow: hidden;
  }
  .wrap {
    width: %dpx; height: %dpx;
    display: flex;
    align-items: flex-start;
    justify-content: flex-start;
  }
  svg { display: block; }
</style>
</head>
<body>
  <div class="wrap">
    %s
  </div>
</body>
</html>',
    width_px, height_px, background,
    width_px, height_px,
    svg_string
  )
  
  tf <- tempfile(fileext = ".html")
  writeLines(html, tf, useBytes = TRUE)
  list(path = tf, width = width_px, height = height_px)
}

# ------------------------------------------------------------------------------
# Internal: sanitize SVG metadata only (not @import)
# ------------------------------------------------------------------------------

#' Sanitize SVG metadata for Chrome rendering
#'
#' @description
#' Removes problematic Inkscape/Sodipodi metadata that can cause issues,
#' but preserves @import rules since Chrome handles them correctly.
#'
#' @param svg_content SVG string.
#' @return Sanitized SVG string.
#' @keywords internal
sanitize_svg_metadata <- function(svg_content) {
  svg <- as.character(svg_content)
  
  svg <- gsub("<sodipodi:namedview[^>]*?/\\s*>", "", svg, perl = TRUE)
  svg <- gsub("<sodipodi:namedview[\\s\\S]*?</sodipodi:namedview\\s*>", "", svg, perl = TRUE)
  svg <- gsub("<inkscape:page[^>]*?/\\s*>", "", svg, perl = TRUE)
  svg <- gsub("<inkscape:page[\\s\\S]*?</inkscape:page\\s*>", "", svg, perl = TRUE)
  svg <- gsub("<metadata[\\s\\S]*?</metadata\\s*>", "", svg, perl = TRUE)
  
  svg <- gsub("\\s+sodipodi:[a-zA-Z0-9_.-]+\\s*=\\s*\"[^\"]*\"", "", svg, perl = TRUE)
  svg <- gsub("\\s+inkscape:[a-zA-Z0-9_.-]+\\s*=\\s*\"[^\"]*\"", "", svg, perl = TRUE)
  
  svg
}

# ------------------------------------------------------------------------------
# Internal: Chrome session management
# ------------------------------------------------------------------------------

#' Safely close a Chromote session
#'
#' @description
#' Closes a chromote session handling pending promises to avoid
#' "Unhandled promise error: timed out waiting for response" warnings.
#' This function processes pending async operations before closing
#' and silently ignores any timeout errors during cleanup.
#'
#' @param session A ChromoteSession object to close.
#' @param timeout_before Seconds to wait for pending promises before closing (default 2).
#' @param timeout_after Seconds to wait for cleanup after closing (default 1).
#'
#' @return NULL (invisibly). Called for side effects.
#' @keywords internal
cleanup_chromote_session <- function(session, 
                                     timeout_before = 2, 
                                     timeout_after = 1) {
  tryCatch({
    if (requireNamespace("later", quietly = TRUE)) {
      later::run_now(timeoutSecs = timeout_before)
    }
    if (!is.null(session)) {
      session$close()
    }
    if (requireNamespace("later", quietly = TRUE)) {
      later::run_now(timeoutSecs = timeout_after)
    }
  }, error = function(e) NULL)
  
  invisible(NULL)
}


#' Start a new Chrome session safely
#'
#' @description
#' Creates a new ChromoteSession, optionally closing an existing one first.
#' Includes a small delay after closing to allow Chrome to clean up.
#'
#' @param old_session Previous session to close (can be NULL).
#'
#' @return New ChromoteSession object.
#' @keywords internal
start_chrome_session <- function(old_session = NULL) {
  if (!is.null(old_session)) {
    tryCatch(cleanup_chromote_session(old_session), error = function(e) NULL)
    Sys.sleep(0.3)
  }
  chromote::ChromoteSession$new()
}


#' Convert a single SVG using an existing Chrome session
#'
#' @description
#' Internal helper that renders an SVG to PNG using an existing Chrome session.
#' Uses fixed-time waiting instead of event-based waiting for reliability.
#'
#' @param b ChromoteSession object.
#' @param svg_content SVG content string (already sanitized).
#' @param scale DPI scale factor.
#' @param background Background color.
#' @param load_wait Seconds to wait for page load (default 0.5).
#'
#' @return Base64 encoded PNG string.
#' @keywords internal
convert_svg_with_session <- function(b, svg_content, scale, background, load_wait = 0.5) {
  svg_content <- sanitize_svg_metadata(svg_content)
  page <- write_svg_html_temp(svg_content, background = background)
  
  file_url <- paste0("file://", normalizePath(page$path, winslash = "/"))
  
  # Navigate without waiting for event (more reliable than loadEventFired)
  b$Page$navigate(file_url, wait_ = FALSE)
  
  # Fixed-time wait for page to load
  Sys.sleep(load_wait)
  
  # Set viewport
  b$Emulation$setDeviceMetricsOverride(
    width = as.integer(page$width),
    height = as.integer(page$height),
    deviceScaleFactor = scale,
    mobile = FALSE
  )
  
  # Small extra delay for rendering
  Sys.sleep(0.1)
  
  # Capture screenshot
  shot <- b$Page$captureScreenshot(format = "png", fromSurface = TRUE)
  
  shot$data
}

# ------------------------------------------------------------------------------
# Public API: Chrome-based conversion (single file)
# ------------------------------------------------------------------------------

#' Convert SVG to PNG using headless Chrome
#'
#' @description
#' Renders an SVG to PNG using headless Chrome via the chromote package.
#' This method provides superior font rendering compared to librsvg/ImageMagick,
#' as Chrome properly handles @font-face rules, web fonts, and CSS features.
#'
#' @param svg_input SVG string or path to an SVG file.
#' @param output_path Output path for the PNG file. If NULL, a temp file is used.
#' @param dpi Resolution in dots per inch (default 300). Chrome uses 96 DPI as base,
#'   so dpi = 300 results in approximately 3.125x scaling.
#' @param background Background color for the HTML page (default "transparent").
#'   Use "white", "#FFFFFF", etc. for a solid background.
#' @param load_wait Seconds to wait for page to load (default 0.5).
#'   Increase if fonts are not rendering correctly.
#'
#' @return Path to the generated PNG file.
#' @export
#'
#' @examples
#' svg <- svg_card("FAR", list(), list())
#' file_name <- tempfile(fileext = ".png")
#' \dontrun{
#' if (chrome_available()) {
#'   png_path <- svg_to_png_chrome(svg, file_name, dpi = 300)
#' }
#' }
svg_to_png_chrome <- function(svg_input,
                              output_path = NULL,
                              dpi = 300,
                              background = "transparent",
                              load_wait = 0.5) {
  
  if (!requireNamespace("chromote", quietly = TRUE)) {
    cli::cli_abort(c(
      "x" = "Package {.pkg chromote} is required.",
      "i" = "Install with: {.code install.packages('chromote')}"
    ))
  }
  if (!requireNamespace("base64enc", quietly = TRUE)) {
    cli::cli_abort(c(
      "x" = "Package {.pkg base64enc} is required.",
      "i" = "Install with: {.code install.packages('base64enc')}"
    ))
  }
  
  # Read SVG content
  if (is.character(svg_input) && length(svg_input) == 1 && file.exists(svg_input)) {
    svg_content <- paste(readLines(svg_input, warn = FALSE), collapse = "\n")
  } else {
    svg_content <- as.character(svg_input)
  }
  
  if (is.null(output_path)) {
    output_path <- tempfile(fileext = ".png")
  }
  ensure_output_dir(output_path)
  
  scale <- dpi / 96
  
  b <- chromote::ChromoteSession$new()
  on.exit(cleanup_chromote_session(b), add = TRUE)
  
  b64 <- convert_svg_with_session(b, svg_content, scale, background, load_wait)
  
  raw <- base64enc::base64decode(b64)
  writeBin(raw, output_path)
  
  output_path
}


#' Convert SVG to PDF using headless Chrome
#'
#' @description
#' Renders an SVG to PDF using headless Chrome via the chromote package.
#' This method produces vector PDFs with perfect font rendering.
#'
#' @param svg_input SVG string or path to an SVG file.
#' @param output_path Output path for the PDF file.
#' @param background Background color for the HTML page (default "transparent").
#' @param print_background Whether to include CSS backgrounds in PDF (default TRUE).
#' @param load_wait Seconds to wait for page to load (default 0.5).
#'
#' @return Path to the generated PDF file.
#' @export
#'
#' @examples
#' \dontrun{
#' svg <- svg_card("FAR", list(), list())
#' if (chrome_available()) {
#'   pdf_path <- svg_to_pdf_chrome(svg, tempfile(fileext = ".pdf"))
#' }
#' }
svg_to_pdf_chrome <- function(svg_input,
                              output_path,
                              background = "transparent",
                              print_background = TRUE,
                              load_wait = 0.5) {
  
  if (!requireNamespace("chromote", quietly = TRUE)) {
    cli::cli_abort(c(
      "x" = "Package {.pkg chromote} is required.",
      "i" = "Install with: {.code install.packages('chromote')}"
    ))
  }
  if (!requireNamespace("base64enc", quietly = TRUE)) {
    cli::cli_abort(c(
      "x" = "Package {.pkg base64enc} is required.",
      "i" = "Install with: {.code install.packages('base64enc')}"
    ))
  }
  
  # Read SVG content
  if (is.character(svg_input) && length(svg_input) == 1 && file.exists(svg_input)) {
    svg_content <- paste(readLines(svg_input, warn = FALSE), collapse = "\n")
  } else {
    svg_content <- as.character(svg_input)
  }
  
  svg_content <- sanitize_svg_metadata(svg_content)
  ensure_output_dir(output_path)
  
  page <- write_svg_html_temp(svg_content, background = background)
  
  b <- chromote::ChromoteSession$new()
  on.exit(cleanup_chromote_session(b), add = TRUE)
  
  file_url <- paste0("file://", normalizePath(page$path, winslash = "/"))
  
  # Navigate without waiting for event
  b$Page$navigate(file_url, wait_ = FALSE)
  Sys.sleep(load_wait)
  
  # Generate PDF
  pdf <- b$Page$printToPDF(
    printBackground = print_background,
    marginTop = 0,
    marginBottom = 0,
    marginLeft = 0,
    marginRight = 0,
    paperWidth = page$width / 96,
    paperHeight = page$height / 96,
    preferCSSPageSize = TRUE
  )
  
  raw <- base64enc::base64decode(pdf$data)
  writeBin(raw, output_path)
  
  output_path
}

# ------------------------------------------------------------------------------
# Public API: Batch conversion
# ------------------------------------------------------------------------------

#' Batch convert SVGs to PNG base64 using headless Chrome
#'
#' @description
#' Converts multiple SVGs to base64-encoded PNG strings using a single
#' Chrome session. Much faster than calling svg_to_png_chrome() repeatedly.
#'
#' @param svg_list List of SVG strings or file paths.
#' @param dpi Resolution (default 300).
#' @param background Background color (default "transparent").
#' @param load_wait Seconds to wait for each page to load (default 0.5).
#'   Increase if conversions are failing.
#' @param restart_every Restart Chrome session every N conversions (default 50).
#'   Helps prevent memory issues and stale connections.
#' @param retry_attempts Number of retry attempts on failure (default 3).
#' @param progress Show progress bar (default TRUE).
#'
#' @return Character vector of base64-encoded PNGs (data URI format).
#'   Returns NA for failed conversions.
#' @export
batch_svg_to_base64_chrome <- function(svg_list,
                                       dpi = 300,
                                       background = "transparent",
                                       load_wait = 0.5,
                                       restart_every = 50,
                                       retry_attempts = 3,
                                       progress = TRUE) {
  
  if (!requireNamespace("chromote", quietly = TRUE)) {
    cli::cli_abort(c(
      "x" = "Package {.pkg chromote} is required.",
      "i" = "Install with: {.code install.packages('chromote')}"
    ))
  }
  
  n <- length(svg_list)
  results <- rep(NA_character_, n)
  scale <- dpi / 96
  
  if (progress) cli::cli_progress_bar("Converting SVGs", total = n)
  
  b <- start_chrome_session(NULL)
  
  on.exit({
    tryCatch(cleanup_chromote_session(b), error = function(e) NULL)
  }, add = TRUE)
  
  for (i in seq_len(n)) {
    # Restart session periodically
    if (i > 1 && ((i - 1) %% restart_every == 0)) {
      if (progress) cli::cli_alert_info("Restarting Chrome session at item {i}...")
      b <- start_chrome_session(b)
    }
    
    svg_input <- svg_list[[i]]
    success <- FALSE
    attempt <- 0
    current_wait <- load_wait
    
    # Read SVG content once
    if (is.character(svg_input) && length(svg_input) == 1 && file.exists(svg_input)) {
      svg_content <- paste(readLines(svg_input, warn = FALSE), collapse = "\n")
    } else {
      svg_content <- as.character(svg_input)
    }
    
    while (!success && attempt < retry_attempts) {
      attempt <- attempt + 1
      
      result <- tryCatch({
        b64 <- convert_svg_with_session(b, svg_content, scale, background, current_wait)
        list(success = TRUE, data = paste0("data:image/png;base64,", b64))
      }, error = function(e) {
        list(success = FALSE, error = e$message)
      })
      
      if (result$success) {
        results[[i]] <- result$data
        success <- TRUE
      } else {
        if (attempt < retry_attempts) {
          if (progress) {
            cli::cli_alert_warning(
              "Item {i} failed (attempt {attempt}/{retry_attempts}). Restarting session..."
            )
          }
          b <- start_chrome_session(b)
          current_wait <- current_wait + 0.3  # Increase wait time on retry
        } else {
          if (progress) {
            cli::cli_alert_danger("Item {i} failed: {result$error}")
          }
        }
      }
    }
    
    if (progress) cli::cli_progress_update()
  }
  
  if (progress) {
    cli::cli_progress_done()
    n_success <- sum(!is.na(results))
    n_failed <- n - n_success
    if (n_failed > 0) {
      cli::cli_alert_warning("Completed: {n_success} successful, {n_failed} failed")
    } else {
      cli::cli_alert_success("All {n} conversions completed successfully")
    }
  }
  
  results
}


#' Batch convert SVGs to PNG files using headless Chrome
#'
#' @description
#' Converts multiple SVGs to PNG files using a single Chrome session.
#' Much faster than calling svg_to_png_chrome() repeatedly.
#'
#' @param svg_list List of SVG strings or file paths.
#' @param output_paths Character vector of output paths. If NULL, temp files are created.
#' @param dpi Resolution (default 300).
#' @param background Background color (default "transparent").
#' @param load_wait Seconds to wait for each page to load (default 0.5).
#'   Increase if conversions are failing.
#' @param restart_every Restart Chrome session every N conversions (default 50).
#' @param retry_attempts Number of retry attempts on failure (default 3).
#' @param progress Show progress bar (default TRUE).
#'
#' @return Character vector of output file paths. Returns NA for failed conversions.
#' @export
batch_svg_to_png_chrome <- function(svg_list,
                                    output_paths = NULL,
                                    dpi = 300,
                                    background = "transparent",
                                    load_wait = 0.5,
                                    restart_every = 50,
                                    retry_attempts = 3,
                                    progress = TRUE) {
  
  if (!requireNamespace("chromote", quietly = TRUE)) {
    cli::cli_abort(c(
      "x" = "Package {.pkg chromote} is required.",
      "i" = "Install with: {.code install.packages('chromote')}"
    ))
  }
  if (!requireNamespace("base64enc", quietly = TRUE)) {
    cli::cli_abort(c(
      "x" = "Package {.pkg base64enc} is required.",
      "i" = "Install with: {.code install.packages('base64enc')}"
    ))
  }
  
  n <- length(svg_list)
  
  if (is.null(output_paths)) {
    output_paths <- vapply(seq_len(n), function(i) tempfile(fileext = ".png"), character(1))
  }
  
  if (length(output_paths) != n) {
    cli::cli_abort("Length of output_paths must match length of svg_list.")
  }
  
  results <- rep(NA_character_, n)
  scale <- dpi / 96
  
  if (progress) cli::cli_progress_bar("Converting SVGs", total = n)
  
  b <- start_chrome_session(NULL)
  
  on.exit({
    tryCatch(cleanup_chromote_session(b), error = function(e) NULL)
  }, add = TRUE)
  
  for (i in seq_len(n)) {
    # Restart session periodically
    if (i > 1 && ((i - 1) %% restart_every == 0)) {
      if (progress) cli::cli_alert_info("Restarting Chrome session at item {i}...")
      b <- start_chrome_session(b)
    }
    
    svg_input <- svg_list[[i]]
    output_path <- output_paths[[i]]
    success <- FALSE
    attempt <- 0
    current_wait <- load_wait
    
    # Read SVG content once
    if (is.character(svg_input) && length(svg_input) == 1 && file.exists(svg_input)) {
      svg_content <- paste(readLines(svg_input, warn = FALSE), collapse = "\n")
    } else {
      svg_content <- as.character(svg_input)
    }
    
    while (!success && attempt < retry_attempts) {
      attempt <- attempt + 1
      
      result <- tryCatch({
        ensure_output_dir(output_path)
        b64 <- convert_svg_with_session(b, svg_content, scale, background, current_wait)
        raw <- base64enc::base64decode(b64)
        writeBin(raw, output_path)
        list(success = TRUE, path = output_path)
      }, error = function(e) {
        list(success = FALSE, error = e$message)
      })
      
      if (result$success) {
        results[[i]] <- result$path
        success <- TRUE
      } else {
        if (attempt < retry_attempts) {
          if (progress) {
            cli::cli_alert_warning(
              "Item {i} failed (attempt {attempt}/{retry_attempts}). Restarting session..."
            )
          }
          b <- start_chrome_session(b)
          current_wait <- current_wait + 0.3
        } else {
          if (progress) {
            cli::cli_alert_danger("Item {i} failed: {result$error}")
          }
        }
      }
    }
    
    if (progress) cli::cli_progress_update()
  }
  
  if (progress) {
    cli::cli_progress_done()
    n_success <- sum(!is.na(results))
    n_failed <- n - n_success
    if (n_failed > 0) {
      cli::cli_alert_warning("Completed: {n_success} successful, {n_failed} failed")
    } else {
      cli::cli_alert_success("All {n} conversions completed successfully")
    }
  }
  
  results
}

# badges.R - SVG badge generation functions

#' Create an SVG Badge
#'
#' Generate a badge similar to shields.io style with label and value.
#' Both sides have properly rounded corners.
#'
#' @param label Label text (left side)
#' @param value Value text (right side)
#' @param color Background color for value area
#' @param font Font family name
#' @param style CSS style string
#' @param fontsize Font size in pixels
#' @param horiz_padding Horizontal padding
#' @param extra_right_pad Extra padding on right side
#' @param class CSS class
#' @param shadow_offset Shadow offset in pixels
#' @param corner_radius Corner radius for rounded rectangle
#' @param height Minimum height (optional)
#' @param as_string Return as character string
#' @return SVG string
#' @export
#' @examples
#' create_badge("UH", "192", "white")
#' create_badge("Recurso Federal", "36,4 milhões", "#4CAF50")
create_badge <- function(label, value, color,
                         font = "Jost", style = "margin:2px;",
                         fontsize = 11,
                         horiz_padding = 5,
                         extra_right_pad = 2,
                         class = "",
                         shadow_offset = 2,
                         corner_radius = 3,
                         height = NULL,
                         as_string = TRUE) {
  
  # Generate unique ID
  id_suffix <- generate_id(label, value, color)
  
  # Measure text dimensions
  metrics_label <- gdtools::str_metrics(label, fontname = font, fontsize = fontsize)
  metrics_value <- gdtools::str_metrics(value, fontname = font, fontsize = fontsize)
  
  width_label <- as.numeric(metrics_label["width"])
  width_value <- as.numeric(metrics_value["width"])
  
  # Define widths
  left_width <- width_label + 2 * horiz_padding
  right_width <- width_value + 2 * horiz_padding + 2 * extra_right_pad
  total_width <- left_width + right_width
  
  # Calculate heights
  height_label <- as.numeric(metrics_label["ascent"]) + as.numeric(metrics_label["descent"])
  height_value <- as.numeric(metrics_value["ascent"]) + as.numeric(metrics_value["descent"])
  current_height <- max(height_label, height_value)
  
  # if (!is.null(height)) {
  #   current_height <- max(height, current_height, na.rm = TRUE)
  # }
  # 
  # final_height <- current_height * 2
  
  if (!is.null(height)) {
    final_height <- height * 2  # Usar altura especificada diretamente
  } else {
    final_height <- current_height * 2
  }
  
  # Text positions (exactly as original)
  left_text_x <- horiz_padding + width_label / 2
  right_text_x <- left_width + horiz_padding + extra_right_pad + width_value / 2
  text_y <- final_height * 0.55
  
  # Determine text colors based on background.
  # For gradients, col2rgb() can't parse the CSS string, so judge lightness
  # from the gradient's first hex stop (fallback: assume a dark background).
  is_grad <- grepl("gradient\\(", color, ignore.case = TRUE)
  bg_is_light <- if (is_grad) {
    hexes <- regmatches(color, gregexpr("#[0-9A-Fa-f]{3,8}", color))[[1]]
    if (length(hexes)) is_light_color(hexes[1]) else FALSE
  } else {
    is_light_color(color)
  }
  if (bg_is_light) {
    value_text_shadow <- "#ccc"
    value_text_main <- "#333"
  } else {
    value_text_shadow <- "#010101"
    value_text_main <- "#fff"
  }

  # Handle gradient colors
  if (is_grad) {
    grad_id <- paste0("grad_", id_suffix)
    svg_grad <- css_gradient_to_svg(color, id = grad_id)
    fill_color <- sprintf("url(#%s)", grad_id)
    defs_block <- sprintf("<defs>\n%s\n</defs>", svg_grad)
  } else {
    fill_color <- color
    defs_block <- ""
  }
  
  # Build SVG (using clipPath for rounded corners on both sides)
  svg <- sprintf(
    '<svg class="%s" style="%s" xmlns="http://www.w3.org/2000/svg" width="%d" height="%d" role="img" aria-label="%s: %s">
      <title>%s: %s</title>
      %s
      <linearGradient id="s%s" x2="0" y2="100%%">
        <stop offset="0" stop-color="#bbb" stop-opacity=".1"/>
        <stop offset="1" stop-opacity=".1"/>
      </linearGradient>
      <clipPath id="r%s">
        <rect width="%d" height="%d" rx="%d" ry="%d" fill="#fff"/>
      </clipPath>
      <g clip-path="url(#r%s)">
        <rect width="%d" height="%d" fill="#555"/>
        <rect x="%d" width="%d" height="%d" fill="%s"/>
        <rect width="%d" height="%d" fill="url(#s%s)"/>
      </g>
      <g fill="#fff" text-anchor="middle" font-family="%s" font-size="%s" dominant-baseline="middle">
        <text x="%.1f" y="%.1f" fill="#010101" text-rendering="geometricPrecision" alignment-baseline="middle" fill-opacity=".3">%s</text>
        <text x="%.1f" y="%.1f" text-rendering="geometricPrecision" alignment-baseline="middle" fill="#fff">%s</text>
        <text x="%.1f" y="%.1f" text-rendering="geometricPrecision" alignment-baseline="middle" fill="%s" fill-opacity=".3">%s</text>
        <text x="%.1f" y="%.1f" text-rendering="geometricPrecision" alignment-baseline="middle" fill="%s">%s</text>
      </g>
    </svg>',
    class, style,
    round(total_width), round(final_height),
    escape_xml(label), escape_xml(value),
    escape_xml(label), escape_xml(value),
    defs_block,
    id_suffix,
    id_suffix,
    round(total_width), round(final_height), corner_radius, corner_radius,
    id_suffix,
    round(total_width), round(final_height),
    round(left_width), round(right_width), round(final_height), fill_color,
    round(total_width), round(final_height),
    id_suffix,
    font, as.character(fontsize),
    left_text_x + shadow_offset, text_y + shadow_offset, escape_xml(label),
    left_text_x, text_y, escape_xml(label),
    right_text_x + shadow_offset, text_y + shadow_offset, value_text_shadow, escape_xml(value),
    right_text_x, text_y, value_text_main, escape_xml(value)
  )
  
  if (as_string) {
    return(svg)
  } else {
    return(list(svg = svg, width = round(total_width), height = round(final_height)))
  }
}

#' Create a row of SVG badges with uniform height
#'
#' Generate multiple badges arranged horizontally with the same height.
#'
#' @param badges_data A list of lists, each containing label, value, and optionally color
#' @param default_color Default color for badges
#' @param spacing Spacing between badges
#' @param font Font family
#' @param fontsize Font size
#' @param uniform_height Force all badges to have the same height (default TRUE)
#' @return SVG string containing all badges
#' @export
#' @examples
#' badges <- list(
#'   list(label = "UH", value = "192"),
#'   list(label = "Recurso Federal", value = "36,4 milhões"),
#'   list(label = "Contrapartida", value = "0,0")
#' )
#' create_badge_row(badges, default_color = "white")
create_badge_row <- function(badges_data, 
                             default_color = "white",
                             spacing = 4,
                             font = "Jost",
                             fontsize = 10,
                             uniform_height = TRUE) {
  
  if (length(badges_data) == 0) {
    return('<svg xmlns="http://www.w3.org/2000/svg" width="0" height="0"></svg>')
  }
  
  # First pass: calculate all dimensions to find max height
  badge_dims <- list()
  max_height <- 0
  
  for (i in seq_along(badges_data)) {
    badge <- badges_data[[i]]
    
    # Measure text dimensions
    metrics_label <- gdtools::str_metrics(badge$label, fontname = font, fontsize = fontsize)
    metrics_value <- gdtools::str_metrics(badge$value, fontname = font, fontsize = fontsize)
    
    height_label <- as.numeric(metrics_label["ascent"]) + as.numeric(metrics_label["descent"])
    height_value <- as.numeric(metrics_value["ascent"]) + as.numeric(metrics_value["descent"])
    current_height <- max(height_label, height_value) * 2
    
    if (current_height > max_height) {
      max_height <- current_height
    }
    
    badge_dims[[i]] <- list(
      label = badge$label,
      value = badge$value,
      color = if (!is.null(badge$color)) badge$color else default_color
    )
  }
  
  # Second pass: generate badges with uniform height
  badge_svgs <- list()
  badge_widths <- numeric()
  
  for (i in seq_along(badges_data)) {
    bd <- badge_dims[[i]]
    
    # Use max_height / 2 as the height parameter since badge_svg doubles it
    target_height <- if (uniform_height) max_height / 2 else NULL
    
    result <- create_badge(
      label = bd$label,
      value = bd$value,
      color = bd$color,
      font = font,
      fontsize = fontsize,
      style = "",
      height = target_height,
      as_string = FALSE
    )
    
    badge_svgs[[i]] <- result$svg
    badge_widths[i] <- result$width
  }
  
  # Calculate total dimensions
  total_width <- sum(badge_widths) + (length(badges_data) - 1) * spacing
  total_height <- max_height
  
  # Position badges
  x_positions <- numeric(length(badges_data))
  current_x <- 0
  for (i in seq_along(badges_data)) {
    x_positions[i] <- current_x
    current_x <- current_x + badge_widths[i] + spacing
  }
  
  # Build combined SVG
  inner_svgs <- ""
  for (i in seq_along(badges_data)) {
    # Extract inner content from each badge SVG
    badge_content <- gsub('<svg[^>]*>', '', badge_svgs[[i]])
    badge_content <- gsub('</svg>', '', badge_content)
    
    inner_svgs <- paste0(inner_svgs, sprintf(
      '<g transform="translate(%.1f, 0)">%s</g>',
      x_positions[i], badge_content
    ))
  }
  
  svg <- sprintf(
    '<svg xmlns="http://www.w3.org/2000/svg" width="%d" height="%d">%s</svg>',
    round(total_width), round(total_height), inner_svgs
  )
  
  return(svg)
}

#' Convert CSS gradient to SVG gradient
#'
#' @param css_gradient CSS gradient string
#' @param id ID for the gradient
#' @return SVG gradient definition
#' @keywords internal
css_gradient_to_svg <- function(css_gradient, id = "grad") {

  # Linear gradient parser supporting directional keywords and angles
  # Handles: "linear-gradient(to right, #color1, #color2)"
  #          "linear-gradient(135deg, #color1, #color2)"
  
  # Check for angle in degrees (e.g., "45deg", "135deg")
  angle_match <- regmatches(css_gradient, regexpr("(\\d+)deg", css_gradient))
  
  if (length(angle_match) > 0 && nchar(angle_match) > 0) {
    # Extract angle value and convert to SVG coordinates
    angle <- as.numeric(gsub("deg", "", angle_match))
    # CSS angles: 0deg = to top, 90deg = to right, 180deg = to bottom
    # SVG uses coordinates, so we need to convert
    rad <- (angle - 90) * pi / 180
    x1 <- sprintf("%.0f%%", 50 - 50 * cos(rad))
    y1 <- sprintf("%.0f%%", 50 - 50 * sin(rad))
    x2 <- sprintf("%.0f%%", 50 + 50 * cos(rad))
    y2 <- sprintf("%.0f%%", 50 + 50 * sin(rad))
  } else if (grepl("to right", css_gradient, ignore.case = TRUE)) {
    x1 <- "0%"; y1 <- "0%"; x2 <- "100%"; y2 <- "0%"
  } else if (grepl("to left", css_gradient, ignore.case = TRUE)) {
    x1 <- "100%"; y1 <- "0%"; x2 <- "0%"; y2 <- "0%"
  } else if (grepl("to bottom", css_gradient, ignore.case = TRUE)) {
    x1 <- "0%"; y1 <- "0%"; x2 <- "0%"; y2 <- "100%"
  } else if (grepl("to top", css_gradient, ignore.case = TRUE)) {
    x1 <- "0%"; y1 <- "100%"; x2 <- "0%"; y2 <- "0%"
  } else {
    # Default: top to bottom
    x1 <- "0%"; y1 <- "0%"; x2 <- "0%"; y2 <- "100%"
  }
  
  # Extract colors (hex codes)
  colors <- regmatches(css_gradient, gregexpr("#[0-9A-Fa-f]{3,8}", css_gradient))[[1]]
  if (length(colors) == 0) {
    colors <- c("#ffffff", "#000000")
  } else if (length(colors) == 1) {
    # Single color: emit a solid gradient of that color instead of inventing black
    colors <- c(colors, colors)
  }
  
  stops <- ""
  for (i in seq_along(colors)) {
    offset <- (i - 1) / (length(colors) - 1) * 100
    stops <- paste0(stops, sprintf(
      '<stop offset="%d%%" stop-color="%s"/>',
      round(offset), colors[i]
    ))
  }
  
  sprintf(
    '<linearGradient id="%s" x1="%s" y1="%s" x2="%s" y2="%s">%s</linearGradient>',
    id, x1, y1, x2, y2, stops
  )
}

test_that("css_gradient_to_svg parses direction keywords", {
  out <- cardargus:::css_gradient_to_svg("linear-gradient(to right, #1a5a3a, #2e7d32)")
  expect_match(out, "<linearGradient")
  expect_match(out, 'x1="0%"')
  expect_match(out, 'x2="100%"')
  expect_match(out, "#1a5a3a")
  expect_match(out, "#2e7d32")
})

test_that("css_gradient_to_svg handles degree angles", {
  out <- cardargus:::css_gradient_to_svg("linear-gradient(135deg, #667eea, #764ba2)")
  expect_match(out, "<linearGradient")
  expect_match(out, "#667eea")
})

test_that("css_gradient_to_svg keeps a single color instead of inventing black (issue #4)", {
  out <- cardargus:::css_gradient_to_svg("linear-gradient(to right, #abcdef)")
  expect_match(out, "#abcdef")
  expect_false(grepl("#000000", out))
})

test_that("css_gradient_to_svg falls back when no colors are present", {
  out <- cardargus:::css_gradient_to_svg("linear-gradient(to right)")
  expect_match(out, "#ffffff")
  expect_match(out, "#000000")
})

test_that("create_badge accepts gradient background colors (issue #14)", {
  expect_no_error(b1 <- create_badge("Test", "OK", "linear-gradient(to right, #4CAF50)"))
  expect_match(b1, "#4CAF50")

  expect_no_error(b2 <- create_badge("Test", "OK", "linear-gradient(135deg, #1a5a3a, #2e7d32)"))
  expect_match(b2, "linearGradient")
  expect_match(b2, "#2e7d32")
})

test_that("create_badge works with solid colors", {
  b <- create_badge("UH", "192", "white")
  expect_match(b, "<svg")
  expect_match(b, "192")
})

# knitr.R - Knitr and Quarto integration
#
# Functions to display cardargus cards in R Markdown and Quarto documents.

#' Display card in knitr/Quarto document (SVG via data URI)
#'
#' @description
#' Embeds the SVG as an `<img>` using a `data:image/svg+xml;base64,...` URI.
#' This is more robust than inline `<svg>` for Pandoc/pkgdown (avoids "unclosed div"
#' warnings). For non-HTML outputs, it falls back to `include_card_png()`.
#'
#' @param svg_string SVG string from `svg_card()`.
#' @param width Display width (CSS units), e.g. `"100%"`, `"500px"`.
#' @param alt Alternative text for accessibility.
#' @param dpi Fallback DPI used when output is not HTML (default 300).
#' @param engine Rendering engine for non-HTML output: `"auto"`, `"chrome"`, or `"rsvg"`.
#' @return Knitr output for the current format.
#' @export
include_card <- function(svg_string,
                         width = "100%",
                         alt = "Card generated by cardargus",
                         dpi = 300,
                         engine = c("auto", "chrome", "rsvg")) {
  if (!requireNamespace("knitr", quietly = TRUE)) {
    cli::cli_abort("Package {.pkg knitr} is required.")
  }
  if (!requireNamespace("base64enc", quietly = TRUE)) {
    cli::cli_abort(c(
      "x" = "Package {.pkg base64enc} is required.",
      "i" = "Install with: {.code install.packages('base64enc')}"
    ))
  }
  
  engine <- match.arg(engine)
  
  # Non-HTML output: SVG-in-IMG won't work reliably -> use PNG
  if (!knitr::is_html_output()) {
    return(include_card_png(svg_string, dpi = dpi, width = width, alt = alt, engine = engine))
  }
  
  # Sanitize to avoid issues with strict parsers / odd metadata
  svg <- sanitize_svg_for_raster(svg_string)
  
  # Ensure UTF-8 bytes for base64
  svg_raw <- charToRaw(enc2utf8(svg))
  svg_b64 <- base64enc::base64encode(svg_raw)
  data_uri <- paste0("data:image/svg+xml;base64,", svg_b64)
  
  html <- sprintf(
    '<div class="cardargus-card" style="width:%s; max-width:100%%;">
  <img src="%s" alt="%s" style="width:100%%; height:auto;" />
</div>',
    width, data_uri, alt
  )
  
  # Blank lines help Pandoc treat it as a raw HTML block
  knitr::asis_output(paste0("\n\n", html, "\n\n"))
}

#' Display card as PNG in knitr/Quarto document
#'
#' Converts an SVG card to PNG and displays it in R Markdown or Quarto documents.
#' Recommended for better compatibility across HTML/PDF/Word.
#'
#' @param svg_string SVG string from `svg_card()`.
#' @param dpi Rasterization DPI (default 300).
#' @param width Display width (CSS units) for HTML outputs.
#' @param alt Alternative text for accessibility.
#' @param background Background color passed to `svg_to_png()` (default transparent).
#' @param engine Rendering engine: `"auto"` (uses Chrome if available, else rsvg),
#'   `"chrome"` (headless Chrome via chromote), or `"rsvg"` (librsvg/magick).
#'   Chrome provides better font rendering for Google Fonts.
#' @return Knitr output for the current format.
#' @export
include_card_png <- function(svg_string,
                             dpi = 300,
                             width = "100%",
                             alt = "Card generated by cardargus",
                             background = "transparent",
                             engine = c("auto", "chrome", "rsvg")) {
  if (!requireNamespace("knitr", quietly = TRUE)) {
    cli::cli_abort("Package {.pkg knitr} is required.")
  }
  if (!requireNamespace("base64enc", quietly = TRUE)) {
    cli::cli_abort(c(
      "x" = "Package {.pkg base64enc} is required.",
      "i" = "Install with: {.code install.packages('base64enc')}"
    ))
  }
  
  engine <- match.arg(engine)
  
  # Always rasterize via package converter (sanitizes + DPI scaling)
  png_path <- save_card_for_knitr(
    svg_string, 
    filename = "cardargus_card", 
    format = "png", 
    dpi = dpi,
    engine = engine
  )
  
  if (knitr::is_html_output()) {
    # Embed as data URI (portable for pkgdown/GitHub)
    png_raw <- readBin(png_path, "raw", n = file.info(png_path)$size)
    png_b64 <- base64enc::base64encode(png_raw)
    data_uri <- paste0("data:image/png;base64,", png_b64)
    
    html <- sprintf(
      '<div class="cardargus-card" style="width:%s; max-width:100%%;">
  <img src="%s" alt="%s" style="width:100%%; height:auto;" />
</div>',
      width, data_uri, alt
    )
    
    return(knitr::asis_output(paste0("\n\n", html, "\n\n")))
  }
  
  # Non-HTML output: write a real file and include it
  knitr::include_graphics(png_path)
}

#' Save card and return path for knitr
#'
#' Saves a card to a file and returns the path for use in knitr chunks.
#'
#' @param svg_string SVG string from `svg_card()`.
#' @param filename Output filename (without extension).
#' @param format Output format: `"svg"` or `"png"`.
#' @param dpi Rasterization DPI for PNG (default 300).
#' @param dir Output directory (defaults to knitr figure directory or tempdir()).
#' @param engine Rendering engine for PNG: `"auto"`, `"chrome"`, or `"rsvg"`.
#' @return Path to the saved file.
#' @export
save_card_for_knitr <- function(svg_string,
                                filename = "card",
                                format = c("svg", "png"),
                                dpi = 300,
                                dir = NULL,
                                engine = c("auto", "chrome", "rsvg")) {
  format <- match.arg(format)
  engine <- match.arg(engine)
  
  if (is.null(dir)) {
    if (requireNamespace("knitr", quietly = TRUE)) {
      fig_path <- knitr::opts_chunk$get("fig.path")
      if (!is.null(fig_path) && nzchar(fig_path)) {
        dir <- dirname(fig_path)
      } else {
        dir <- tempdir()
      }
    } else {
      dir <- tempdir()
    }
  }
  
  if (!dir.exists(dir)) dir.create(dir, recursive = TRUE)
  
  output_path <- file.path(dir, paste0(filename, ".", format))
  
  if (format == "svg") {
    save_svg(svg_string, output_path)
  } else {
    # Determine which engine to use
    use_chrome <- FALSE
    
    if (engine == "chrome") {
      if (chrome_available()) {
        use_chrome <- TRUE
      } else {
        cli::cli_warn("Chrome not available, falling back to rsvg.")
      }
    } else if (engine == "auto") {
      # Auto: prefer Chrome if available (better font rendering)
      use_chrome <- chrome_available()
    }
    
    if (use_chrome) {
      svg_to_png_chrome(svg_string, output_path, dpi = dpi, background = "transparent")
    } else {
      svg_to_png(svg_string, output_path, dpi = dpi, background = "transparent")
    }
  }
  
  output_path
}

#' Register cardargus knitr engine
#'
#' Registers a custom knitr engine named `"cardargus"` so that SVG cards can be
#' rendered directly from chunks.
#'
#' @return Invisible NULL.
#' @export
#' @examples
#' # In your setup chunk:
#' register_cardargus_knitr()
#' 
#' # Then use cardargus as chunk engine:
#' # ```{cardargus}
#' # svg_card(title = "My Card", ...)
#' # ```
register_cardargus_knitr <- function() {
  if (!requireNamespace("knitr", quietly = TRUE)) {
    cli::cli_abort(c(
      "x" = "Package {.pkg knitr} is required.",
      "i" = "Install with: {.code install.packages('knitr')}"
    ))
  }
  
  knitr::knit_engines$set(cardargus = function(options) {
    code <- paste(options$code, collapse = "\n")
    svg_result <- eval(parse(text = code), envir = knitr::knit_global())
    
    if (is.character(svg_result) && grepl("^<\\?xml|^<svg", svg_result)) {
      out_width <- if (!is.null(options$out.width)) options$out.width else "100%"
      knitr::knit_print(include_card_png(svg_result, width = out_width))
    } else {
      knitr::engine_output(options, code, svg_result)
    }
  })
  
  invisible(NULL)
}

#' Create a grob for grid/ggplot2
#'
#' Wrap an SVG card as a raster grob so it can be used with grid graphics.
#'
#' @param svg_string SVG string from `svg_card()`.
#' @param dpi Rasterization DPI (default 150).
#' @param engine Rendering engine: `"auto"`, `"chrome"`, or `"rsvg"`.
#' @return A `grid::rasterGrob` object.
#' @export
card_to_grob <- function(svg_string, dpi = 150, engine = c("auto", "chrome", "rsvg")) {
  if (!requireNamespace("grid", quietly = TRUE)) {
    cli::cli_abort("Package {.pkg grid} is required.")
  }
  if (!requireNamespace("magick", quietly = TRUE)) {
    cli::cli_abort(c(
      "x" = "Package {.pkg magick} is required.",
      "i" = "Install with: {.code install.packages('magick')}"
    ))
  }
  
  engine <- match.arg(engine)
  
  # Determine which engine to use
  use_chrome <- FALSE
  if (engine == "chrome") {
    if (chrome_available()) {
      use_chrome <- TRUE
    } else {
      cli::cli_warn("Chrome not available, falling back to rsvg.")
    }
  } else if (engine == "auto") {
    use_chrome <- chrome_available()
  }
  
  # Rasterize
  tmp_png <- tempfile(fileext = ".png")
  if (use_chrome) {
    svg_to_png_chrome(svg_string, tmp_png, dpi = dpi, background = "transparent")
  } else {
    svg_to_png(svg_string, tmp_png, dpi = dpi, background = "transparent")
  }
  
  img <- magick::image_read(tmp_png)
  raster_data <- as.raster(img)
  
  grid::rasterGrob(raster_data, interpolate = TRUE)
}

# cards.R - SVG card generation

#' Create an Information Card in SVG Format
#'
#' Generate a complete information card as an SVG with embedded styles,
#' fonts, badges, logos, and field labels/values.
#'
#' @param title Card title (e.g., "FAR", "FNHIS")
#' @param badges_data List of badge data (label, value, color)
#' @param fields List of field rows, each row is a list of fields with label, value, 
#'   and optionally with_icon. The with_icon parameter can be:
#'   \itemize{
#'     \item TRUE - uses the default house icon
#'     \item FALSE or NULL - no icon
#'     \item A character string - path to an SVG file or raw SVG code
#'   }
#' @param bg_color Background color of the card. Can be a solid color (e.g., "#fab255") 
#'   or a CSS gradient (e.g., "linear-gradient(to right, #1a5a3a, #2e7d32)" or 
#'   "linear-gradient(135deg, #667eea, #764ba2)").
#' @param width Card width in pixels
#' @param padding Padding inside the card
#' @param corner_radius Corner radius for rounded corners
#' @param font Font family
#' @param title_fontsize Title font size
#' @param title_color Color for the card title (default "white")
#' @param label_fontsize Label font size
#' @param value_fontsize Value font size
#' @param label_color Color for field labels (default "white")
#' @param value_bg_color Background color for value boxes
#' @param value_text_color Text color for values
#' @param show_house_icon Show house icon next to empreendimento
#' @param logos Character vector of logo file paths or SVG strings for top right.
#'   Use `get_svg_path("filename.svg")` for bundled logos, or any local path.
#' @param logos_height Height for top-right logos (default 40)
#' @param bottom_logos Character vector of logo file paths or SVG strings for bottom left.
#' @param bottom_logos_height Height for bottom-left logos (default 30)
#' @param gap_to_footer Distance (px) between the last info block and the footer row.
#' @param footer_row_padding_bottom Bottom padding (px) under the footer row.
#' @param footer Footer text (e.g., update timestamp)
#' @param footer_fontsize Footer font size
#' @param footer_color Color for footer text (default "white")
#' @param uniform_row_height If TRUE, keep the height inside a row.
#' @param show_viewer If TRUE (and interactive), preview the SVG in the Viewer.
#' @return SVG string
#' @export
#' @examples
#' # With default house icon
#' fields <- list(
#'   list(
#'     list(label = "Empreendimento", value = "CAIARA II", with_icon = TRUE)
#'   )
#' )
#' 
#' # With custom icon
#' custom_icon <- '<svg width="50" height="50"><circle cx="25" cy="25" r="20" fill="white"/></svg>'
#' fields <- list(
#'   list(
#'     list(label = "Projeto", value = "Meu Projeto", with_icon = custom_icon)
#'   )
#' )
#' 
#' badges <- list(
#'   list(label = "UH", value = "192"),
#'   list(label = "Recurso Federal", value = "36,4 milhões")
#' )
#' 
#' # With file paths for logos
#' svg_card("FAR", badges, fields, 
#'          bg_color = "#fab255",
#'          logos = c("path/to/logo1.svg", "path/to/logo2.svg"),
#'          bottom_logos = c("path/to/gov_logo.svg"))
#' 
#' # With gradient background
#' svg_card("MCMV", badges, fields,
#'          bg_color = "linear-gradient(to right, #1a5a3a, #2e7d32)")
#' 
#' # Diagonal gradient
#' svg_card("Programa", badges, fields,
#'          bg_color = "linear-gradient(135deg, #667eea, #764ba2)")
svg_card <- function(title = "FAR",
                     badges_data = list(),
                     fields = list(),
                     bg_color = "#fab255",
                     width = 500,
                     padding = 20,
                     corner_radius = 8,
                     font = "Jost",
                     title_fontsize = 16,
                     title_color = "white",
                     label_fontsize = 11,
                     value_fontsize = 11,
                     label_color = "white",
                     value_bg_color = "#f8f8ff",
                     value_text_color = "#212529",
                     show_house_icon = TRUE,
                     logos = list(),
                     logos_height = 40,
                     bottom_logos = list(),
                     bottom_logos_height = 30,
                     footer = NULL,
                     gap_to_footer = 6,
                     footer_row_padding_bottom = 6,
                     footer_fontsize = 8,
                     footer_color = "white",
                     uniform_row_height = TRUE,
                     show_viewer = interactive()
                     ) {
  
  # Generate unique ID for this card
  card_id <- generate_id(title, bg_color, width)
  
  # ===== GRADIENT BACKGROUND SUPPORT =====
  # Detect if bg_color is a CSS gradient and convert to SVG

  if (grepl("gradient\\(", bg_color, ignore.case = TRUE)) {
    bg_grad_id <- paste0("bg_grad_", card_id)
    bg_gradient_def <- css_gradient_to_svg(bg_color, id = bg_grad_id)
    bg_fill <- sprintf("url(#%s)", bg_grad_id)
  } else {
    bg_gradient_def <- ""
    bg_fill <- bg_color
  }
  
  # Calculate content width
  content_width <- width - 2 * padding
  
  # Initialize Y position tracker
  current_y <- padding
  
  # Collect all SVG elements
  elements <- character()
  
  # ===== DEFS SECTION (styles, gradients, etc.) =====
  defs <- sprintf('
    <defs>
      <style>
        @import url("https://fonts.googleapis.com/css2?family=%s:wght@400;500;600;700&amp;display=swap");
        .card-title { font-family: "%s", sans-serif; font-weight: 700; font-size: %dpx; fill: %s; }
        .field-label { font-family: "%s", sans-serif; font-weight: 600; font-size: %dpx; fill: %s; }
        .field-value { font-family: "%s", sans-serif; font-weight: 400; font-size: %dpx; fill: %s; }
        .footer-text { font-family: "%s", sans-serif; font-style: italic; font-size: %dpx; fill: %s; opacity: 0.8; }
      </style>
      <clipPath id="card-clip-%s">
        <rect width="%d" rx="%d" ry="%d"/>
      </clipPath>
      %s
    </defs>',
    gsub(" ", "+", font),
    font, title_fontsize, title_color,
    font, label_fontsize, label_color,
    font, value_fontsize, value_text_color,
    font, footer_fontsize, footer_color,
    card_id, width, corner_radius, corner_radius,
    bg_gradient_def
  )
  
  # ===== TITLE =====
  has_top_logos <- length(logos) > 0
  
  # altura do header = maior entre altura das logos e altura "útil" do título
  # (use um fator para não ficar apertado quando título é pequeno)
  title_block_h <- title_fontsize * 1.2
  header_height <- max(title_block_h, if (has_top_logos) logos_height else title_block_h)
  
  # y do título:
  # - com logos: centraliza no meio do header
  # - sem logos: mantém comportamento antigo (baseline normal)
  if (has_top_logos) {
    title_y <- current_y + header_height / 2
    title_baseline <- "middle"
  } else {
    title_y <- current_y + title_fontsize
    title_baseline <- "alphabetic"
  }
  
  elements <- c(elements, sprintf(
    '<text x="%d" y="%.1f" class="card-title" dominant-baseline="%s">%s</text>',
    padding, title_y, title_baseline, escape_xml(title)
  ))
  
  # avança o cursor pelo header inteiro
  current_y <- current_y + header_height + 15
  
  
  # ===== BADGES (with uniform height) =====
  if (length(badges_data) > 0) {
    badge_x <- padding
    badge_y <- current_y
    
    # First pass: calculate max height for uniform badges
    max_badge_height <- 0
    for (badge in badges_data) {
      metrics_label <- gdtools::str_metrics(badge$label, fontname = font, fontsize = 10)
      metrics_value <- gdtools::str_metrics(badge$value, fontname = font, fontsize = 10)
      height_label <- as.numeric(metrics_label["ascent"]) + as.numeric(metrics_label["descent"])
      height_value <- as.numeric(metrics_value["ascent"]) + as.numeric(metrics_value["descent"])
      badge_h <- max(height_label, height_value) * 2
      if (badge_h > max_badge_height) max_badge_height <- badge_h
    }
    
    # Second pass: create badges with uniform height
    for (i in seq_along(badges_data)) {
      badge <- badges_data[[i]]
      color <- if (!is.null(badge$color)) badge$color else "white"
      badge_result <- create_badge(
        label = badge$label,
        value = badge$value,
        color = color,
        font = font,
        fontsize = value_fontsize,
        style = "",
        shadow_offset = 1,
        height = max_badge_height / 2,  # badge_svg doubles this
        as_string = FALSE
      )
      
      # Replace IDs to make them unique within the card context
      badge_svg_content <- badge_result$svg
      badge_svg_content <- gsub('id="s([^"]+)"', sprintf('id="s\\1_%d"', i), badge_svg_content)
      badge_svg_content <- gsub('id="r([^"]+)"', sprintf('id="r\\1_%d"', i), badge_svg_content)
      badge_svg_content <- gsub('#s([^"\\)]+)', sprintf('#s\\1_%d', i), badge_svg_content)
      badge_svg_content <- gsub('#r([^"\\)]+)', sprintf('#r\\1_%d', i), badge_svg_content)
      
      # Remove the outer svg tags and extract content
      badge_inner <- gsub('<\\?xml[^>]*\\?>', '', badge_svg_content)
      badge_inner <- gsub('<svg[^>]*>', '', badge_inner)
      badge_inner <- gsub('</svg>', '', badge_inner)
      
      # Wrap in a group with translation (no nested svg)
      elements <- c(elements, sprintf(
        '<g transform="translate(%.1f, %.1f)">%s</g>',
        badge_x, badge_y, badge_inner
      ))
      
      badge_x <- badge_x + badge_result$width + 4
    }
    
    current_y <- current_y + max_badge_height + 15
  }
  
  # ===== FIELDS =====
  row_spacing <- 12
  field_spacing <- 10
  value_box_padding_x <- 10
  value_box_padding_y <- 8
  value_box_radius <- 4
  
  for (row in fields) {
    if (length(row) == 0) next
    
    # Calculate field widths
    num_fields <- length(row)
    available_width <- content_width - (num_fields - 1) * field_spacing
    
    # Check for icon in first field
    # with_icon can be: TRUE (default house icon), FALSE/NULL (no icon), or a string (custom SVG)
    first_field_icon <- row[[1]]$with_icon
    has_icon <- !is.null(first_field_icon) && !isFALSE(first_field_icon)
    icon_width <- if (has_icon) 55 else 0
    
    # Determine the icon SVG to use
    if (has_icon) {
      if (isTRUE(first_field_icon)) {
        # Use default house icon
        icon_svg <- icon_house(50, 56)
        icon_height <- 56
      } else if (is.character(first_field_icon)) {
        # Use custom SVG provided
        icon_svg <- first_field_icon
        # Try to extract height from SVG (xml2 with regex fallback), default to 56
        icon_h <- svg_dimension(first_field_icon, "height", prefer = "attr")
        icon_height <- if (!is.na(icon_h)) icon_h else 56
      } else {
        has_icon <- FALSE
        icon_width <- 0
      }
    }
    
    if (has_icon && num_fields == 1) {
      field_widths <- content_width - icon_width - field_spacing
    } else {
      field_widths <- rep(available_width / num_fields, num_fields)
      if (has_icon) {
        field_widths[1] <- field_widths[1] - icon_width - field_spacing
      }
    }
    
    # Calculate row height based on wrapped text
    row_height <- 0
    field_data <- list()
    
    for (i in seq_along(row)) {
      field <- row[[i]]
      fw <- if (i == 1 && has_icon) field_widths[1] else field_widths[i]
      
      # Wrap the value text
      wrapped <- wrap_text(
        field$value, 
        fw - 2 * value_box_padding_x, 
        font, 
        value_fontsize
      )
      
      field_height <- label_fontsize + 4 + wrapped$height + 2 * value_box_padding_y
      row_height <- max(row_height, field_height)
      
      field_data[[i]] <- list(
        field = field,
        width = fw,
        wrapped = wrapped
      )
    }
    
    # Draw icon if needed
    field_x <- padding
    if (has_icon) {
      icon_y <- current_y + (row_height - icon_height) / 2
      elements <- c(elements, sprintf(
        '<g transform="translate(%.1f, %.1f)">%s</g>',
        field_x, icon_y, icon_svg
      ))
      field_x <- field_x + icon_width + field_spacing
    }
    
    # Draw each field
    for (i in seq_along(row)) {
      fd <- field_data[[i]]
      field <- fd$field
      fw <- fd$width
      wrapped <- fd$wrapped
      
      # Skip icon adjustment for first field (already done)
      if (i == 1 && has_icon) {
        # Already positioned
      } else if (i > 1) {
        field_x <- field_x + field_data[[i-1]]$width + field_spacing
      }
      
      # Draw label
      elements <- c(elements, sprintf(
        '<text x="%.1f" y="%.1f" class="field-label">%s</text>',
        field_x, current_y + label_fontsize, escape_xml(field$label)
      ))
      
      # Draw value box background
      # box_y <- current_y + label_fontsize + 4
      # box_height <- wrapped$height + 2 * value_box_padding_y
      box_y <- current_y + label_fontsize + 4
      
      # If uniform_row_height, make all boxes in this row the same height
      box_height <- if (isTRUE(uniform_row_height)) {
        row_height - (label_fontsize + 4)
      } else {
        wrapped$height + 2 * value_box_padding_y
      }
      
      
      elements <- c(elements, sprintf(
        '<rect x="%.1f" y="%.1f" width="%.1f" height="%.1f" rx="%d" fill="%s" stroke="#dee2e6" stroke-width="1"/>',
        field_x, box_y, fw, box_height, value_box_radius, value_bg_color
      ))
      
      # Draw value text (possibly multi-line)
      #text_y <- box_y + value_box_padding_y + value_fontsize * 0.85
      if (isTRUE(uniform_row_height)) {
        text_block_h <- length(wrapped$lines) * wrapped$line_height
        extra <- max(0, box_height - 2 * value_box_padding_y - text_block_h)
        text_y <- box_y + value_box_padding_y + (extra / 2) + value_fontsize * 0.85
      } else {
        text_y <- box_y + value_box_padding_y + value_fontsize * 0.85
      }
      
      for (line in wrapped$lines) {
        elements <- c(elements, sprintf(
          '<text x="%.1f" y="%.1f" class="field-value">%s</text>',
          field_x + value_box_padding_x, text_y, escape_xml(line)
        ))
        text_y <- text_y + wrapped$line_height
      }
      
    }
    
    current_y <- current_y + row_height + row_spacing
  }
  
  # ===== FOOTER (text left) + BOTTOM LOGOS (right, same line) =====
  footer_elements <- character()
  #footer_height <- 0
  footer_present <- (!is.null(footer) || length(bottom_logos) > 0)

  # Footer Y positions are computed after final_height is known so we can:
  # - prevent overlaps when footer logos are tall
  # - keep the last field row close to the bottom (small gap)
  # - bottom-align the footer logos with the card edge (inside padding)

  # Calculate final height
  # ===== FINAL HEIGHT (depends on footer/logo heights) =====
  if (footer_present) {
    has_bottom_logos <- (length(bottom_logos) > 0)
    footer_row_height <- max(footer_fontsize, if (has_bottom_logos) bottom_logos_height else 0)
    final_height <- current_y + gap_to_footer + footer_row_height + footer_row_padding_bottom
  } else {
    final_height <- current_y + padding
  }
  
  # ===== TOP-RIGHT LOGOS =====
  top_logos_svg <- ""
  if (length(logos) > 0) {
    logo_result <- create_logo_row(
      logos = logos,
      target_height = logos_height,
      spacing = 10,
      card_width = width,
      x_offset = padding,
      y_offset = padding
    )
    top_logos_svg <- logo_result$svg_content
  }
  
  
  # ===== FOOTER ELEMENTS (placed using final_height) =====
  if (footer_present) {
    has_bottom_logos <- (length(bottom_logos) > 0)
    footer_row_height <- max(footer_fontsize, if (has_bottom_logos) bottom_logos_height else 0)

    # Footer row is anchored to the bottom of the card
    footer_row_y_top <- final_height - footer_row_padding_bottom - footer_row_height
    footer_row_y_bottom <- footer_row_y_top + footer_row_height

    if (!is.null(footer)) {
      footer_y <- if (has_bottom_logos) {
        footer_row_y_top + footer_row_height / 2
      } else {
        footer_row_y_bottom - 2
      }

      footer_baseline <- if (has_bottom_logos) "middle" else "alphabetic"

      footer_elements <- c(footer_elements, sprintf(
        '<text x="%d" y="%.1f" class="footer-text" dominant-baseline="%s">%s</text>',
        padding, footer_y, footer_baseline, escape_xml(footer)
      ))
    }

    if (has_bottom_logos) {
      # Bottom-align logos with the card edge (inside footer_row_padding_bottom)
      logos_y <- final_height - footer_row_padding_bottom - bottom_logos_height
      logo_row <- create_logo_row(
        logos = bottom_logos,
        target_height = bottom_logos_height,
        spacing = 10,
        card_width = width,
        x_offset = padding,
        y_offset = logos_y
      )
      footer_elements <- c(footer_elements, logo_row$svg_content)
    }
  }
# ===== ASSEMBLE FINAL SVG =====
  svg <- sprintf(
    '<?xml version="1.0" encoding="UTF-8"?>
<svg xmlns="http://www.w3.org/2000/svg" width="%d" height="%d" viewBox="0 0 %d %d">
  %s
  <!-- Background -->
  <rect width="%d" height="%d" rx="%d" fill="%s"/>
  <!-- Content -->
  %s
  <!-- Top-right logos -->
  %s
</svg>',
    width, round(final_height), width, round(final_height),
    defs,
    width, round(final_height), corner_radius, bg_fill,
    paste(c(elements, footer_elements), collapse = "\n  "),
    top_logos_svg
  )
  
  # In interactive / iterative use, also render in the Viewer while still returning the SVG.
  if (isTRUE(show_viewer) && interactive()) {
    if (!requireNamespace("htmltools", quietly = TRUE)) {
      cli::cli_warn("Package {.pkg htmltools} is recommended to preview SVGs in the Viewer.")
    } else {
      preview <- htmltools::browsable(htmltools::HTML(svg))
      # html_print triggers the Viewer pane in RStudio / Posit
      try(htmltools::html_print(preview), silent = TRUE)
    }
  }

  return(svg)
}

---
title: "Font Management"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Font Management}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Overview

cardargus uses **Jost** as its default font, a modern sans-serif from Google Fonts. This article explains how to configure and manage fonts for best results.

## Quick Setup

```{r setup, eval=FALSE}
library(cardargus)

# Automatically setup fonts (recommended)
setup_fonts()
```

This function:

1. Registers Jost and Montserrat from Google Fonts
2. Enables showtext auto mode
3. Prepares the system for font rendering

## Check Availability

```{r check, eval=FALSE}
# Check if a font is available
font_available("Jost")
font_available("Arial")

# List cached fonts
list_fonts()
```

## Register Google Fonts

```{r google, eval=FALSE}
# Register a specific Google Font
register_google_font("Roboto")
register_google_font("Open Sans")
register_google_font("Lato")
```

## Install Fonts Locally

For offline use or better performance, download fonts to your local cache:

```{r install, eval=FALSE}
# Download and cache fonts
install_fonts()

# Install specific fonts
install_fonts(fonts = c("Jost", "Montserrat", "Roboto"))

# Check cache location
font_cache_dir()
```

## Using Custom Fonts

```{r custom, eval=FALSE}
# Use a different font in cards
card <- svg_card(
  title = "MY CARD",
  font = "Montserrat",  # Use Montserrat instead of Jost
  ...
)
```

## CSS for SVG

cardargus embeds fonts via Google Fonts CSS in SVGs:

```{r css, eval=FALSE}
# Generate CSS for a font
css <- get_font_css("Jost")
cat(css)
# @import url("https://fonts.googleapis.com/css2?family=Jost:wght@400;500;600;700&display=swap");

# Multiple weights
css <- get_font_css("Montserrat", weights = c(400, 700))
```

## Font Cache

cardargus caches downloaded fonts in a user directory:

```{r cache, eval=FALSE}
# Get cache directory
font_cache_dir()
#> "/home/user/.cache/cardargus"

# List cached fonts
list_fonts()
```

## Access Package Resources

```{r resources, eval=FALSE}
# SVGs directory (bundled logos)
svgs_dir()

# Path to a specific SVG
get_svg_path("morar_bem.svg")

# List available SVGs
list_bundled_svgs()
```

## Troubleshooting

### Font not rendering correctly

1. Run `setup_fonts()` before creating cards
2. Check if font is available: `font_available("Jost")`
3. Install required packages:

```{r packages, eval=FALSE}
install.packages(c("showtext", "sysfonts", "systemfonts"))
```

### PNG export with rsvg

For correct font rendering in PNG exports with rsvg:

```{r png, eval=FALSE}
# Setup fonts first
setup_fonts()

# Download fonts for embedding
install_fonts()

# Create and export
card <- svg_card(...)
svg_to_png(card, "card.png", dpi = 300)
```

### PNG export with Chrome (recommended)

For **perfect font rendering**, use Chrome-based conversion:

```{r chrome, eval=FALSE}
# Ensure Chrome is available
ensure_chrome(download = TRUE)

# Create and export
card <- svg_card(...)
svg_to_png_chrome(card, "card.png", dpi = 300)
```

Chrome handles Google Fonts automatically via `@import`, so no local font installation is needed.

### Fonts on servers

On servers without GUI:

```{r server, eval=FALSE}
# Option 1: Download fonts locally
install_fonts()

# Update font cache (Linux)
system("fc-cache -fv")

# Option 2: Use Chrome rendering (recommended)
ensure_chrome(download = TRUE)
svg_to_png_chrome(card, "card.png")
```

## Recommended Fonts

| Font | Type | Description |
|------|------|-------------|
| Jost | Sans-serif | Default, modern |
| Montserrat | Sans-serif | Elegant alternative |
| Roboto | Sans-serif | Readable, Google Material |
| Open Sans | Sans-serif | Neutral, versatile |
| Lato | Sans-serif | Friendly, rounded |

## Dependencies

For full font functionality:

```r
install.packages(c(
  "showtext",      # Font rendering
  "sysfonts",      # Font management
  "systemfonts",   # System font detection
  "gdtools"        # Text metrics
))
```

For Chrome-based rendering (recommended):

```r
install.packages("chromote")
```

---
title: "Export and Conversion"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Export and Conversion}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Overview

cardargus provides two rendering engines for converting SVG to PNG/PDF:
 
| Engine | Package | Fonts | Best For |
|--------|---------|-------|----------|
| **rsvg** | `rsvg`, `magick` | Embedded only | Speed, no Chrome dependency |
| **Chrome** | `chromote` | Web fonts + embedded | Perfect font rendering |

## Saving as SVG

The simplest way to save a card:

```{r svg, eval=FALSE}
library(cardargus)

card <- svg_card(
  title = "PROJECT",
  badges_data = list(list(label = "Status", value = "Active")),
  fields = list(list(list(label = "Name", value = "Test"))),
  bg_color = "#3498db"
)

# Save to file (with embedded fonts)
save_svg(card, "my_card.svg")
```

## Converting to PNG

### Standard conversion (rsvg)

Uses librsvg/ImageMagick. Fast and reliable, but requires fonts to be embedded in the SVG:

```{r png-basic, eval=FALSE}
# Convert SVG string to PNG
svg_to_png(card, "my_card.png")

# With custom DPI (default is 300)
svg_to_png(card, "my_card.png", dpi = 300)
```

### Chrome-based conversion (recommended for fonts)

Uses headless Chrome via the `chromote` package. Provides **perfect font rendering**, including Google Fonts loaded via `@import`:

```{r png-chrome, eval=FALSE}
# Check if Chrome is available
if (chrome_available()) {
  # High-quality PNG with Chrome
  svg_to_png_chrome(card, "my_card.png", dpi = 300)
}

# Convert to PDF (vector output)
svg_to_pdf_chrome(card, "my_card.pdf")
```

#### Installing Chrome support

```{r chrome-install, eval=FALSE}
# Install chromote package
install.packages("chromote")

# Check if Chrome is available
chrome_available()

# If Chrome is not installed, download it automatically
ensure_chrome(download = TRUE)

# This downloads "Chrome for Testing" (~150MB) to your user cache
# No admin/root permissions required!
```

### Background options

```{r png-bg, eval=FALSE}
# Transparent background (preserves rounded corners)
svg_to_png(card, "card_transparent.png", dpi = 300)

# White background
svg_to_png(card, "card_white.png", dpi = 300, background = "white")

# Chrome also supports backgrounds
svg_to_png_chrome(card, "card_white.png", dpi = 300, background = "white")
```

### PNG with margin

Add a margin around the card:

```{r png-margin, eval=FALSE}
# Add 20px margin with white background
svg_to_png_with_margin(
  card,
  output_path = "card_with_margin.png",
  margin = 20,
  background = "white",
  dpi = 300
)
```

## Multiple formats

Export to multiple formats at once:

```{r formats, eval=FALSE}
# Export to SVG and PNG
svg_to_formats(
  card,
  output_base = "exports/my_card",  # Without extension
  formats = c("svg", "png"),
  dpi = 300
)
# Creates: exports/my_card.svg, exports/my_card.png
```

## Batch processing

Convert multiple cards efficiently:

```{r batch, eval=FALSE}
# Create multiple cards
cards <- list(
  svg_card(title = "Card 1", badges_data = list(), fields = list()),
  svg_card(title = "Card 2", badges_data = list(), fields = list()),
  svg_card(title = "Card 3", badges_data = list(), fields = list())
)

# Batch convert to PNG
batch_svg_to_png(
  svg_list = cards,
  output_dir = "output/cards",
  prefix = "card",
  dpi = 300,
  background = "white"
)
# Creates: output/cards/card_001.png, card_002.png, card_003.png
```

## Displaying in R Markdown / Quarto

### Choosing the rendering engine

All knitr functions support an `engine` parameter:

- `"auto"` (default): Uses Chrome if available, otherwise rsvg
- `"chrome"`: Forces Chrome (errors if unavailable)
- `"rsvg"`: Forces rsvg/magick

### Inline SVG display

```{r inline, eval=FALSE}
# Display card directly in document (HTML only)
include_card(card)

# With custom width
include_card(card, width = "80%")
```

### PNG display (better compatibility)

```{r png-display, eval=FALSE}
# Display as PNG with auto engine selection
include_card_png(card, dpi = 150)

# Force Chrome for best font rendering
include_card_png(card, dpi = 300, engine = "chrome")

# Force rsvg (faster, no Chrome dependency)
include_card_png(card, dpi = 300, engine = "rsvg")
```

### Save for knitr

```{r knitr-save, eval=FALSE}
# Save and get path for knitr::include_graphics
path <- save_card_for_knitr(
  card, 
  filename = "my_card", 
  format = "png", 
  dpi = 300,
  engine = "chrome"  # or "auto", "rsvg"
)
knitr::include_graphics(path)
```

### Custom knitr engine

```{r engine, eval=FALSE}
# Register the cardargus engine
register_cardargus_knitr()

# Then in a chunk with engine='cardargus':
# ```{cardargus}
# svg_card(title = "Dynamic Card", ...)
# ```
```

## Integration with grid/ggplot2

Convert cards to grobs for use with grid graphics:

```{r grob, eval=FALSE}
library(grid)

# Convert card to grob (auto-selects best engine)
grob <- card_to_grob(card, dpi = 150)

# Force Chrome rendering
grob <- card_to_grob(card, dpi = 150, engine = "chrome")

# Draw in a grid viewport
grid.newpage()
grid.draw(grob)
```

## Quality settings

### DPI recommendations

| Use Case | DPI | Notes |
|----------|-----|-------|
| Screen/web | 72-96 | Smaller file size |
| Standard print | 150 | Good balance |
| High-quality print | 300 | Publication quality |
| Large format | 300+ | Posters, banners |

### DPI and Chrome scaling

Chrome uses 96 DPI as its base resolution. When you set `dpi = 300`, cardargus calculates a scale factor:

```
scale = dpi / 96 = 300 / 96 ≈ 3.125x
```

This means a 400×300 SVG becomes a 1250×938 PNG at 300 DPI.

### File size considerations

```{r size, eval=FALSE}
# Smaller file (lower quality)
svg_to_png_chrome(card, "card_web.png", dpi = 96)

# Medium file (balanced)
svg_to_png_chrome(card, "card_standard.png", dpi = 150)

# Larger file (high quality)
svg_to_png_chrome(card, "card_print.png", dpi = 300)
```

## Comparing engines

```{r compare, eval=FALSE}
# Same card, different engines
svg_to_png(card, "card_rsvg.png", dpi = 300)
svg_to_png_chrome(card, "card_chrome.png", dpi = 300)

# Chrome generally produces better results for:
# - Google Fonts and custom web fonts
# - Complex CSS styling
# - Font weights and variants
```

## Error handling

```{r errors, eval=FALSE}
# Safe conversion with fallback
convert_card <- function(card, output_path, dpi = 300) {
  if (chrome_available()) {
    svg_to_png_chrome(card, output_path, dpi = dpi)
  } else {
    message("Chrome not available, using rsvg")
    svg_to_png(card, output_path, dpi = dpi)
  }
}

# With explicit error handling
tryCatch({
  svg_to_png_chrome(card, "output.png", dpi = 300)
  message("Conversion successful!")
}, error = function(e) {
  message("Chrome conversion failed: ", e$message)
  message("Falling back to rsvg...")
  svg_to_png(card, "output.png", dpi = 300)
})
```

## Troubleshooting

### Fonts not rendering correctly with rsvg

This is expected behavior. librsvg cannot:

- Load fonts via `@import url(...)`
- Make HTTP requests for web fonts

**Solution**: Use `svg_to_png_chrome()` or ensure fonts are embedded with `save_svg()`.

### Chrome not found

```{r chrome-not-found, eval=FALSE}
# Check Chrome availability
chrome_available()

# Option 1: Download Chrome for Testing automatically
ensure_chrome(download = TRUE)

# Option 2: Set path to existing Chrome manually
Sys.setenv(CHROMOTE_CHROME = "/path/to/chrome")

# Option 3: Find where Chrome is installed
find_chrome_path()
```

### Slow Chrome conversion

Chrome has startup overhead. For batch processing, consider:

```{r batch-tip, eval=FALSE}
# For many cards, rsvg may be faster if fonts are embedded
svg_to_png(card, "output.png", dpi = 300)

# Or accept the Chrome overhead for perfect rendering
svg_to_png_chrome(card, "output.png", dpi = 300)
```

### Timeout errors

```{r timeout, eval=FALSE}
# Increase timeout for complex SVGs
svg_to_png_chrome(card, "output.png", timeout = 60)  # 60 seconds
```

---
title: "Introduction to cardargus"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to cardargus}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## What is cardargus?

**cardargus** is an R package for creating informative cards in SVG format. Cards are self-contained, with all styles, fonts, and graphic elements embedded in the SVG file itself.

## Installation

```{r setup, eval=FALSE}
# Install from GitHub
devtools::install_github("castlab/cardargus")
```

```{r load, eval=FALSE}
library(cardargus)
```

## Creating your first card

The simplest way to create a card is using the `svg_card()` function:

```{r first-card, eval=FALSE}
library(cardargus)

card <- svg_card(
  title = "PROJECT",
  badges_data = list(
    list(label = "Units", value = "192", color = "white"),
    list(label = "Budget", value = "$5.2M", color = "white"),
    list(label = "Status", value = "Active", color = "#4CAF50")
  ),
  fields = list(
    list(
      list(label = "Project Name", value = "Downtown Housing", with_icon = TRUE)
    ),
    list(
      list(label = "Address", value = "123 Main Street, City Center")
    ),
    list(
      list(label = "City", value = "New York"),
      list(label = "State", value = "NY")
    ),
    list(
      list(label = "Developer", value = "ABC Construction"),
      list(label = "Owner", value = "City Housing Authority")
    )
  ),
  bg_color = "#3498db",
  width = 500,
  footer = "Source: Housing Department - January 2026"
)
```

## Displaying cards in R Markdown / Quarto

Use `include_card()` to display cards inline in your documents:

```{r display, eval=FALSE}
# Display card as SVG (best quality)
include_card(card)

# Display card as PNG (better compatibility)
include_card_png(card, dpi = 150)
```

For chunk-based workflow, register the cardargus engine:

```{r engine, eval=FALSE}
# In setup chunk
register_cardargus_knitr()

# Then use cardargus chunks:
# ```{cardargus}
# svg_card(title = "My Card", ...)
# ```
```

## Saving cards

### As SVG

```{r save-svg, eval=FALSE}
save_svg(card, "my_card.svg")
```

### As PNG

```{r save-png, eval=FALSE}
# Standard conversion (uses rsvg)
svg_to_png(card, "my_card.png", dpi = 300)

# Chrome-based conversion (best font rendering)
svg_to_png_chrome(card, "my_card.png", dpi = 300)

# PNG with white background
svg_to_png(card, "my_card.png", dpi = 300, background = "white")
```

### Chrome rendering (recommended)

For **perfect font rendering** with Google Fonts, use headless Chrome:

```{r chrome, eval=FALSE}
# Check if Chrome is available
chrome_available()

# PNG with Chrome (best quality)
svg_to_png_chrome(card, "my_card.png", dpi = 300)

# Vector PDF with Chrome
svg_to_pdf_chrome(card, "my_card.pdf")

# In R Markdown / Quarto
include_card_png(card, dpi = 300, engine = "chrome")
```

Install `chromote` for Chrome support: `install.packages("chromote")`

## Using bundled logos

The package includes several SVG logos ready to use:

```{r logos, eval=FALSE}
# List available logos
list_bundled_svgs()

# Use in a card
card <- svg_card(
  title = "FAR",
  badges_data = list(...),
  fields = list(...),
  bg_color = "#fab255",
  logos = c(get_svg_path("morar_bem.svg"), get_svg_path("seduh.svg")),
  logos_height = 40,
  bottom_logos = c(get_svg_path("gov_pe3.svg")),
  bottom_logos_height = 35
)
```

## Custom cards

For more control, use `svg_card()` directly with all options:

```{r custom-card, eval=FALSE}
# Define badges
badges <- list(
  list(label = "Units", value = "500", color = "white"),
  list(label = "Investment", value = "$50M", color = "#4CAF50")
)

# Define fields with custom icons
fields <- list(
  list(
    list(label = "Project", value = "Housing Development", 
         with_icon = icon_building())  # Or use a file path: "/path/to/icon.svg"
  ),
  list(
    list(label = "City", value = "Boston"),
    list(label = "State", value = "MA")
  ),
  list(
    list(label = "Start", value = "Jan/2024"),
    list(label = "End", value = "Dec/2025"),
    list(label = "Progress", value = "45%")
  )
)

# Create card
card <- svg_card(
  title = "HOUSING",
  badges_data = badges,
  fields = fields,
  bg_color = "#2c3e50",
  title_color = "#ecf0f1",
  width = 500
)
```

## Next steps

- [Customizing Cards](customization.html) - Colors, fonts, and styles
- [Working with Badges](badges.html) - Shields.io-style badges
- [Icons and Logos](icons.html) - Using custom icons
- [Font Management](fonts.html) - Google Fonts integration
- [Export and Conversion](export.html) - PNG, PDF, and batch processing

---
title: "Customizing Cards"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Customizing Cards}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Colors

### Background color

```{r bg-color, eval=FALSE}
library(cardargus)

# Orange card
card_orange <- svg_card(title = "ALERT", bg_color = "#fab255", ...)

# Blue card
card_blue <- svg_card(title = "INFO", bg_color = "#3498db", ...)
 
# Green card
card_green <- svg_card(title = "SUCCESS", bg_color = "#2ecc71", ...)

# Red card
card_red <- svg_card(title = "URGENT", bg_color = "#e74c3c", ...)

# Dark card
card_dark <- svg_card(title = "PREMIUM", bg_color = "#2c3e50", ...)
```

### Gradient backgrounds

You can use CSS-style linear gradients instead of solid colors:

```{r gradient, eval=FALSE}
# Horizontal gradient (left to right)
card <- svg_card(
  title = "HOUSING",
  bg_color = "linear-gradient(to right, #1a5a3a, #2e7d32)",
  ...
)

# Diagonal gradient (135 degrees)
card <- svg_card(
  title = "PROGRAM", 
  bg_color = "linear-gradient(135deg, #667eea, #764ba2)",
  ...
)

# Vertical gradient (top to bottom)
card <- svg_card(
  title = "PROJECT",
  bg_color = "linear-gradient(to bottom, #00c6ff, #0072ff)",
  ...
)

# Multi-color gradient
card <- svg_card(
  title = "RAINBOW",
  bg_color = "linear-gradient(to right, #ff6b6b, #feca57, #48dbfb)",
  ...
)
```

Supported gradient directions:

- `to right` - horizontal, left to right
- `to left` - horizontal, right to left
- `to bottom` - vertical, top to bottom (default)
- `to top` - vertical, bottom to top
- `45deg`, `90deg`, `135deg`, etc. - angle-based directions

### Text colors

```{r text-colors, eval=FALSE}
card <- svg_card(
  title = "MY CARD",
  title_color = "#FFD700",      # Gold title
  label_color = "#E0E0E0",      # Light gray labels
  footer_color = "white",       # White footer
  value_text_color = "#212529", # Dark gray values
  value_bg_color = "#f8f8ff",   # Field background
  ...
)
```

## Fonts

cardargus uses Google Fonts by default. The default font is "Jost", but you can use any Google Font:

```{r fonts, eval=FALSE}
# Using Roboto
card <- svg_card(
  title = "CARD",
  font = "Roboto",
  ...
)

# Using Open Sans
card <- svg_card(
  title = "CARD",
  font = "Open Sans",
  ...
)

# Using Montserrat
card <- svg_card(
  title = "CARD",
  font = "Montserrat",
  ...
)
```

## Font sizes

```{r font-sizes, eval=FALSE}
card <- svg_card(
  title = "CARD",
  title_fontsize = 32,    # Larger title
  label_fontsize = 12,    # Labels
  value_fontsize = 11,    # Values
  footer_fontsize = 8,    # Footer
  ...
)
```

## Dimensions

### Card width

```{r width, eval=FALSE}
# Narrow card
card_narrow <- svg_card(width = 400, ...)

# Default card
card_default <- svg_card(width = 500, ...)

# Wide card
card_wide <- svg_card(width = 600, ...)
```

### Padding and corners

```{r padding, eval=FALSE}
card <- svg_card(
  padding = 25,         # Internal padding
  corner_radius = 12,   # More rounded corners
  ...
)
```

## Custom footer

```{r footer, eval=FALSE}
# With automatic timestamp
card <- svg_card(
  footer = NULL,  # Uses automatic timestamp
  ...
)

# With custom text
card <- svg_card(
  footer = "Source: Housing Dept. - Updated January 2026",
  ...
)

# Without footer
card <- svg_card(
  footer = "",
  ...
)
```

## Custom logos

```{r logos, eval=FALSE}
# Without logos
card <- svg_card(
  logos = list(),
  ...
)

# With file paths
card <- svg_card(
  logos = c("path/to/logo1.svg", "path/to/logo2.svg"),
  logos_height = 40,
  ...
)

# With bundled logos
card <- svg_card(
  logos = c(get_svg_path("morar_bem.svg")),
  logos_height = 40,
  bottom_logos = c(get_svg_path("gov_pe3.svg")),
  bottom_logos_height = 35,
  ...
)
```

## Complete example

```{r complete, eval=FALSE}
card <- svg_card(
  title = "HOUSING PROGRAM",
  badges_data = list(
    list(label = "Units", value = "1,000", color = "white"),
    list(label = "Investment", value = "$100M", color = "#4CAF50")
  ),
  fields = list(
    list(list(label = "Project", value = "Downtown Development", with_icon = TRUE)),
    list(
      list(label = "City", value = "Boston"),
      list(label = "State", value = "MA")
    )
  ),
  bg_color = "#2c3e50",
  width = 550,
  padding = 25,
  corner_radius = 12,
  font = "Roboto",
  title_fontsize = 24,
  title_color = "#ecf0f1",
  label_color = "#bdc3c7",
  footer = "Real-time updated data",
  footer_color = "#95a5a6"
)
```

## Displaying in documents

```{r display, eval=FALSE}
# In R Markdown or Quarto
include_card(card)

# Or as PNG for better compatibility
include_card_png(card, dpi = 150)
```

---
title: "Working with Badges"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Working with Badges}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Overview

Badges in cardargus are inspired by [shields.io](https://shields.io/) and consist of a label-value pair with customizable colors.

## Creating a single badge

```{r single-badge, eval=FALSE}
library(cardargus)

# Basic badge
badge <- create_badge(
  label = "Status",
  value = "Active",
  color = "#4CAF50"
)

# Save to file
writeLines(badge, "status_badge.svg")
```

## Badge colors

The right side of the badge uses the specified color. The left side is always gray (#555):

```{r colors, eval=FALSE}
# Green for success
badge_success <- create_badge("Build", "Passing", "#4CAF50")

# Red for failure
badge_error <- create_badge("Build", "Failing", "#e74c3c")

# Blue for information
badge_info <- create_badge("Version", "1.0.0", "#3498db")

# Orange for warning
badge_warning <- create_badge("Coverage", "45%", "#fab255")

# White (default)
badge_default <- create_badge("License", "MIT", "white")
```

## Automatic text color

The badge automatically adjusts text color based on background brightness:

```{r auto-color, eval=FALSE}
# Dark background -> white text
create_badge("Test", "Dark", "#2c3e50")

# Light background -> dark text
create_badge("Test", "Light", "#f1c40f")
```

## Badge rows

Create multiple badges in a horizontal row:

```{r badge-row, eval=FALSE}
badges <- list(
  list(label = "Units", value = "192"),
  list(label = "Budget", value = "$36.4M"),
  list(label = "Status", value = "Active", color = "#4CAF50")
)

# All badges will have uniform height
row <- create_badge_row(badges, default_color = "white", uniform_height = TRUE)
```

## Badges in cards

Badges are automatically positioned at the top of cards:

```{r in-cards, eval=FALSE}
card <- svg_card(
  title = "PROJECT",
  badges_data = list(
    list(label = "Units", value = "500", color = "white"),
    list(label = "Budget", value = "$50M", color = "#4CAF50"),
    list(label = "Phase", value = "2/3", color = "#3498db")
  ),
  fields = list(...),
  bg_color = "#fab255"
)
```

## Uniform badge height

When using `create_badge_row()` or in cards, all badges automatically have the same height for visual consistency:

```{r uniform, eval=FALSE}
# These badges will all have the same height
badges <- list(
  list(label = "A", value = "1"),              # Short
  list(label = "Long Label", value = "Value"), # Long
  list(label = "X", value = "Y")               # Short
)

row <- create_badge_row(badges, uniform_height = TRUE)
```

## Customizing badge appearance

```{r customize, eval=FALSE}
badge <- create_badge(
  label = "Custom",
  value = "Badge",
  color = "#9b59b6",
  font = "Roboto",          # Custom font
  fontsize = 12,            # Font size
  style = "margin: 5px;"    # Additional CSS
)
```

## Saving badges

```{r save, eval=FALSE}
# Single badge
badge <- create_badge("Version", "1.0.0", "#3498db")
writeLines(badge, "version.svg")

# Badge row
row <- create_badge_row(badges)
writeLines(row, "badges.svg")
```

## Badge in R Markdown

```{r rmarkdown, eval=FALSE}
# Display inline
badge <- create_badge("Status", "Active", "#4CAF50")
knitr::asis_output(badge)
```

---
title: "Icons and Logos"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Icons and Logos}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Built-in Icons

cardargus includes several SVG icons for common use cases:

```{r icons, eval=FALSE}
library(cardargus)

# House icon (default for fields with with_icon = TRUE)
icon_house(width = 50, height = 56)

# Building icon
icon_building(width = 50, height = 56)

# Construction icon
icon_construction(width = 50, height = 56)

# Map pin icon
icon_map_pin(width = 50, height = 56)

# Money icon
icon_money(width = 50, height = 56)
```

## Using icons in fields

Icons can be added to fields in three ways:

1. **TRUE**: Uses the default house icon
2. **Built-in icon function**: e.g., `icon_building()`
3. **File path**: Any SVG file on your system

```{r field-icons, eval=FALSE}
fields <- list(
  # Default house icon
  list(
    list(label = "Project", value = "Housing Complex", with_icon = TRUE)
  ),
  # Built-in building icon
  list(
    list(label = "Building", value = "Tower A", 
         with_icon = icon_building())
  ),
  # Construction icon
  list(
    list(label = "Status", value = "Under Construction",
         with_icon = icon_construction())
  ),
  # Custom SVG file path
  list(
    list(label = "Custom", value = "My Item",
         with_icon = "/path/to/my_icon.svg")
  )
)

card <- svg_card(
  title = "PROJECT",
  badges_data = list(),
  fields = fields,
  bg_color = "#3498db"
)
```

## Customizing icons

All icons accept customization parameters:

```{r custom-icons, eval=FALSE}
# Custom colors
icon <- icon_house(
  width = 60,
  height = 70,
  stroke_color = "#e74c3c",  # Red outline
  stroke_width = 3,
  fill = "#ffebee"           # Light red fill
)

# Building with custom size
building <- icon_building(
  width = 40,
  height = 48,
  stroke_color = "#2c3e50",
  fill = "#ecf0f1"
)
```

## Bundled Logos

The package includes several SVG logos ready to use:

```{r bundled, eval=FALSE}
# List all bundled SVGs
list_bundled_svgs()
#> [1] "gov_pe3.svg" "morar_bem.svg" "seduh.svg" "tree.svg" "example_card.svg"

# Get full path to a logo
path <- get_svg_path("morar_bem.svg")

# Get the SVGs directory
svgs_dir()
```

## Using logos in cards

Logos can be file paths or SVG strings:

```{r logos-cards, eval=FALSE}
card <- svg_card(
  title = "FAR",
  badges_data = list(
    list(label = "Units", value = "192", color = "white")
  ),
  fields = list(...),
  bg_color = "#fab255",
  
  # Top-right logos (bundled or custom paths)
  logos = c(
    get_svg_path("morar_bem.svg"),
    get_svg_path("seduh.svg"),
    "/path/to/my_custom_logo.svg"  # Any local file
  ),
  logos_height = 40,
  
  # Bottom-left logos
  bottom_logos = c(
    get_svg_path("gov_pe3.svg")
  ),
  bottom_logos_height = 35
)
```

## Loading external SVGs

Use `load_svg_for_embed()` to load and resize external SVG files:
 
```{r external, eval=FALSE}
# Load and resize an external SVG
logo_data <- load_svg_for_embed(
  svg_path = "path/to/my_logo.svg",
  target_height = 50
)

# The result contains:
# - svg_content: The processed SVG string
# - width: Calculated width (maintains aspect ratio)
# - height: The target height
```

## Creating logo rows

For advanced positioning, use the logo row functions:

```{r logo-rows, eval=FALSE}
# Top-right logo row
logo_row <- create_logo_row(
  logos = c("logo1.svg", "logo2.svg"),
  target_height = 40,
  spacing = 15,
  card_width = 500,
  x_offset = 20,
  y_offset = 20
)

# Bottom-left logo row
bottom_row <- create_bottom_logo_row(
  logos = c("logo3.svg"),
  target_height = 30,
  spacing = 10,
  x_offset = 20,
  card_height = 400,
  y_offset = 20
)
```

## Custom SVG logos

You can use any SVG string as a logo:

```{r custom-svg, eval=FALSE}
# Create a simple custom logo
my_logo <- '<svg width="50" height="50" viewBox="0 0 50 50">
  <circle cx="25" cy="25" r="20" fill="#3498db"/>
  <text x="25" y="30" text-anchor="middle" fill="white" font-size="16">AB</text>
</svg>'

# Use in a card
card <- svg_card(
  title = "PROJECT",
  logos = list(my_logo),
  logos_height = 40,
  ...
)
```

## Summary: How to use icons and logos

| Element | How to specify |
|---------|----------------|
| Default icon | `with_icon = TRUE` |
| Built-in icon | `with_icon = icon_house()` |
| Custom SVG file | `with_icon = "/path/to/file.svg"` |
| Bundled logo | `logos = c(get_svg_path("name.svg"))` |
| Custom logo file | `logos = c("/path/to/logo.svg")` |
| SVG string | `logos = list('<svg>...</svg>')` |

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fonts.R
\name{list_bundled_svgs}
\alias{list_bundled_svgs}
\title{List available bundled SVG files}
\usage{
list_bundled_svgs()
}
\value{
Character vector of available SVG filenames
}
\description{
List available bundled SVG files
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xml_utils.R
\name{num_lead}
\alias{num_lead}
\title{Extract the leading number from a dimension string}
\usage{
num_lead(x)
}
\arguments{
\item{x}{A character such as \code{"500"}, \code{"40px"}, \code{"100\%"}, or \code{NA}.}
}
\value{
Numeric value, or \code{NA_real_} if none is found.
}
\description{
Extract the leading number from a dimension string
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logos.R
\name{create_bottom_logo_row}
\alias{create_bottom_logo_row}
\title{Create logo row for bottom-left corner of card}
\usage{
create_bottom_logo_row(
  logos,
  target_height = 30,
  spacing = 10,
  x_offset = 20,
  card_height = 400,
  y_offset = 20
)
}
\arguments{
\item{logos}{List of SVG strings or file paths}

\item{target_height}{Height for all logos (default 30)}

\item{spacing}{Horizontal spacing between logos (default 10)}

\item{x_offset}{Left margin from card edge}

\item{card_height}{Total card height for y positioning}

\item{y_offset}{Bottom margin}
}
\value{
A list with svg_content and total_width
}
\description{
Takes a list of SVG logos and arranges them horizontally for the bottom-left.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logos.R
\name{icon_map_pin}
\alias{icon_map_pin}
\title{Map Pin Icon SVG}
\usage{
icon_map_pin(
  width = 50,
  height = 56,
  stroke_color = "white",
  stroke_width = 2,
  fill = "none"
)
}
\arguments{
\item{width}{Width of the icon}

\item{height}{Height of the icon}

\item{stroke_color}{Stroke color}

\item{stroke_width}{Stroke width}

\item{fill}{Fill color}
}
\value{
SVG string
}
\description{
Generate a map pin/location icon SVG.
}
\examples{
icon_map_pin(50, 56)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fonts.R
\name{list_fonts}
\alias{list_fonts}
\title{List registered/cached fonts}
\usage{
list_fonts()
}
\value{
Character vector of font family names that are cached
}
\description{
List registered/cached fonts
}
\examples{
list_fonts()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logos.R
\name{load_svg_for_embed}
\alias{load_svg_for_embed}
\title{Load and process external SVG file for embedding}
\usage{
load_svg_for_embed(svg_path, target_height = 40, target_width = NULL)
}
\arguments{
\item{svg_path}{Path to the SVG file}

\item{target_height}{Desired height in pixels}

\item{target_width}{Optional desired width (calculated from aspect ratio if NULL)}
}
\value{
A list with svg_content, width, and height
}
\description{
Reads an SVG file and processes it to be embedded inside another SVG.
Removes XML declarations, adjusts dimensions, and prepares for embedding.
}
\details{
This function is useful when you want to embed custom logos or icons in cards.
You can pass any SVG file path to the \code{logos}, \code{bottom_logos}, or \code{with_icon}
parameters of \code{svg_card()}.
}
\examples{
\dontrun{ # Need a external svg file
# Load a custom logo
logo <- load_svg_for_embed("/path/to/logo.svg", target_height = 40)

# Or just pass the path directly to svg_card():
svg_card(
  title = "My Card",
  logos = c("/path/to/logo1.svg", "/path/to/logo2.svg"),
  ...
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{is_light_color}
\alias{is_light_color}
\title{Check if a color is light}
\usage{
is_light_color(color)
}
\arguments{
\item{color}{A hex color string or color name}
}
\value{
Logical indicating if the color is light
}
\description{
Check if a color is light
}
\examples{
is_light_color("#FFFFFF")
is_light_color("#000000")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fonts.R
\name{font_available}
\alias{font_available}
\title{Check if a font is available for embedding}
\usage{
font_available(family)
}
\arguments{
\item{family}{Font family name}
}
\value{
TRUE if font is cached (any supported format), FALSE otherwise
}
\description{
Check if a font is available for embedding
}
\examples{
font_available("Jost")
font_available("Montserrat")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chrome.R
\name{find_chrome_path}
\alias{find_chrome_path}
\title{Find Chrome executable path}
\usage{
find_chrome_path()
}
\value{
Path to Chrome executable, or NULL if not found.
}
\description{
Attempts to find a Chrome or Chromium executable on the system.
Checks common installation paths and environment variables.
}
\examples{
path <- find_chrome_path()
if (!is.null(path)) {
  message("Chrome found at: ", path)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fonts.R
\name{get_svg_path}
\alias{get_svg_path}
\title{Get path to a bundled SVG file}
\usage{
get_svg_path(filename, height = NULL, width = NULL)
}
\arguments{
\item{filename}{Name of the SVG file (e.g., "morar_bem.svg")}

\item{height}{Optional target height (px). If provided, returns the SVG content resized
for embedding instead of the file path.}

\item{width}{Optional target width (px). Only used when returning resized SVG content.}
}
\value{
If \code{height} and \code{width} are both NULL, returns the full file path.
Otherwise returns the resized SVG content (character string).
}
\description{
Returns the full path to a SVG file bundled with the package.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fonts.R
\name{detect_svg_fonts}
\alias{detect_svg_fonts}
\title{Detect font families used in an SVG}
\usage{
detect_svg_fonts(svg_content)
}
\arguments{
\item{svg_content}{Character SVG.}
}
\value{
Character vector of font family names (unique).
}
\description{
Detect font families used in an SVG
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chrome.R
\name{find_chrome_in_dir}
\alias{find_chrome_in_dir}
\title{Find Chrome executable in a directory}
\usage{
find_chrome_in_dir(dir)
}
\description{
Find Chrome executable in a directory
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chrome.R
\name{chrome_cache_dir}
\alias{chrome_cache_dir}
\title{Get Chrome cache directory}
\usage{
chrome_cache_dir()
}
\description{
Get Chrome cache directory
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cards.R
\name{svg_card}
\alias{svg_card}
\title{Create an Information Card in SVG Format}
\usage{
svg_card(
  title = "FAR",
  badges_data = list(),
  fields = list(),
  bg_color = "#fab255",
  width = 500,
  padding = 20,
  corner_radius = 8,
  font = "Jost",
  title_fontsize = 16,
  title_color = "white",
  label_fontsize = 11,
  value_fontsize = 11,
  label_color = "white",
  value_bg_color = "#f8f8ff",
  value_text_color = "#212529",
  show_house_icon = TRUE,
  logos = list(),
  logos_height = 40,
  bottom_logos = list(),
  bottom_logos_height = 30,
  footer = NULL,
  gap_to_footer = 6,
  footer_row_padding_bottom = 6,
  footer_fontsize = 8,
  footer_color = "white",
  uniform_row_height = TRUE,
  show_viewer = interactive()
)
}
\arguments{
\item{title}{Card title (e.g., "FAR", "FNHIS")}

\item{badges_data}{List of badge data (label, value, color)}

\item{fields}{List of field rows, each row is a list of fields with label, value,
and optionally with_icon. The with_icon parameter can be:
\itemize{
\item TRUE - uses the default house icon
\item FALSE or NULL - no icon
\item A character string - path to an SVG file or raw SVG code
}}

\item{bg_color}{Background color of the card. Can be a solid color (e.g., "#fab255")
or a CSS gradient (e.g., "linear-gradient(to right, #1a5a3a, #2e7d32)" or
"linear-gradient(135deg, #667eea, #764ba2)").}

\item{width}{Card width in pixels}

\item{padding}{Padding inside the card}

\item{corner_radius}{Corner radius for rounded corners}

\item{font}{Font family}

\item{title_fontsize}{Title font size}

\item{title_color}{Color for the card title (default "white")}

\item{label_fontsize}{Label font size}

\item{value_fontsize}{Value font size}

\item{label_color}{Color for field labels (default "white")}

\item{value_bg_color}{Background color for value boxes}

\item{value_text_color}{Text color for values}

\item{show_house_icon}{Show house icon next to empreendimento}

\item{logos}{Character vector of logo file paths or SVG strings for top right.
Use \code{get_svg_path("filename.svg")} for bundled logos, or any local path.}

\item{logos_height}{Height for top-right logos (default 40)}

\item{bottom_logos}{Character vector of logo file paths or SVG strings for bottom left.}

\item{bottom_logos_height}{Height for bottom-left logos (default 30)}

\item{footer}{Footer text (e.g., update timestamp)}

\item{gap_to_footer}{Distance (px) between the last info block and the footer row.}

\item{footer_row_padding_bottom}{Bottom padding (px) under the footer row.}

\item{footer_fontsize}{Footer font size}

\item{footer_color}{Color for footer text (default "white")}

\item{uniform_row_height}{If TRUE, keep the height inside a row.}

\item{show_viewer}{If TRUE (and interactive), preview the SVG in the Viewer.}
}
\value{
SVG string
}
\description{
Generate a complete information card as an SVG with embedded styles,
fonts, badges, logos, and field labels/values.
}
\examples{
# With default house icon
fields <- list(
  list(
    list(label = "Empreendimento", value = "CAIARA II", with_icon = TRUE)
  )
)

# With custom icon
custom_icon <- '<svg width="50" height="50"><circle cx="25" cy="25" r="20" fill="white"/></svg>'
fields <- list(
  list(
    list(label = "Projeto", value = "Meu Projeto", with_icon = custom_icon)
  )
)

badges <- list(
  list(label = "UH", value = "192"),
  list(label = "Recurso Federal", value = "36,4 milhões")
)

# With file paths for logos
svg_card("FAR", badges, fields, 
         bg_color = "#fab255",
         logos = c("path/to/logo1.svg", "path/to/logo2.svg"),
         bottom_logos = c("path/to/gov_logo.svg"))

# With gradient background
svg_card("MCMV", badges, fields,
         bg_color = "linear-gradient(to right, #1a5a3a, #2e7d32)")

# Diagonal gradient
svg_card("Programa", badges, fields,
         bg_color = "linear-gradient(135deg, #667eea, #764ba2)")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xml_utils.R
\name{svg_xml_parse}
\alias{svg_xml_parse}
\title{Parse an SVG string into an xml2 document}
\usage{
svg_xml_parse(svg_content)
}
\arguments{
\item{svg_content}{SVG string (or object coercible to character).}
}
\value{
An \code{xml_document}, or \code{NULL} if \code{xml2} is unavailable or parsing
fails (including editor SVGs with undeclared namespace prefixes).
}
\description{
Parse an SVG string into an xml2 document
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xml_utils.R
\name{svg_dimension}
\alias{svg_dimension}
\title{Read a single root SVG dimension robustly}
\usage{
svg_dimension(
  svg_content,
  attr = c("width", "height"),
  prefer = c("attr", "viewBox")
)
}
\arguments{
\item{svg_content}{SVG string.}

\item{attr}{\code{"width"} or \code{"height"}.}

\item{prefer}{Which source to try first: \code{"attr"} (width/height attribute,
the default) or \code{"viewBox"}.}
}
\value{
Numeric value in CSS px, or \code{NA_real_} if not found.
}
\description{
Read a single root SVG dimension robustly
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chrome.R
\name{start_chrome_session}
\alias{start_chrome_session}
\title{Start a new Chrome session safely}
\usage{
start_chrome_session(old_session = NULL)
}
\arguments{
\item{old_session}{Previous session to close (can be NULL).}
}
\value{
New ChromoteSession object.
}
\description{
Creates a new ChromoteSession, optionally closing an existing one first.
Includes a small delay after closing to allow Chrome to clean up.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chrome.R
\name{get_chrome_download_info}
\alias{get_chrome_download_info}
\title{Get Chrome for Testing download info}
\usage{
get_chrome_download_info()
}
\description{
Get Chrome for Testing download info
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conversion.R
\name{batch_svg_to_png}
\alias{batch_svg_to_png}
\title{Batch convert multiple SVG cards to PNG}
\usage{
batch_svg_to_png(
  svg_list,
  output_dir = ".",
  prefix = "card",
  dpi = 300,
  background = "transparent"
)
}
\arguments{
\item{svg_list}{List of SVG strings.}

\item{output_dir}{Output directory.}

\item{prefix}{File name prefix.}

\item{dpi}{Resolution.}

\item{background}{Background color for PNG output.}
}
\value{
Character vector of output paths.
}
\description{
Convert a list of SVG strings to PNG files in a given directory.
Fonts are embedded automatically for consistent rendering.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{wrap_text}
\alias{wrap_text}
\title{Wrap text to fit within a given width}
\usage{
wrap_text(text, max_width, font = "Jost", fontsize = 12)
}
\arguments{
\item{text}{Text to wrap}

\item{max_width}{Maximum width in pixels}

\item{font}{Font name}

\item{fontsize}{Font size}
}
\value{
List with wrapped lines and total height
}
\description{
Wrap text to fit within a given width
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conversion.R
\name{save_svg}
\alias{save_svg}
\title{Save SVG string to file (sanitized + embedded fonts)}
\usage{
save_svg(svg_content, output_path)
}
\arguments{
\item{svg_content}{SVG string (or object coercible to character).}

\item{output_path}{Output file path.}
}
\value{
Path to the saved SVG file.
}
\description{
Saves an SVG string to disk. Before saving, the function:
\enumerate{
\item sanitizes the SVG to remove problematic Inkscape/Sodipodi metadata that can
break strict XML parsers, and
\item detects and embeds fonts (WOFF2 via @font-face) for deterministic rendering.
}

This function expects the font helpers to be available in the package:
\code{detect_svg_fonts()}, \code{ensure_cardargus_fonts()}, and \code{embed_svg_fonts()}.
}
\examples{
svg <- svg_card("FAR", list(), list())
save_svg(svg, tempfile(fileext = ".svg"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fonts.R
\name{install_fonts}
\alias{install_fonts}
\title{Pre-download fonts for offline use}
\usage{
install_fonts(
  fonts = c("Jost", "Montserrat", "Roboto", "Open Sans"),
  verbose = TRUE
)
}
\arguments{
\item{fonts}{Character vector of Google Font names to download.
Default is c("Jost", "Montserrat", "Roboto", "Open Sans").}

\item{verbose}{Print status messages}
}
\value{
Named logical vector indicating success for each font
}
\description{
Downloads and caches the specified fonts (or default fonts)
so they are available for PNG conversion without internet access.
}
\examples{
\donttest{
install_fonts()
install_fonts(c("Jost", "Roboto"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chrome.R
\name{get_chrome_platform}
\alias{get_chrome_platform}
\title{Get platform identifier for Chrome downloads}
\usage{
get_chrome_platform()
}
\description{
Get platform identifier for Chrome downloads
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chrome.R
\name{svg_to_png_chrome}
\alias{svg_to_png_chrome}
\title{Convert SVG to PNG using headless Chrome}
\usage{
svg_to_png_chrome(
  svg_input,
  output_path = NULL,
  dpi = 300,
  background = "transparent",
  load_wait = 0.5
)
}
\arguments{
\item{svg_input}{SVG string or path to an SVG file.}

\item{output_path}{Output path for the PNG file. If NULL, a temp file is used.}

\item{dpi}{Resolution in dots per inch (default 300). Chrome uses 96 DPI as base,
so dpi = 300 results in approximately 3.125x scaling.}

\item{background}{Background color for the HTML page (default "transparent").
Use "white", "#FFFFFF", etc. for a solid background.}

\item{load_wait}{Seconds to wait for page to load (default 0.5).
Increase if fonts are not rendering correctly.}
}
\value{
Path to the generated PNG file.
}
\description{
Renders an SVG to PNG using headless Chrome via the chromote package.
This method provides superior font rendering compared to librsvg/ImageMagick,
as Chrome properly handles @font-face rules, web fonts, and CSS features.
}
\examples{
svg <- svg_card("FAR", list(), list())
file_name <- tempfile(fileext = ".png")
\dontrun{
if (chrome_available()) {
  png_path <- svg_to_png_chrome(svg, file_name, dpi = 300)
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{generate_id}
\alias{generate_id}
\title{Generate unique ID suffix}
\usage{
generate_id(...)
}
\arguments{
\item{...}{Values to hash}
}
\value{
Character string with unique ID
}
\description{
Produces a short, collision-resistant suffix derived from the supplied
content plus a session-local counter. Using a counter (instead of
\code{\link[stats:runif]{stats::runif()}}) keeps output deterministic and reproducible within a
session while remaining unique across calls.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xml_utils.R
\name{svg_font_families}
\alias{svg_font_families}
\title{Detect font families referenced by an SVG}
\usage{
svg_font_families(svg_content)
}
\arguments{
\item{svg_content}{SVG string.}
}
\value{
Unique character vector of font family names (generics removed).
}
\description{
Detect font families referenced by an SVG
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logos.R
\name{icon_house}
\alias{icon_house}
\title{House Icon SVG}
\usage{
icon_house(
  width = 50,
  height = 56,
  stroke_color = "white",
  stroke_width = 35,
  fill = "none"
)
}
\arguments{
\item{width}{Width of the icon}

\item{height}{Height of the icon}

\item{stroke_color}{Stroke color}

\item{stroke_width}{Stroke width}

\item{fill}{Fill color (default none)}
}
\value{
SVG string
}
\description{
Generate a house/home icon SVG. You can also use any SVG file path
instead of built-in icons.
}
\examples{
icon_house(50, 56)

# You can also use a custom SVG file:
# svg_card(..., with_icon = "/path/to/my_icon.svg")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xml_utils.R
\name{parse_viewbox}
\alias{parse_viewbox}
\title{Parse a viewBox string into width/height}
\usage{
parse_viewbox(vb)
}
\arguments{
\item{vb}{A viewBox string \code{"minx miny width height"} or \code{NA}.}
}
\value{
Numeric vector \code{c(width, height)}, possibly \code{c(NA, NA)}.
}
\description{
Parse a viewBox string into width/height
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/knitr.R
\name{include_card}
\alias{include_card}
\title{Display card in knitr/Quarto document (SVG via data URI)}
\usage{
include_card(
  svg_string,
  width = "100\%",
  alt = "Card generated by cardargus",
  dpi = 300,
  engine = c("auto", "chrome", "rsvg")
)
}
\arguments{
\item{svg_string}{SVG string from \code{svg_card()}.}

\item{width}{Display width (CSS units), e.g. \code{"100\%"}, \code{"500px"}.}

\item{alt}{Alternative text for accessibility.}

\item{dpi}{Fallback DPI used when output is not HTML (default 300).}

\item{engine}{Rendering engine for non-HTML output: \code{"auto"}, \code{"chrome"}, or \code{"rsvg"}.}
}
\value{
Knitr output for the current format.
}
\description{
Embeds the SVG as an \verb{<img>} using a \verb{data:image/svg+xml;base64,...} URI.
This is more robust than inline \verb{<svg>} for Pandoc/pkgdown (avoids "unclosed div"
warnings). For non-HTML outputs, it falls back to \code{include_card_png()}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chrome.R
\name{ensure_chrome}
\alias{ensure_chrome}
\title{Ensure Chrome is available, downloading if necessary}
\usage{
ensure_chrome(download = FALSE, verbose = TRUE)
}
\arguments{
\item{download}{If TRUE and Chrome is not found, attempt to download
Chrome for Testing (default FALSE).}

\item{verbose}{Print status messages (default TRUE).}
}
\value{
TRUE if Chrome is available (or was successfully downloaded),
FALSE otherwise.
}
\description{
Checks if Chrome is available and optionally downloads a standalone
Chrome for Testing if not found. This ensures Chrome-based rendering
works without requiring a system-wide Chrome installation.
}
\details{
When \code{download = TRUE}, this function will download "Chrome for Testing",
a standalone Chrome distribution designed for automation. The download
is approximately 150MB and is cached in the user's data directory.

Alternatively, you can:
\itemize{
\item Install Chrome/Chromium system-wide
\item Set the \code{CHROMOTE_CHROME} environment variable to point to an existing installation
}
}
\examples{
# Check and report status
ensure_chrome()

# Download Chrome if not available
\dontrun{
ensure_chrome(download = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conversion.R
\name{write_svg_tempfile}
\alias{write_svg_tempfile}
\title{Write SVG content to a temporary .svg file}
\usage{
write_svg_tempfile(svg_content)
}
\arguments{
\item{svg_content}{Character SVG content.}
}
\value{
Path to a temporary SVG file.
}
\description{
Write SVG content to a temporary .svg file
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logos.R
\name{create_logo_row}
\alias{create_logo_row}
\title{Create logo row for top-right corner of card}
\usage{
create_logo_row(
  logos,
  target_height = 40,
  spacing = 10,
  card_width = 500,
  x_offset = 20,
  y_offset = 20
)
}
\arguments{
\item{logos}{List of SVG strings or file paths}

\item{target_height}{Height for all logos (default 40)}

\item{spacing}{Horizontal spacing between logos (default 10)}

\item{card_width}{Total card width to calculate positioning}

\item{x_offset}{Right margin from card edge}

\item{y_offset}{Top margin}
}
\value{
A list with svg_content and total_width
}
\description{
Takes a list of SVG logos and arranges them horizontally with proper spacing.
Returns the SVG elements positioned for the top-right corner.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/badges.R
\name{css_gradient_to_svg}
\alias{css_gradient_to_svg}
\title{Convert CSS gradient to SVG gradient}
\usage{
css_gradient_to_svg(css_gradient, id = "grad")
}
\arguments{
\item{css_gradient}{CSS gradient string}

\item{id}{ID for the gradient}
}
\value{
SVG gradient definition
}
\description{
Convert CSS gradient to SVG gradient
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fonts.R
\name{svgs_dir}
\alias{svgs_dir}
\title{Get the path to package SVGs directory}
\usage{
svgs_dir()
}
\value{
Character string with the path to SVGs directory
}
\description{
Returns the path to the inst/svgs directory where SVG files are stored.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fonts.R
\name{download_google_font_woff2}
\alias{download_google_font_woff2}
\title{Download a Google Font as WOFF2 to the cache}
\usage{
download_google_font_woff2(family, weight = "400")
}
\arguments{
\item{family}{Font family name.}

\item{weight}{Numeric/character weight (default "400").}
}
\value{
Path to cached WOFF2 file or NA_character_.
}
\description{
Download a Google Font as WOFF2 to the cache
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fonts.R
\name{font_cache_dir}
\alias{font_cache_dir}
\title{Get font cache directory}
\usage{
font_cache_dir(persistent = TRUE)
}
\arguments{
\item{persistent}{Logical. If TRUE (default), uses persistent cache via
\code{tools::R_user_dir()}. If FALSE or if persistent cache is unavailable,
uses a session-specific temporary directory.}
}
\value{
A character path to the cache directory.
}
\description{
Returns the directory where cardargus caches downloaded font files.
Fonts in this directory are automatically embedded in SVG/PNG exports.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/badges.R
\name{create_badge}
\alias{create_badge}
\title{Create an SVG Badge}
\usage{
create_badge(
  label,
  value,
  color,
  font = "Jost",
  style = "margin:2px;",
  fontsize = 11,
  horiz_padding = 5,
  extra_right_pad = 2,
  class = "",
  shadow_offset = 2,
  corner_radius = 3,
  height = NULL,
  as_string = TRUE
)
}
\arguments{
\item{label}{Label text (left side)}

\item{value}{Value text (right side)}

\item{color}{Background color for value area}

\item{font}{Font family name}

\item{style}{CSS style string}

\item{fontsize}{Font size in pixels}

\item{horiz_padding}{Horizontal padding}

\item{extra_right_pad}{Extra padding on right side}

\item{class}{CSS class}

\item{shadow_offset}{Shadow offset in pixels}

\item{corner_radius}{Corner radius for rounded rectangle}

\item{height}{Minimum height (optional)}

\item{as_string}{Return as character string}
}
\value{
SVG string
}
\description{
Generate a badge similar to shields.io style with label and value.
Both sides have properly rounded corners.
}
\examples{
create_badge("UH", "192", "white")
create_badge("Recurso Federal", "36,4 milhões", "#4CAF50")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{text_height}
\alias{text_height}
\title{Calculate text height using gdtools}
\usage{
text_height(text, font = "Jost", fontsize = 12)
}
\arguments{
\item{text}{Text to measure}

\item{font}{Font name}

\item{fontsize}{Font size in pixels}
}
\value{
Numeric height in pixels
}
\description{
Calculate text height using gdtools
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fonts.R
\name{register_font}
\alias{register_font}
\title{Register a local font file for embedding}
\usage{
register_font(font_path, family = NULL)
}
\arguments{
\item{font_path}{Path to a local .ttf or .woff2 font file}

\item{family}{Font family name to register (e.g., "Jost"). If NULL,
the filename without extension is used.}
}
\value{
Path to the cached font file (invisible)
}
\description{
Copies a local font file (TTF or WOFF2) to the cardargus cache directory
so it can be embedded in SVG exports.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{escape_xml}
\alias{escape_xml}
\title{Escape XML special characters}
\usage{
escape_xml(text)
}
\arguments{
\item{text}{Text to escape}
}
\value{
Escaped text
}
\description{
Escape XML special characters
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fonts.R
\name{prepare_svg_for_raster}
\alias{prepare_svg_for_raster}
\title{Prepare SVG for rasterization (sanitize + embed fonts)}
\usage{
prepare_svg_for_raster(svg_content)
}
\arguments{
\item{svg_content}{Character SVG.}
}
\value{
A sanitized SVG with embedded WOFF2 fonts.
}
\description{
Removes Google Fonts @import rules (not supported by librsvg), sanitizes
editor metadata (handled by sanitize_svg_for_raster()), then embeds fonts
referenced by the SVG into @font-face blocks.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fonts.R
\name{font_cache_path}
\alias{font_cache_path}
\title{Build a cache filename for a font family}
\usage{
font_cache_path(family)
}
\arguments{
\item{family}{Font family name.}
}
\value{
Path to the cached font file.
}
\description{
Build a cache filename for a font family
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xml_utils.R
\name{svg_dims}
\alias{svg_dims}
\title{Read SVG width and height robustly}
\usage{
svg_dims(svg_content, prefer = c("attr", "viewBox"))
}
\arguments{
\item{svg_content}{SVG string.}

\item{prefer}{Which source to try first per dimension (see \code{\link[=svg_dimension]{svg_dimension()}}).}
}
\value{
A list with numeric \code{width} and \code{height} (either may be NA).
}
\description{
Read SVG width and height robustly
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chrome.R
\name{svg_to_pdf_chrome}
\alias{svg_to_pdf_chrome}
\title{Convert SVG to PDF using headless Chrome}
\usage{
svg_to_pdf_chrome(
  svg_input,
  output_path,
  background = "transparent",
  print_background = TRUE,
  load_wait = 0.5
)
}
\arguments{
\item{svg_input}{SVG string or path to an SVG file.}

\item{output_path}{Output path for the PDF file.}

\item{background}{Background color for the HTML page (default "transparent").}

\item{print_background}{Whether to include CSS backgrounds in PDF (default TRUE).}

\item{load_wait}{Seconds to wait for page to load (default 0.5).}
}
\value{
Path to the generated PDF file.
}
\description{
Renders an SVG to PDF using headless Chrome via the chromote package.
This method produces vector PDFs with perfect font rendering.
}
\examples{
\dontrun{
svg <- svg_card("FAR", list(), list())
if (chrome_available()) {
  pdf_path <- svg_to_pdf_chrome(svg, tempfile(fileext = ".pdf"))
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fonts.R
\name{register_google_font}
\alias{register_google_font}
\title{Register Google Font (sysfonts)}
\usage{
register_google_font(font_family)
}
\arguments{
\item{font_family}{Font family name (e.g., "Jost")}
}
\value{
Invisible NULL
}
\description{
Registers a Google Font using \pkg{sysfonts}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conversion.R
\name{sanitize_svg_for_raster}
\alias{sanitize_svg_for_raster}
\title{Sanitize SVG for rasterization engines (librsvg / ImageMagick)}
\usage{
sanitize_svg_for_raster(svg_input)
}
\arguments{
\item{svg_input}{An SVG string or an object coercible to character
(e.g. \code{htmltools::HTML}).}
}
\value{
A character string containing a sanitized SVG.
}
\description{
SVGs exported by editors such as Inkscape may contain \verb{sodipodi:*} and/or
\verb{inkscape:*} nodes/attributes. If these namespace prefixes appear without the
corresponding \verb{xmlns:*} declarations, strict parsers (notably \code{librsvg}) can
fail with errors like:
\preformatted{Namespace prefix sodipodi on namedview is not defined}

This helper removes non-rendering metadata blocks and attributes commonly
responsible for parse failures, producing a more interoperable SVG string.
It also removes Google Fonts @import rules which are not supported by
rasterization engines.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fonts.R
\name{setup_fonts}
\alias{setup_fonts}
\title{Setup showtext for cardargus}
\usage{
setup_fonts(fonts = c("Jost", "Montserrat"), auto = TRUE)
}
\arguments{
\item{fonts}{Character vector of Google Font names to register.}

\item{auto}{Enable showtext auto mode.}
}
\value{
Invisible NULL
}
\description{
Registers fonts (via sysfonts) and optionally enables showtext auto mode.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_font_css}
\alias{get_font_css}
\title{Get Google Font CSS for embedding in SVG}
\usage{
get_font_css(font_name = "Jost", weights = c(400, 500, 600, 700))
}
\arguments{
\item{font_name}{Name of the Google Font}

\item{weights}{Vector of font weights to include}
}
\value{
Character string with CSS @font-face rules
}
\description{
Get Google Font CSS for embedding in SVG
}
\examples{
get_font_css("Jost")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/knitr.R
\name{save_card_for_knitr}
\alias{save_card_for_knitr}
\title{Save card and return path for knitr}
\usage{
save_card_for_knitr(
  svg_string,
  filename = "card",
  format = c("svg", "png"),
  dpi = 300,
  dir = NULL,
  engine = c("auto", "chrome", "rsvg")
)
}
\arguments{
\item{svg_string}{SVG string from \code{svg_card()}.}

\item{filename}{Output filename (without extension).}

\item{format}{Output format: \code{"svg"} or \code{"png"}.}

\item{dpi}{Rasterization DPI for PNG (default 300).}

\item{dir}{Output directory (defaults to knitr figure directory or tempdir()).}

\item{engine}{Rendering engine for PNG: \code{"auto"}, \code{"chrome"}, or \code{"rsvg"}.}
}
\value{
Path to the saved file.
}
\description{
Saves a card to a file and returns the path for use in knitr chunks.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\docType{package}
\name{cardargus-package}
\alias{cardargus}
\alias{cardargus-package}
\title{cardargus: Generate SVG Information Cards with Embedded Fonts and Badges}
\description{
\if{html}{\figure{logo.svg}{options: style='float: right' alt='logo' width='120'}}

Create self-contained SVG information cards with embedded 'Google Fonts', shields-style badges, and custom logos. Cards are fully portable SVG files ideal for dashboards, reports, and web applications. Includes functions to export cards to PNG format and display them in 'R Markdown' and 'Quarto' documents.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://strategicprojects.github.io/cardargus/}
  \item \url{https://github.com/StrategicProjects/cardargus}
  \item Report bugs at \url{https://github.com/StrategicProjects/cardargus/issues}
}

}
\author{
\strong{Maintainer}: Andre Leite \email{leite@castlab.org}

Authors:
\itemize{
  \item Andre Leite \email{leite@castlab.org}
  \item Hugo Vasconcelos \email{hugo.vasconcelos@ufpe.br}
  \item Diogo Bezerra \email{diogo.bezerra@ufpe.br}
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xml_utils.R
\name{svg_geometry_regex}
\alias{svg_geometry_regex}
\title{Regex fallback for \code{\link[=svg_geometry]{svg_geometry()}}}
\usage{
svg_geometry_regex(svg_content)
}
\arguments{
\item{svg_content}{SVG string.}
}
\description{
Regex fallback for \code{\link[=svg_geometry]{svg_geometry()}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chrome.R
\name{convert_svg_with_session}
\alias{convert_svg_with_session}
\title{Convert a single SVG using an existing Chrome session}
\usage{
convert_svg_with_session(b, svg_content, scale, background, load_wait = 0.5)
}
\arguments{
\item{b}{ChromoteSession object.}

\item{svg_content}{SVG content string (already sanitized).}

\item{scale}{DPI scale factor.}

\item{background}{Background color.}

\item{load_wait}{Seconds to wait for page load (default 0.5).}
}
\value{
Base64 encoded PNG string.
}
\description{
Internal helper that renders an SVG to PNG using an existing Chrome session.
Uses fixed-time waiting instead of event-based waiting for reliability.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fonts.R
\name{embed_svg_fonts}
\alias{embed_svg_fonts}
\title{Embed a font file into an SVG via @font-face}
\usage{
embed_svg_fonts(svg_content, font_family, woff2_path)
}
\arguments{
\item{svg_content}{Character SVG.}

\item{font_family}{Font family name to embed.}

\item{woff2_path}{Path to a font file (.woff2, .ttf, .woff, or .otf).}
}
\value{
SVG string with embedded @font-face rule.
}
\description{
Embed a font file into an SVG via @font-face
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conversion.R
\name{parse_svg_root_dim}
\alias{parse_svg_root_dim}
\title{Parse a numeric width/height from the root <svg ...> tag}
\usage{
parse_svg_root_dim(svg_content, attr = c("width", "height"))
}
\arguments{
\item{svg_content}{Character SVG.}

\item{attr}{"width" or "height".}
}
\value{
Numeric value (in CSS px) or NA_real_ if not found.
}
\description{
Tries:
\enumerate{
\item width="..." / height="..." (supports px, numbers)
\item viewBox="minx miny width height" as fallback
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conversion.R
\name{svg_to_png_with_margin}
\alias{svg_to_png_with_margin}
\title{Convert SVG to PNG with optional margin and background}
\usage{
svg_to_png_with_margin(
  svg_input,
  output_path = NULL,
  margin = 20,
  margin_color = "transparent",
  dpi = 300,
  background = "transparent"
)
}
\arguments{
\item{svg_input}{SVG string or path to SVG file.}

\item{output_path}{Output path for PNG file (optional; a temp file is used if NULL).}

\item{margin}{Margin in pixels to add around the card.}

\item{margin_color}{Color of the margin area (default transparent).}

\item{dpi}{Resolution in dots per inch.}

\item{background}{Background color for the card rasterization (default transparent).}
}
\value{
Path to the generated PNG file.
}
\description{
Creates a PNG with extra margin around the card.
Fonts are embedded before rasterization for consistent appearance.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logos.R
\name{custom_logo_svg}
\alias{custom_logo_svg}
\title{Generic logo placeholder SVG}
\usage{
custom_logo_svg(svg_content, width = 40, height = 40)
}
\arguments{
\item{svg_content}{Raw SVG content or path to SVG file}

\item{width}{Desired width}

\item{height}{Desired height}
}
\value{
SVG string
}
\description{
\strong{Deprecated.} Use \code{\link[=load_svg_for_embed]{load_svg_for_embed()}} instead,
or simply pass the SVG file path directly to \code{svg_card()}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conversion.R
\name{svg_to_png}
\alias{svg_to_png}
\title{Convert SVG to PNG}
\usage{
svg_to_png(
  svg_input,
  output_path = NULL,
  width = NULL,
  height = NULL,
  dpi = 300,
  background = "transparent"
)
}
\arguments{
\item{svg_input}{SVG string or path to an SVG file.}

\item{output_path}{Output path for the PNG file (optional; a temp file is used if NULL).}

\item{width}{Output width in pixels (NULL to infer from SVG and scale by DPI).}

\item{height}{Output height in pixels (NULL to infer from SVG and scale by DPI).}

\item{dpi}{Resolution in dots per inch (default 300 for high quality).}

\item{background}{Background color. Use "transparent" or "none" for transparency
(default), or specify a color like "white", "#FFFFFF", etc.}
}
\value{
Path to the generated PNG file.
}
\description{
Convert an SVG string or SVG file path to a high-quality PNG image.
The function sanitizes the SVG and embeds required WOFF2 fonts (downloaded
on demand into a user cache) to ensure consistent font rendering.

\strong{Important note about DPI}: \code{rsvg} rasterizes primarily based on pixel
dimensions. To make DPI matter, this function scales output pixel size by
\code{(dpi / 96)} when \code{width} / \code{height} are not explicitly provided.
}
\examples{
svg <- svg_card("FAR", list(), list())
file_name <- tempfile(fileext = ".png")
png_path <- svg_to_png(svg, file_name, dpi = 300)
png_path <- svg_to_png(svg, file_name, dpi = 300, background = "white")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chrome.R
\name{cleanup_chromote_session}
\alias{cleanup_chromote_session}
\title{Safely close a Chromote session}
\usage{
cleanup_chromote_session(session, timeout_before = 2, timeout_after = 1)
}
\arguments{
\item{session}{A ChromoteSession object to close.}

\item{timeout_before}{Seconds to wait for pending promises before closing (default 2).}

\item{timeout_after}{Seconds to wait for cleanup after closing (default 1).}
}
\value{
NULL (invisibly). Called for side effects.
}
\description{
Closes a chromote session handling pending promises to avoid
"Unhandled promise error: timed out waiting for response" warnings.
This function processes pending async operations before closing
and silently ignores any timeout errors during cleanup.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xml_utils.R
\name{svg_geometry}
\alias{svg_geometry}
\title{Read SVG root geometry (attribute and viewBox dimensions)}
\usage{
svg_geometry(svg_content)
}
\arguments{
\item{svg_content}{SVG string.}
}
\value{
A list with numeric \code{attr_w}, \code{attr_h}, \code{vb_w}, \code{vb_h} (any may be NA).
}
\description{
Read SVG root geometry (attribute and viewBox dimensions)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chrome.R
\name{sanitize_svg_metadata}
\alias{sanitize_svg_metadata}
\title{Sanitize SVG metadata for Chrome rendering}
\usage{
sanitize_svg_metadata(svg_content)
}
\arguments{
\item{svg_content}{SVG string.}
}
\value{
Sanitized SVG string.
}
\description{
Removes problematic Inkscape/Sodipodi metadata that can cause issues,
but preserves @import rules since Chrome handles them correctly.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conversion.R
\name{svg_to_formats}
\alias{svg_to_formats}
\title{Convert SVG to multiple formats}
\usage{
svg_to_formats(
  svg_input,
  output_base,
  formats = c("svg", "png"),
  dpi = 300,
  background = "transparent"
)
}
\arguments{
\item{svg_input}{SVG string or file path.}

\item{output_base}{Base name for output files (without extension).}

\item{formats}{Vector of formats to generate ("png", "svg", "pdf").}

\item{dpi}{Resolution for raster formats.}

\item{background}{Background color for PNG output.}
}
\value{
Named list with paths to generated files.
}
\description{
Convert an SVG string or file to multiple formats. Supported formats are:
\itemize{
\item \code{"svg"} - saves the SVG
\item \code{"png"} - rasterizes to PNG via \code{\link{svg_to_png}}
\item \code{"pdf"} - converts to PDF (prefers \pkg{rsvg})
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xml_utils.R
\name{normalize_font_families}
\alias{normalize_font_families}
\title{Normalize a vector of raw font-family values}
\usage{
normalize_font_families(families)
}
\arguments{
\item{families}{Character vector of raw font-family values.}
}
\description{
Takes the first family of each declaration, strips quotes/whitespace, dedups,
and drops CSS generic family names.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{compress_number}
\alias{compress_number}
\title{Compress number to abbreviated format}
\usage{
compress_number(x, digits = 1)
}
\arguments{
\item{x}{Numeric value to compress}

\item{digits}{Number of decimal places}
}
\value{
Character string with abbreviated number
}
\description{
Compress number to abbreviated format
}
\examples{
compress_number(1234567)
compress_number(36400000)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chrome.R
\name{write_svg_html_temp}
\alias{write_svg_html_temp}
\title{Create a temporary HTML file wrapping an SVG}
\usage{
write_svg_html_temp(
  svg_string,
  width_px = NULL,
  height_px = NULL,
  background = "transparent"
)
}
\arguments{
\item{svg_string}{SVG content as character string.}

\item{width_px}{Width in pixels. If NULL, extracted from SVG.}

\item{height_px}{Height in pixels. If NULL, extracted from SVG.}

\item{background}{Background color (default "transparent").}
}
\value{
A list with path, width, and height.
}
\description{
Create a temporary HTML file wrapping an SVG
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/knitr.R
\name{include_card_png}
\alias{include_card_png}
\title{Display card as PNG in knitr/Quarto document}
\usage{
include_card_png(
  svg_string,
  dpi = 300,
  width = "100\%",
  alt = "Card generated by cardargus",
  background = "transparent",
  engine = c("auto", "chrome", "rsvg")
)
}
\arguments{
\item{svg_string}{SVG string from \code{svg_card()}.}

\item{dpi}{Rasterization DPI (default 300).}

\item{width}{Display width (CSS units) for HTML outputs.}

\item{alt}{Alternative text for accessibility.}

\item{background}{Background color passed to \code{svg_to_png()} (default transparent).}

\item{engine}{Rendering engine: \code{"auto"} (uses Chrome if available, else rsvg),
\code{"chrome"} (headless Chrome via chromote), or \code{"rsvg"} (librsvg/magick).
Chrome provides better font rendering for Google Fonts.}
}
\value{
Knitr output for the current format.
}
\description{
Converts an SVG card to PNG and displays it in R Markdown or Quarto documents.
Recommended for better compatibility across HTML/PDF/Word.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fonts.R
\name{ensure_cardargus_fonts}
\alias{ensure_cardargus_fonts}
\title{Ensure cardargus fonts are available (WOFF2 cache)}
\usage{
ensure_cardargus_fonts(families, weight = "400")
}
\arguments{
\item{families}{Character vector of font families.}

\item{weight}{Weight to request when downloading (default "400").}
}
\value{
Named character vector family -> font file path (NA if unavailable).
}
\description{
Ensure cardargus fonts are available (WOFF2 cache)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/badges.R
\name{create_badge_row}
\alias{create_badge_row}
\title{Create a row of SVG badges with uniform height}
\usage{
create_badge_row(
  badges_data,
  default_color = "white",
  spacing = 4,
  font = "Jost",
  fontsize = 10,
  uniform_height = TRUE
)
}
\arguments{
\item{badges_data}{A list of lists, each containing label, value, and optionally color}

\item{default_color}{Default color for badges}

\item{spacing}{Spacing between badges}

\item{font}{Font family}

\item{fontsize}{Font size}

\item{uniform_height}{Force all badges to have the same height (default TRUE)}
}
\value{
SVG string containing all badges
}
\description{
Generate multiple badges arranged horizontally with the same height.
}
\examples{
badges <- list(
  list(label = "UH", value = "192"),
  list(label = "Recurso Federal", value = "36,4 milhões"),
  list(label = "Contrapartida", value = "0,0")
)
create_badge_row(badges, default_color = "white")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chrome.R
\name{batch_svg_to_base64_chrome}
\alias{batch_svg_to_base64_chrome}
\title{Batch convert SVGs to PNG base64 using headless Chrome}
\usage{
batch_svg_to_base64_chrome(
  svg_list,
  dpi = 300,
  background = "transparent",
  load_wait = 0.5,
  restart_every = 50,
  retry_attempts = 3,
  progress = TRUE
)
}
\arguments{
\item{svg_list}{List of SVG strings or file paths.}

\item{dpi}{Resolution (default 300).}

\item{background}{Background color (default "transparent").}

\item{load_wait}{Seconds to wait for each page to load (default 0.5).
Increase if conversions are failing.}

\item{restart_every}{Restart Chrome session every N conversions (default 50).
Helps prevent memory issues and stale connections.}

\item{retry_attempts}{Number of retry attempts on failure (default 3).}

\item{progress}{Show progress bar (default TRUE).}
}
\value{
Character vector of base64-encoded PNGs (data URI format).
Returns NA for failed conversions.
}
\description{
Converts multiple SVGs to base64-encoded PNG strings using a single
Chrome session. Much faster than calling svg_to_png_chrome() repeatedly.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/knitr.R
\name{card_to_grob}
\alias{card_to_grob}
\title{Create a grob for grid/ggplot2}
\usage{
card_to_grob(svg_string, dpi = 150, engine = c("auto", "chrome", "rsvg"))
}
\arguments{
\item{svg_string}{SVG string from \code{svg_card()}.}

\item{dpi}{Rasterization DPI (default 150).}

\item{engine}{Rendering engine: \code{"auto"}, \code{"chrome"}, or \code{"rsvg"}.}
}
\value{
A \code{grid::rasterGrob} object.
}
\description{
Wrap an SVG card as a raster grob so it can be used with grid graphics.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/knitr.R
\name{register_cardargus_knitr}
\alias{register_cardargus_knitr}
\title{Register cardargus knitr engine}
\usage{
register_cardargus_knitr()
}
\value{
Invisible NULL.
}
\description{
Registers a custom knitr engine named \code{"cardargus"} so that SVG cards can be
rendered directly from chunks.
}
\examples{
# In your setup chunk:
register_cardargus_knitr()

# Then use cardargus as chunk engine:
# ```{cardargus}
# svg_card(title = "My Card", ...)
# ```
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chrome.R
\name{chrome_available}
\alias{chrome_available}
\title{Check if Chrome/Chromium is available for rendering}
\usage{
chrome_available(verbose = FALSE)
}
\arguments{
\item{verbose}{Print status messages (default FALSE).}
}
\value{
TRUE if Chrome is available, FALSE otherwise.
}
\description{
Checks whether the chromote package can find and use a Chrome or Chromium
installation for headless rendering.
}
\examples{
if (chrome_available()) {
   cat("Using Chrome")
} else {
   cat("Using Magick")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xml_utils.R
\name{svg_font_families_regex}
\alias{svg_font_families_regex}
\title{Regex fallback for \code{\link[=svg_font_families]{svg_font_families()}}}
\usage{
svg_font_families_regex(svg_content)
}
\arguments{
\item{svg_content}{SVG string.}
}
\description{
Regex fallback for \code{\link[=svg_font_families]{svg_font_families()}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conversion.R
\name{ensure_output_dir}
\alias{ensure_output_dir}
\title{Ensure output directory exists}
\usage{
ensure_output_dir(path)
}
\arguments{
\item{path}{Output file path.}
}
\description{
Ensure output directory exists
}
\keyword{internal}
